#
# See the file "LICENSE" for the full license governing this code.
#
#
# This function checks if the specfied patches are installed.
# For type = db (default) the function will return a hash with all homes and their patch details
#   that needs at least one patch installed.
# For type = grid the function will return a hash with only patches that haven't been installed yet.
#   There is a difference because of the way patches are applied for grid, opatchauto vs regular provider.
#
# It will check all registered Oracle homes for the installed patches.
#
Puppet::Functions.create_function(:'ora_install::ora_patches_missing') do
  dispatch :check_patch_list do
    param 'Hash', :patch_list
    optional_param "Enum['db','grid']", :type
    return_type 'Hash'
  end

  def check_patch_list(patch_list, type='db')
    scope = closure_scope
    oracle_homes = scope['ora_install_homes'] || {}

    #
    # When the ora_install_homes fact is empty, no patches are yet installed so all specified 
    # patches are missing
    #
    return patch_list if oracle_homes.empty?

    #
    # Validate that all homes contain a valid Opatch
    #
    oracle_homes['opatch_version'].each do |key, value|
      if value == 'Invalid'
        fail "Either ORACLE_HOME '#{key}' or '#{key}/OPatch/opatch' not found. Faulty oraInventory entry? Please fix!"
      end
    end

    #
    # Convert a patch list with subpatches to a full list
    #
    converted_patch_list = call_function('ora_install::ora_physical_patches', patch_list)
    Puppet.debug("converted patch list: #{converted_patch_list}")

    # Convertv the existing patches to the home:patch_id format
    existing_patches = oracle_homes['installed_patches'].nil? ? [] : oracle_homes['installed_patches'].collect { |key, val| val.map { |patch| "#{key}:#{patch}" } }.flatten
    Puppet.debug("existing patches: #{existing_patches}")

    #
    # Now determine the missing patches
    #
    missing_patches = converted_patch_list.select do | patch |
      if patch =~ /remove$/
        if existing_patches.include?(patch.chomp(':remove'))
          true
        else
          false
        end
      else
        !existing_patches.include?(patch)
      end
    end
    Puppet.debug("missing patches: #{missing_patches}")

    # If no patches missing, return empty hash
    return {} if missing_patches.empty?

    homes_to_be_patched = Facter.value(:kernel) == 'windows' ? missing_patches.map { |patch| "#{patch.split(':')[0]}:#{patch.split(':')[1]}" }.uniq : missing_patches.map { |patch| patch.split(':')[0] }.uniq
    if type == 'grid'
      missing_patch_numbers = missing_patches.collect { |element| element.split(':')[1] }
      patch_list_out = patch_list.select { |patch, _patch_info| (missing_patch_numbers & patch_list[patch]['sub_patches']).any? }
    else
      patch_list_out = Facter.value(:kernel) == 'windows' ? patch_list.select { |patch, _patch_info| homes_to_be_patched.include?("#{patch.split(':')[0]}:#{patch.split(':')[1]}") } : patch_list.select { |patch, _patch_info| homes_to_be_patched.include?(patch.split(':')[0]) }
    end
    Puppet.debug("patch_list_out: #{patch_list_out}")
    return patch_list_out
  end
end
