# frozen_string_literal: true

#
# See the file "LICENSE" for the full license governing this code.
#
require 'fileutils'
require 'easy_type'
# rubocop:disable Metrics
Puppet::Type.type(:db_directory_structure).provide(:db_directory_structure) do
  def configure
    name            = resource[:name]
    create_orainst  = resource[:create_orainst_only]
    ora_inventory   = resource[:ora_inventory_dir]
    user            = resource[:os_user]
    group           = resource[:os_group]
    oracle_base     = resource[:oracle_base_dir]
    download_folder = resource[:download_dir]

    if create_orainst
      Puppet.info "creating #{ora_inst_file}"
      ora_inst_content = "inventory_loc=#{ora_inventory}\ninst_group=#{group}\n"
      File.open(ora_inst_file, 'w') { |f| f.write ora_inst_content }
      FileUtils.chmod 0664, ora_inst_file
      FileUtils.chown user, group, ora_inst_file
    else
      ensure_directory ora_inventory do
        owned_by_oracle ora_inventory, user, group
      end
      if Puppet::Util::Platform.windows?
        ensure_directory download_folder
      else
        ensure_directory oracle_base
        # Always change the user and group of the oracle base
        owned_by_oracle oracle_base, user, group
        ensure_directory download_folder do
          allow_everybody download_folder
        end
      end
    end
  end

  private

  def create_orainst_file
    Puppet.info "creating #{ora_inst_file}"
    ora_inst_content = "inventory_loc=#{ora_inventory}\ninst_group=#{group}\n"
    File.open(ora_inst_file, 'w') { |f| f.write ora_inst_content }
    FileUtils.chmod 0664, ora_inst_file
    FileUtils.chown user, group, ora_inst_file
  end

  def ensure_directory(path)
    if Dir.exist?(path)
      Puppet.info "Directory #{path} already exists"
    else
      Puppet.info "Creating directory #{path}"
      FileUtils.mkdir_p path
      yield if block_given?
    end
  end

  def owned_by_oracle(path, user, group)
    Puppet.info "Setting oracle ownership for #{path} with 0775"
    FileUtils.chmod 0775, path
    FileUtils.chown user, group, path
  end

  def allow_everybody(path)
    Puppet.info "Setting public permissions 0777 for #{path}"
    FileUtils.chmod 0777, path
  end

  def ora_inst_file
    case Facter.value(:kernel)
    when 'Linux' then '/etc/oraInst.loc'
    when 'SunOS' then '/var/opt/oracle/oraInst.loc'
    when 'windows' then 'C:\\Program Files\\Oracle\\Inventory\\oraInst.loc'
    else fail 'unsupported OS'
    end
  end
end
# rubocop:enable Metrics
