#
# See the file "LICENSE" for the full license governing this code.
#
require 'easy_type'
require 'puppet_x/enterprisemodules/ora_install/get_homes'
require 'puppet_x/enterprisemodules/ora_install/commands'

Puppet::Type.type(:ora_opatch).provide(:base) do
  include EasyType::Provider
  extend ::PuppetX::EnterpriseModules::OraInstall::GetHomes
  include PuppetX::EnterpriseModules::OraInstall::Commands

  mk_resource_methods

  def os_user
    @os_user ||= self.class.user_for_file(oracle_product_home_dir)
  end

  desc 'Base provider for Oracle patches'

  mk_resource_methods

  confine :true => false # This is NEVER a valid provider. It is just used as a base class

  def self.prefetch(requested_patches)
    if use_fact?(requested_patches)
      orainst_dir = ora_invdir
      installed_patches = oracle_homes.collect { |h| patches_in_home(h, orainst_dir).collect { |p| p[:name] } }.flatten
    else
      orainst_dir = check_single_orainst_dir(requested_patches)
      installed_patches = oracle_homes(requested_patches).collect { |h| patches_in_home(h, orainst_dir, requested_patches).collect { |p| p[:name] } }.flatten
    end
    #
    # With the `remove_patches_before` option you can remove a set of patches before you install
    # the current one. However, sometimes you need to reinstall the same patch again. But only after an other
    # patch is (re)installed.
    # 
    # Puppet then at the begin of the run detects the patch is already installed and only reinstalls
    # it on a second run.
    # 
    # To bypass this behaviour, we check here if a patch needs to be removed as part of
    # `remove_patches_before` and mark it absent here. So Puppet will immediately reinstall it. 
    #
    patches_to_remove = requested_patches.select { |k, v| !installed_patches.include?(k) && v['ensure'] == :present && v['remove_patches_before'] != [] }.collect do | patch, info |
      home = patch.split(':').first
      info['remove_patches_before'].collect { |p| "#{home}:#{p}" }
    end.flatten.uniq
    requested_patches.each_key do |patch_name|
      if installed_patches.include?(patch_name) && !patches_to_remove.include?(patch_name)
        requested_patches[patch_name].provider = new(:name => name, :ensure => :present)
      end
    end
  end

  [:patch_id, :oracle_product_home_dir, :orainst_dir, :source, :tmp_dir].each do |prop|
    define_method(prop) do
      resource[prop]
    end
  end

  def remove_incompatible_patches
    force_remove = resource.force_remove
    fact = Facter.value(:ora_install_homes) || {}
    @installed_patches ||= fact.dig('installed_patches', oracle_product_home_dir) || []
    resource[:remove_patches_before].each do |remove_patch_id|
      if @installed_patches.include?(remove_patch_id) || force_remove.to_s == 'true'
        Puppet.debug "Incompatible patch #{remove_patch_id} will be removed from #{oracle_product_home_dir}."
        begin
          opatch("rollback -id #{remove_patch_id} -silent -oh #{oracle_product_home_dir} -local", {:os_user => os_user, :oracle_home => oracle_product_home_dir})
        rescue => e
          raise unless force_remove

          Puppet.debug "uninstall of patch #{remove_patch_id} failed with error #{e.message}, but we are continuing..."
        end
        #
        # because multiple patches can have the same incompatible patch, make sure
        # it is only removed once.
        #
        @installed_patches.delete(remove_patch_id)
      else
        Puppet.debug "Incompatible patch #{remove_patch_id} not found in #{oracle_product_home_dir}."
      end
    end
  end

  private

  def self.check_single_orainst_dir(resources)
    orainst_dir = resources.map { |k, v| v.orainst_dir }.uniq
    raise "ora_opatch doesn't support multiple orainst_dir" if orainst_dir.size > 1

    orainst_dir.first
  end

  def self.use_fact?(resources)
    resources.map do |_name, properties|
      if properties.use_fact.is_a?(String)
        true if properties.use_fact == 'true'
      elsif properties.use_fact.is_a?(TrueClass)
        properties.use_fact
      elsif properties.use_fact.is_a?(FalseClass)
        properties.use_fact
      elsif properties.use_fact.is_a?(Symbol)
        true if properties.use_fact == :true
      end
    end.any?
  end
end
