#
# See the file "LICENSE" for the full license governing this code.
#
require 'puppet_x/enterprisemodules/ora_install/core'

module Puppet
  #
  Type.newtype(:ora_tab_entry) do
    include EasyType
    include ::PuppetX::EnterpriseModules::OraInstall::GetHomes
    extend ::PuppetX::EnterpriseModules::OraInstall::GetHomes

    desc <<-EOD
    With this type you can manage entries in the systems oratab file.

    Here is an example on how to use this:

      ora_tab_entry { 'MYDB':
          ensure      => 'present',
          comment     => 'My own database',
          oracle_home => '/u01/app/oracle/product/12.2.0.1/db_home1',
          startup     => 'Y',
        }

    EOD

    to_get_raw_resources do
      if oratab_exists?
        entries = oratab_content.split("\n").select {|e| e[0] != '#' && e != ''}
        entries.collect {|e| Hash[[:name,:home, :startup, :comment].zip(e.split(/[:#]/))]}
      else
        []
      end
    end

    on_create do
      with_content { |c| add_entry(c) }
    end

    on_modify do
      with_content do |content|
        change_entry(content)
      end
    end

    on_destroy do
      with_content { |c| remove_entry(c) }
    end

    def with_content
      if oratab_exists?
        uid = File.stat(oratab_file).uid
        user = Etc.getpwuid(uid).name
        gid = File.stat(oratab_file).gid
        group = Etc.getgrgid(gid).name
        content = yield(oratab_content)
      else
        #
        # When the oratab does not yet exists, we use user and group root
        # we are not sure the oracle user exists and what it is
        #
        user    = 'root'
        group   = 'root'
        content = yield('')
      end
      File.write(oratab_file, content)
      FileUtils.chown(user, group, oratab_file)
      FileUtils.chmod(0755, oratab_file)
    end

    def add_entry(content)
      if content[-1] == "\n"
        content += "#{name}:#{oracle_home}:#{startup}\n"
      else
        content += "\n#{name}:#{oracle_home}:#{startup}\n"
      end
    end

    def remove_entry(content)
      content.split("\n").select {|e| e !~ /^#{name}:.*$/}.join("\n")
    end

    def change_entry(content)
      home = oracle_home || provider.oracle_home
      start_value = startup || provider.startup
      content.split("\n").collect {|e| e.gsub(/^#{name}:.*$/,"#{name}:#{home}:#{start_value}")}.join("\n")
    end

    ensurable
    parameter :name
    property  :oracle_home
    property  :startup
    property  :comment
  end
end
