require 'rexml/document'
require 'facter'
require_relative './core'

module PuppetX
  module EnterpriseModules
    module OraInstall
      module Facts
        include ::PuppetX::EnterpriseModules::OraInstall::GetHomes
        if Facter.value(:kernel) == 'windows'
          require_relative './windows_core'
          include PuppetX::EnterpriseModules::OraInstall::WindowsCore
        end

        def get_database_user(home)
          database_user = Facter.value('override_database_user')
          return database_user unless database_user.nil?

          # puts "database user is oracle"
          if File.directory?("#{home}/bin")
            Facter::Util::Resolution.exec("/bin/stat --format=%U #{home}/bin/oracle")
          else
            'oracle'
          end
        end

        def get_su_command
          if os == 'Linux'
            return 'su -l '
          elsif os == 'SunOS'
            return 'su - '
          end

          'su -l '
        end

        def get_opatch_version(home)
          version = lsinventory(home).scan(/OPatch [V|v]ersion\s*:\s*(.*)/).flatten.first
          Puppet.debug "ora_install opatch version #{home}/OPatch/opatch: #{version}"
          version.nil? ? 'Invalid' : version
        end

        def jdk_clause(home)
          Dir.exist?("#{home}/jdk") ? "-jdk #{home}/jdk" : ''
        end

        def lsinventory(home)
          @lsinventory ||= {}
          if os == 'windows'
            @lsinventory[home] ||= `#{home}\\OPatch\\opatch.bat lsinventory -jdk #{home}\\jdk`
          elsif supports_nolog?(home)
            @lsinventory[home] ||= Facter::Util::Resolution.exec("su #{user_for_file(home)} -c 'ORACLE_HOME=#{home} #{home}/OPatch/opatch lsinventory -jdk #{home}/jdk -customLogDir /tmp'")
            # We remove the created directory here, because it sometimes creates a dir with an other owner, which causes troubles for the next oracle home
            Puppet.debug `rm -rf /tmp/opatch`
            @lsinventory[home]
          else
            if supports_nolog?(home)
              @lsinventory[home] ||= Facter::Util::Resolution.exec("su #{user_for_file(home)} -c 'ORACLE_HOME=#{home} #{home}/OPatch/opatch lsinventory #{jdk_clause(home)} -customLogDir /tmp'")
              # We remove the created directory here, because it sometimes creates a dir with an other owner, which causes troubles for the next oracle home
              Puppet.debug `rm -rf /tmp/opatch`
              @lsinventory[home]
            else
              @lsinventory[home] ||= Facter::Util::Resolution.exec("su #{user_for_file(home)} -c 'ORACLE_HOME=#{home} #{home}/OPatch/opatch lsinventory #{jdk_clause(home)}'")
            end
          end
        end

        def supports_nolog?(home)
          @lsinventory_help ||= {}
          @lsinventory_help[home] ||= Facter::Util::Resolution.exec("su #{user_for_file(home)} -c 'ORACLE_HOME=#{home} #{home}/OPatch/opatch lsinventory #{jdk_clause(home)} -help'")
          @lsinventory_help[home] =~ /customLogDir/
        end

        def get_home_name(home)
          inventory_loc = get_orainst_loc
          return 'Invalid' unless File.exist?("#{inventory_loc}/ContentsXML/inventory.xml")

          file = File.read("#{inventory_loc}/ContentsXML/inventory.xml")
          doc = REXML::Document.new(file)
          doc.elements.each('/INVENTORY/HOME_LIST/HOME') do |element|
            return element.attributes['NAME'] if element.attributes['LOC'] == home
          end
        end

        def get_version(home)
          version = opatch_installed?(home) ? lsinventory(home).scan(/^(?:Oracle Database|Oracle Client|Oracle Grid Infrastructure|Oracle GoldenGate)\s.+?\s+(.*)$/).flatten.first : 'Invalid'
          version.nil? ? 'Invalid' : version
        end

        def get_installed_patches(oracle_home)
          if File.directory?(oracle_home)
            ora_user = user_for_file(oracle_home)
            installed_patches = patches_in_home(oracle_home, ora_user).flatten.sort
          else
            installed_patches = ['Invalid']
          end
          cleanup_opatch_residue(oracle_home) if Facter.value(:kernel) == 'Linux'
          installed_patches
        end

        def patches_in_home(oracle_product_home_dir, os_user)
          if opatch_installed?(oracle_product_home_dir)
            lsinventory(oracle_product_home_dir).scan(/Patch\s.(\d+)\s.*:\sapplied on/).flatten
          else
            #
            # When Opatch is not installed, we don't report any patches
            #
            ['Invalid']
          end
        end

        def opatch_installed?(oracle_home)
          if os == 'windows'
            File.exist?("#{oracle_home}\\OPatch\\opatch.bat")
          else
            File.exist?("#{oracle_home}/OPatch/opatch")
          end
        end

        def first_non_root_file(dir)
          Dir.new(dir).each { |file| break file if File.stat("#{dir}/#{file}").uid.nonzero? }
        end

        def cleanup_opatch_residue(home)
          # Some older versions of Opatch leave files and directories arround
          # When running opatch often, this causes some issues. This is to make
          # sure all files created by running Opatch as part of the fact, are remopved
          basename = Time.now.strftime('%Y-%m-%d_%H-%M-*%p')
          Puppet.debug `rm -rf /tmp/OraInstall*`
          Puppet.debug `rm -rf /tmp/opatch`
          FileUtils.rm_f("#{home}/cfgtoollogs/opatch/lsinv/lsinventory#{basename}.txt")
        end

        def get_defined_sids(home)
          if os == 'windows'
            home_name = get_home_name(home)
            regkey = Win32::Registry::HKEY_LOCAL_MACHINE.open("SOFTWARE\\Oracle\\KEY_#{home_name}")
            regkey.reject { |k, _v1, _v| k.scan(/ORA_(.*)_AUTOSTART/) == [] }.flatten.collect { |k| k.to_s.scan(/ORA_(.*)_AUTOSTART/) }.flatten.uniq
          else
            quoted_home = Regexp.quote(home)
            oratab_content.scan(/^([+-_A-Za-z].*):#{quoted_home}:[YN].*$/).flatten.uniq
          end
        end

        def get_running_processes(home)
          {
            'sids'        => get_running_sids(home),
            'listeners'   => get_running_listeners(home),
            'conn_mgrs'   => get_running_connmgrs(home)
          }
        end

        def get_running_sids(home)
          if os == 'windows'
            raw_array = []
            services = `for /f "tokens=3" %i in ('tasklist /svc /fi "imagename eq oracle.exe" ^| findstr "oracle.exe"') do @echo %i`
            services.split("\n").each do |service|
              db = service.gsub('OracleService', '')
              regkey = Win32::Registry::HKEY_LOCAL_MACHINE.open("SYSTEM\\CurrentControlSet\\Services\\#{service}")
              oracle_home = regkey['ImagePath'].gsub("\\bin\\ORACLE.EXE #{db}", '')
              raw_array << "#{db} ,#{oracle_home}"
            end
            raw_info = raw_array.join("\n")
          else
            command = %Q(ps aux | egrep "(ora|asm)_pmon" | awk '{gsub("(ora|asm)_pmon_", ""); print $2, $NF}' |
                         while read pid sid; do
                             printf "%-20s,%-80s\n" $sid  $(ls -l /proc/$pid/exe | awk '{ print $NF }' | sed 's/\\/bin\\/oracle$//' | sort | uniq)
                         done)
            raw_info = Facter::Util::Resolution.exec(command)
            unless $?.success?
              puts 'Fetching running SIDS, needs access to /proc/pid/exe file system.'
              puts 'Probably using docker in non-privileged mode.'
              puts 'Run docker in privileged mode to fix the issue.'
              nil
            end
          end
          return {} if raw_info.empty?

          databases = raw_info.split("\n").select { |e| e.downcase.include?(home.downcase)}.collect { |e| e.split(',').first.strip }.sort
          db_hash = databases.collect do |db|
            db_mode_role = db =~ /ASM/ ? ['n/a', 'n/a'] : get_mode_role(db, home).split
            {
              db => {
                'open_mode' => db_mode_role[0],
                'database_role' => db_mode_role[1]
              }
            }
          end
          Hash[*db_hash.collect { |h| h.to_a }.flatten]
        end

        def get_running_listeners(home)
          if os == 'windows'
            listener_services = `for /f "tokens=3" %i in ('tasklist /svc /fi "imagename eq tnslsnr.exe" ^| findstr "tnslsnr.exe"') do @echo %i`.split("\n")
            listeners = listener_services.map do |svc|
              svc_home_name, lsnr_name = svc.scan(/Oracle(.*)TNSListener(.*)/).flatten
              if svc_home_name == get_home_name(home)
                if lsnr_name.empty?
                  'LISTENER'
                else
                  lsnr_name
                end
              end
            end
            listeners.compact
          else
            command = %Q(ps -eo pid,cmd | grep "#{home}/bin/tns[l]snr" | grep -v cman | sed 's/-inherit//' | sed 's/-no_crs_notify//' | sed 's/-mode proxy//' |
                        while read pid command listener; do
                            printf "%-9s %-80s\n" $listener  $(ls -l /proc/$pid/exe | awk '{ print $NF }' | sort | uniq)
                        done)
            raw_info = Facter::Util::Resolution.exec(command)
            if $?.success?
              return [] if raw_info.empty?
              raw_info.split("\n").select{|e| e.include?(home)}.collect {|e| e.split.first}.sort
            else
              command = %Q(ps -eo pid,cmd | grep "#{home}/bin/tns[l]snr" | sed 's/-inherit//' | sed 's/-no_crs_notify//' |
                          while read pid command listener; do
                              printf "%-9s %-80s\n" $listener  $(ls -l /proc/$pid/exe | awk '{ print $NF }' | sort | uniq)
                          done)
              raw_info = Facter::Util::Resolution.exec(command)
              if $?.success?
                return [] if raw_info.empty?

                raw_info.split("\n").select { |e| e.include?(home) }.collect { |e| e.split.first }.sort
              else
                puts 'Fetching running listeners, needs access to /proc/pid/exe file system.'
                puts 'Probably using docker in non-privileged mode.'
                puts 'Run docker in privileged mode to fix the issue.'
                nil
              end
            end
          end
        end

        def get_running_connmgrs(home)
          command = %Q(ps -eo pid,cmd | grep "#{home}/bin/cmadmin" | grep -v grep | awk '{print $3}')
          if os == 'windows'
            raw_info = ""
          else
            raw_info = Facter::Util::Resolution.exec(command)
          end
          raw_info.split("\n").compact
        end

        def get_mode_role(sid, home)
          if os == 'windows'
            service_state = `sc query OracleService#{sid}`.scan(/STATE *:.*(RUNNING)/).flatten.first
            if service_state == 'RUNNING'
              sql = %(select replace(open_mode, chr(32), chr(95)) as open_mode
                           , replace(database_role, chr(32), chr(95)) as database_role
                        from v$database)
              execute_sql(sid, sql)
            else
              'Invalid Invalid'
            end
          else
            command     = %(sqlplus -S /nolog << EOF
              connect / as sysdba
              set heading off
              select replace(open_mode, chr(32), chr(95)) as open_mode
                   , replace(database_role, chr(32), chr(95)) as database_role
                from v\\$database;
EOF)
            `su #{user_for_file(home)} -c 'export ORACLE_HOME="#{home}"; \
                                          export PATH="#{home}/bin:$PATH"; \
                                          export ORACLE_SID="#{sid}"; \
                                          unset TNS_ADMIN; \
                                          unset TWO_TASK; \
                                          export LD_LIBRARY_PATH="#{home}/lib"; \
                                          #{command}'`
          end
        end

        def get_product_version(oracle_home)
          product_version = {
            'name'    => get_home_name(oracle_home),
            'os_user' => user_for_file(oracle_home),
            'version' => get_version(oracle_home)
          }
          cleanup_opatch_residue(oracle_home) if Facter.value(:kernel) == 'Linux'
          product_version
        end
      end
    end
  end
end
