#
# ora_install::autostartdatabase
#
# @summary This defined type create's a startup script for the specified database and enables the service.
# The end result is that the named Oracle database is
# restarted after a system restart.
# 
# Here is an example on how to use it:
# 
# ``` puppet
# ora_install::autostartdatabase{ 'autostart oracle':
#   oracle_home             => '/oracle/product/12.1/db',
#   user                    => 'oracle',
#   db_name                 => 'test',
# }
# ```
#
# @param [Optional[String[1]]] db_domain
#    The domain of the database.
#
# @param [String[1]] db_name
#    The name of the database.
#
# @param [Enum['AUTOMATIC', 'MANUAL', 'NORESTART']] db_policy
#    The srvctl management policy for the database
#    Valid values are:
#    - `AUTOMATIC`
#    - `MANUAL`
#    - `NORESTART`
#    The default value is: `AUTOMATIC`
#
# @param [Enum['database', 'grid']] db_type
#    The type of the database used to specify if the database should be started by an init script or srvctl.
#    Valid values are:
#    - `grid`
#    - `database`
#    The default value is: 'database'
#
# @param [String[1]] group
#    The os group to use for these Oracle puppet definitions.
#    The default value is: `dba`
#
# @param [Optional[Hash]] limits
#    The limits for the systemd service.
#    The default value is: undef
#    Below is an axample. Currently only the 'hard' values are used.
#    ```yaml
#    ora_install::autostartdatabase::limits:
#      '*/nofile':
#        soft: 2048
#        hard: 8192
#      'oracle/nofile':
#        soft: 65536
#        hard: 65536
#      'oracle/nproc':
#        soft: 2048
#        hard: 16384
#      'oracle/stack':
#        soft: 10240
#        hard: 32768
#    ```
#
# @param [Variant[Boolean, Enum['on_failure']]] logoutput
#    log the outputs of Puppet exec or not.
#    When you specify `true` Puppet will log all output of `exec` types.
#    Valid values are:
#    - `true`
#    - `false`
#    - `on_failure`
#
# @param [Stdlib::Absolutepath] oracle_home
#    A directory to be used as Oracle home directory for this software.
#
# @param [String[1]] service_name
#    The service name to start.
#
# @param [Enum['OPEN', 'MOUNT', 'READONLY']] start_option
#    The srvctl startup options for the database.
#    Valid values are:
#    - `OPEN`
#    - `MOUNT`
#    - `READ ONLY`
#    The default value is: `OPEN`
#
# @param [Optional[String[1]]] systemd_template
#    Use custom EPP template for systemd service
#    The default value is: undef
#
# @param [Optional[Hash]] systemd_template_vars
#    The variables to use when specifying a custom EPP template for systemd service
#    The default value is: undef
#
# @param [String[1]] user
#    The user used for the specified installation.
#    The install class will not create the user for you. You must do that yourself.
#    The default value is: `oracle`
#
#
# See the file "LICENSE" for the full license governing this code.
#
define ora_install::autostartdatabase (
  Stdlib::Absolutepath                   $oracle_home           = undef,
  String[1]                              $db_name               = undef,
  String[1]                              $user                  = 'oracle',
  String[1]                              $group                 = 'dba',
  String[1]                              $service_name          = 'dbora',
  Enum['database','grid']                $db_type               = 'database',
  Enum['OPEN','MOUNT','READ ONLY']       $start_option          = 'OPEN',
  Enum['AUTOMATIC','MANUAL','NORESTART'] $db_policy             = 'AUTOMATIC',
  Optional[String[1]]                    $db_domain             = undef,
  Optional[Hash]                         $limits                = undef,
  Optional[String[1]]                    $systemd_template      = undef,
  Optional[Hash]                         $systemd_template_vars = undef,
  Variant[Boolean,Enum['on_failure']]    $logoutput             = lookup({ name => 'logoutput', default_value => 'on_failure' }),
) {
  ora_install { $title: ensure => 'present' }
  $exec_path    = '/usr/local/bin:/bin:/usr/bin:/usr/local/sbin:/usr/sbin:/sbin:'

  easy_type::debug_evaluation() # Show local variable on extended debug

  if $db_type == 'grid' {
    if $db_domain {
      $domain_clause = "-m ${db_domain}"
    } else {
      $domain_clause = ''
    }
    exec { "adding database ${db_name}:${oracle_home} as has service":
      command     => "${oracle_home}/bin/srvctl add database -d ${db_name} -o ${oracle_home} -s ${start_option} -n ${db_name} -y ${db_policy} ${domain_clause}",
      onlyif      => "${oracle_home}/bin/srvctl status database -d ${db_name} | grep -q 'Resource ora.*.db does not exist'",
      environment => ["ORACLE_SID=${db_name}","ORACLE_HOME=${oracle_home}",'ORAENV_ASK=NO'],
      cwd         => $oracle_home,
      user        => $user,
      group       => $group,
      path        => $exec_path,
      logoutput   => $logoutput,
      noop        => ora_install::changed_oracle_home_for($db_name, $oracle_home),
    }
    ~> exec { "updating status of database ${db_name}:${oracle_home}":
      command     => "${oracle_home}/bin/srvctl start database -d ${db_name}",
      onlyif      => "${oracle_home}/bin/srvctl status database -d ${db_name} | grep -q 'Database is not running.'",
      environment => ["ORACLE_SID=${db_name}","ORACLE_HOME=${oracle_home}",'ORAENV_ASK=NO'],
      cwd         => $oracle_home,
      user        => $user,
      group       => $group,
      path        => $exec_path,
      refreshonly => true,
      logoutput   => $logoutput,
      noop        => ora_install::changed_oracle_home_for($db_name, $oracle_home),
    }
  } else {
    ora_install::prepareautostart_for { $service_name:
      oracle_home           => $oracle_home,
      user                  => $user,
      group                 => $group,
      limits                => $limits,
      systemd_template      => $systemd_template,
      systemd_template_vars => $systemd_template_vars,
      noop                  => ora_install::changed_oracle_home_for($db_name, $oracle_home),
    }

    # Add the 'Y' to the startup value of oratab. We use resource_value,
    # because it may already be in te catalog and we ONLY want to
    # manage the startup property. We use allow create because it might also
    # no yet be available in the catalog.
    #
    # Skip this if we have a changed Oracle home. Use a noop because
    # then we can see something is about to change.
    #
    unless $facts['kernel'] == 'Windows' {
      resource_value { "Ora_tab_entry[${db_name}]startup":
        value          => 'Y',
        allow_redefine => true,
        allow_create   => true,
        warnings       => false,
        noop           => ora_install::changed_oracle_home_for($db_name, $oracle_home),
      }
    }
  }
}
