#
# ora_install::database
#
# @summary This defined type allows you to create or delete a database.
# Under the hood the calls the `dbca` utility to do the work. When you need more control over the individual settings when creating the database, you must use the [`ora_database` custom type of the `ora_config` module](https://www.enterprisemodules.com/docs/ora_config/ora_database.html)
# 
# Here is an example on how to use this defined type to create a database:
# 
# ```puppet
# ora_install::database{ 'testDb_Create':
#   action                    => 'create',
#   oracle_base               => '/oracle',
#   oracle_home               => '/oracle/product/11.2/db',
#   version                   => '11.2',
#   user                      => 'oracle',
#   group                     => 'dba',
#   download_dir              => '/install',
#   action                    => 'create',
#   db_name                   => 'test',
#   db_domain                 => 'oracle.com',
#   db_port                   => '1521',
#   sys_password              => 'Welcome01',
#   system_password           => 'Welcome01',
#   data_file_destination     => "/oracle/oradata",
#   recovery_area_destination => "/oracle/flash_recovery_area",
#   character_set             => "AL32UTF8",
#   nationalcharacter_set     => "UTF8",
#   init_params               => {'open_cursors'        => '1000',
#                                 'processes'           => '600',
#                                 'job_queue_processes' => '4' },
#   sample_schema             => 'TRUE',
#   memory_percentage         => "40",
#   memory_total              => "800",
#   database_type             => "MULTIPURPOSE",
#   em_configuration          => "NONE",
#   require                   => Ora_install::Listener['start listener'],
# }
# ```
#
# @param [Enum['create', 'delete']] action
#    The action that must be taken.
#    create or delete.
#    Valid values are:
#    - `create`
#    - `delete`
#    The default value is: `create`
#
# @param [String[1]] asm_diskgroup
#    The ASM diskgroup to use for the database.
#    The default value is: `DATA`
#
# @param [Optional[Easy_type::Password]] asm_snmp_password
#    The password to use for the ASMSNMP user.
#
# @param [String[1]] character_set
#    The character set to use for running the database.
#    The default value is: `AL32UTF8`
#
# @param [Optional[String[1]]] cluster_nodes
#    A comma seperated list of the nodes running in the RAC cluster.
#    Example: `node1,node2`
#
# @param [Boolean] container_database
#    Database is a container for pluggable databases.
#    When you want to add pluggable database to this database, specify a value of `true`.
#    The default value is: `false`
#
# @param [Optional[String[1]]] data_file_destination
#    The location of the data file's.
#
# @param [Enum['MULTIPURPOSE', 'DATA_WAREHOUSING', 'OLTP']] database_type
#    The kind of database you want to create.
#    Valid values are:
#    - `MULTIPURPOSE`
#    - `DATA_WAREHOUSING`
#    - `OLTP`
#    The default value is: `MULTIPURPOSE`
#
# @param [Enum['SINGLE', 'RAC', 'RACONE']] db_conf_type
#    The type of database that needs to be installed.
#    Valid values are:
#    - `SINGLE`
#    - `RAC`
#    - `RACONE`
#
# @param [Optional[String[1]]] db_domain
#    The domain of the database.
#
# @param [String[1]] db_name
#    The name of the database.
#
# @param [Integer] db_port
#    The IP port to use for the database.
#    The default value is: `1521`
#
# @param [Optional[Easy_type::Password]] db_snmp_password
#    Password to use for the dbsnmp user.
#
# @param [Stdlib::Absolutepath] download_dir
#    The directory where the Puppet software puts all downloaded files.
#    Before Puppet can actually use remote files, they must be downloaded first. Puppet uses this directory to put all files in.
#    The default value is: `/install`
#
# @param [Enum['CENTRAL', 'LOCAL', 'ALL', 'NONE', 'DBEXPRESS', 'BOTH']] em_configuration
#    The type of Enterprise Manager configuration.
#    Valid values are:
#    - `CENTRAL`   (all versions)
#    - `LOCAL`     (version < 12.2)
#    - `ALL`       (version < 12.2)
#    - `NONE`      (all versions)
#    - `DBEXPRESS` (version >= 12.2)
#    - `BOTH`      (version >= 12.2)
#    The default value is: `NONE`
#
# @param [String[1]] group
#    The os group to use for these Oracle puppet definitions.
#    The default value is: `dba`
#
# @param [Optional[Variant[String[1], Hash]]] init_params
#    The init parameters to use for the database.
#    You can use either a comma separated string for init_params or a Hash.
#    ### Using comma separated string
#    Here is an example using a comma separated string:
#    ``` puppet
#    ora_install::database{'my_database'
#      ...
#      init_params => "open_cursors=1000,processes=600,job_queue_processes=4",
#      ...
#    }
#    ```
#    ### Using a Hash
#    Here is an example using a Hash:
#    ``` puppet
#    ora_install::database{'my_database'
#      ...
#      init_params  => {'open_cursors'        => '1000',
#                      'processes'           => '600',
#                      'job_queue_processes' => '4' },
#      }
#      ...
#    }
#    ```
#
# @param [Variant[Boolean, Enum['on_failure']]] logoutput
#    log the outputs of Puppet exec or not.
#    When you specify `true` Puppet will log all output of `exec` types.
#    Valid values are:
#    - `true`
#    - `false`
#    - `on_failure`
#
# @param [Integer] memory_percentage
#    The percentage of physical memory for Oracle.
#    The default value is: `40`
#
# @param [Integer] memory_total
#    The total memory in MB to allocate to Oracle.
#    The default value is: `800`
#
# @param [String[1]] nationalcharacter_set
#    Use this NLS Character set.
#    The default value is: `UTF8`
#
# @param [String[1]] oracle_base
#    A directory to use as Oracle base directory.
#
# @param [String[1]] oracle_home
#    A directory to be used as Oracle home directory for this software.
#
# @param [Optional[String[1]]] puppet_download_mnt_point
#    The base path of all remote files for the defined type or class.
#    The default value is: `puppet:///modules/ora_install`
#
# @param [Optional[String[1]]] recovery_area_destination
#    Location of the recovary files.
#
# @param [Optional[String[1]]] recovery_diskgroup
#    Diskgroup to use for recovery files.
#
# @param [Enum['TRUE', 'FALSE']] sample_schema
#    Load the sample schema or not.
#    **Attention** This is a string and not a boolean value.
#    The default value is: `TRUE`
#
# @param [Enum['FS', 'CFS', 'ASM']] storage_type
#    The storage type to use.
#    Valid values are:
#    - `FS`
#    - `CFS`
#    - `ASM`
#    `FS`
#
# @param [Easy_type::Password] sys_password
#    The password to use for the `SYS` user.
#
# @param [Easy_type::Password] system_password
#    The password to use for the `SYSTEM` user.
#
# @param [Optional[String[1]]] template
#    Use the specified template to create the database.
#    You can choose between a set of pre-delivered templates, but also specify your own templates.
#    ## Using a pre-delivered template
#    `ora_install` delivers a few predefined templates.
#    - dbtemplate_11GR2
#    - dbtemplate_12.1
#    - dbtemplate_12.1_asm
#    Here is an example on how to use this:
#    ```puppet
#    ora_install::database{ 'testDb_Create':
#      ...
#      template                  => 'dbtemplate_12.1',
#      ...
#    }
#    ```
#    ## Using your own template
#    When you specify a template name not pre-delivered by `ora_install`, the defined type will look in the directory specified by `puppet_download_mnt_point` for your own template.
#    Here is an example:
#    ```puppet
#      template                   => 'my_dbtemplate_11gR2_asm',
#      puppet_download_mnt_point  => '/vagrant', # 'ora_install' etc
#    ```
#    The template must be have the following extension dbt.erb like `dbtemplate_12.1.dbt.erb`
#    - Click here for an [12.1 db instance template example](https://github.com/enterprisemodules/ora_install/blob/master/templates/dbtemplate_12.1.dbt.erb)
#    - Click here for an [11.2 db asm instance template example](https://github.com/enterprisemodules/ora_install/blob/master/templates/dbtemplate_11gR2_asm.dbt.erb)
#
# @param [Enum['non-seed', 'seed']] template_type
#    The template type `DBCA` uses.
#    There are two types of templates:
#    - `non-seed`
#    - `seed`
#    The default value is: `non-seed`
#
# @param [String[1]] user
#    The user used for the specified installation.
#    The install class will not create the user for you. You must do that yourself.
#    The default value is: `oracle`
#
# @param [Enum['AUTO', 'AUTO_SGA', 'CUSTOM_SGA']] memory_mgmt_type
#    The type of memory management to use for the database.
#    The options are:
#    - `AUTO`
#    - `AUTO_SGA`
#    - `CUSTOM_SGA`
#    The default is `AUTO_SGA`
#
# @param [Ora_install::ShortVersion] version
#    The version that is installed in the used Oracle home.
#    Puppet uses this value to decide on version specific actions.
#
#
# See the file "LICENSE" for the full license governing this code.
#
define ora_install::database (
  # lint:ignore:strict_indent
  Easy_type::Password                 $sys_password,
  Easy_type::Password                 $system_password,
  Optional[Easy_type::Password]       $asm_snmp_password         = undef,
  Optional[Easy_type::Password]       $db_snmp_password          = undef,
  String[1]                           $oracle_base               = undef,
  String[1]                           $oracle_home               = undef,
  Ora_install::ShortVersion           $version                   = '12.2',
  String[1]                           $user                      = 'oracle',
  String[1]                           $group                     = 'dba',
  Stdlib::Absolutepath                $download_dir              = '/install',
  Enum['create','delete']             $action                    = 'create',
  Optional[String[1]]                 $template                  = undef,
  Enum['non-seed','seed']             $template_type             = 'non-seed',
  String[1]                           $db_name                   = 'orcl',
  Optional[String[1]]                 $db_domain                 = undef,
  Integer                             $db_port                   = 1521,
  Optional[String[1]]                 $data_file_destination     = undef,
  Optional[String[1]]                 $recovery_area_destination = undef,
  String[1]                           $character_set             = 'AL32UTF8',
  String[1]                           $nationalcharacter_set     = 'AL16UTF16',
  Optional[Variant[String[1], Hash]]  $init_params               = undef,
  Enum['TRUE','FALSE']                $sample_schema             = 'TRUE',
  Integer                             $memory_percentage         = 40,
  Integer                             $memory_total              = 1000,
  Enum['AUTO','AUTO_SGA','CUSTOM_SGA']
                                      $memory_mgmt_type          = 'AUTO_SGA',
  Enum['MULTIPURPOSE','DATA_WAREHOUSING','OLTP']
                                      $database_type             = 'MULTIPURPOSE',
  Enum['CENTRAL','LOCAL','ALL','NONE','DBEXPRESS','BOTH']
                                      $em_configuration          = 'NONE',
  Enum['FS','CFS','ASM']              $storage_type              = 'FS',
  String[1]                           $asm_diskgroup             = 'DATA',
  Optional[String[1]]                 $recovery_diskgroup        = undef,
  Enum['SINGLE','RAC','RACONE']       $db_conf_type              = 'SINGLE',
  Optional[String[1]]                 $cluster_nodes             = undef,
  Boolean                             $container_database        = false,
  Optional[String[1]]                 $puppet_download_mnt_point = undef,
  Variant[Boolean,Enum['on_failure']] $logoutput                 = lookup({ name => 'logoutput', default_value => 'on_failure' }),
  # lint:endignore:strict_indent
) {
  ora_install { $title: ensure => 'present' }

  $sanitized_title = regsubst($title, '[^a-zA-Z0-9.-]', '_', 'G')

  if ( $action == 'create' ) {
    $operation_type = 'createDatabase'
  } elsif ( $action == 'delete' ) {
    $operation_type = 'deleteDatabase'
  }

  case $version {
    '11.2': {
      unless ( $em_configuration in ['CENTRAL','LOCAL','ALL','NONE']) { fail("em_configuration should be defined as CENTRAL, LOCAL, ALL or NONE for version ${version}") }
      if ( $container_database ) { fail('container or pluggable database is not supported on version 11.2') }
      $action_param = ''
      $delete_command = "${oracle_home}/bin/dbca -silent ${action_param} -responseFile ${download_dir}/database_${sanitized_title}.rsp"
      case $storage_type {
        'ASM': {
          $df_dest = regsubst($data_file_destination,'\+','')
          $reco_dest = regsubst($recovery_area_destination,'\+','')
          $storage_param = "-storageType ${storage_type} -diskGroupName ${df_dest} -recoveryGroupName ${reco_dest}"
        }
        'CFS','FS': {
          $storage_param = "-storageType ${storage_type} -datafileDestination ${data_file_destination} -recoveryAreaDestination ${recovery_area_destination}"
        }
        default: {}
      }
      $memory_mgmt_type_param = ''
      $db_conf_type_param = ''
      if ( $cluster_nodes ) {
        $nodelist_param = "-nodelist ${cluster_nodes}"
      } else {
        $nodelist_param = ''
      }
      $ignore_prereq_param = ''
    }
    '12.1': {
      unless ( $em_configuration in ['CENTRAL','LOCAL','ALL','NONE']) { fail("em_configuration should be defined as CENTRAL, LOCAL, ALL or NONE for version ${version}") }
      $action_param = ''
      $delete_command = "${oracle_home}/bin/dbca -silent ${action_param} -responseFile ${download_dir}/database_${sanitized_title}.rsp"
      case $storage_type {
        'ASM': {
          $df_dest = regsubst($data_file_destination,'\+','')
          $reco_dest = regsubst($recovery_area_destination,'\+','')
          $storage_param = "-storageType ${storage_type} -diskGroupName ${df_dest} -recoveryGroupName ${reco_dest}"
        }
        'CFS','FS': {
          $storage_param = "-storageType ${storage_type} -datafileDestination ${data_file_destination} -recoveryAreaDestination ${recovery_area_destination}"
        }
        default: {}
      }
      $memory_mgmt_type_param = ''
      $db_conf_type_param = "-databaseConfType ${db_conf_type}"
      if ( $cluster_nodes ) {
        $nodelist_param = "-nodelist ${cluster_nodes}"
      } else {
        $nodelist_param = ''
      }
      $ignore_prereq_param = '-ignorePrereqs'
    }
    '12.2': {
      unless ( $em_configuration in ['CENTRAL','DBEXPRESS','BOTH','NONE']) { fail("em_configuration should be defined as CENTRAL, DBEXPRESS, BOTH or NONE for version ${version}") }
      $action_param = "-${operation_type}"
      # TODO: Check this for RAC, -sid should be different
      $delete_command = "${oracle_home}/bin/dbca -silent ${action_param} -sourceDB ${sanitized_title} -sid ${sanitized_title} -sysDBAPassword \"${sys_password.unwrap}\" -sysDBAUserName sys"
      $storage_param = "-storageType ${storage_type} -datafileDestination ${data_file_destination} -recoveryAreaDestination ${recovery_area_destination}"
      $memory_mgmt_type_param = "-memoryMgmtType ${memory_mgmt_type}"
      $db_conf_type_param = "-databaseConfigType ${db_conf_type}"
      if ( $cluster_nodes ) {
        $nodelist_param = "-nodelist ${cluster_nodes}"
      } else {
        $nodelist_param = ''
      }
      $ignore_prereq_param = '-ignorePrereqs'
    }
    '18.0', '19.0', '21.0': {
      if ( $version == '21.0' ) {
        unless ( $container_database ) {
          fail("Oracle version ${version} only supports container_database=true, container_database=${container_database} specified")
        }
      }
      unless ( $em_configuration in ['CENTRAL','DBEXPRESS','BOTH','NONE']) { fail("em_configuration should be defined as CENTRAL, DBEXPRESS, BOTH or NONE for version ${version}") }
      $action_param = "-${operation_type}"
      # TODO: Check this for RAC, -sid should be different
      $delete_command = "${oracle_home}/bin/dbca -silent ${action_param} -sourceDB ${sanitized_title} -sid ${sanitized_title} -sysDBAPassword \"${sys_password.unwrap}\" -sysDBAUserName sys"
      $storage_param = "-storageType ${storage_type} -datafileDestination ${data_file_destination} -recoveryAreaDestination ${recovery_area_destination}"
      $memory_mgmt_type_param = "-memoryMgmtType ${memory_mgmt_type}"
      $db_conf_type_param = "-databaseConfigType ${db_conf_type}"
      if ( $cluster_nodes ) {
        $nodelist_param = "-nodelist ${cluster_nodes}"
      } else {
        $nodelist_param = ''
      }
      $ignore_prereq_param = '-ignorePrereqs'
    }
    default: {}
  }

  # if ( $memory_percentage and $memory_total ) { fail("cannot specify both memory_percentage and memory_total, choose one") }

  $exec_path = '/usr/local/bin:/bin:/usr/bin:/usr/local/sbin:/usr/sbin:/sbin'

  if $asm_snmp_password == undef {
    $asm_snmp_password_setting = ''
  } else {
    $asm_snmp_password_setting = "-asmsnmpPassword '${asm_snmp_password.unwrap}'"
  }

  if $db_snmp_password == undef {
    $db_snmp_password_setting = ''
  } else {
    $db_snmp_password_setting = "-dbsnmpPassword '${db_snmp_password.unwrap}'"
  }

  if ( $puppet_download_mnt_point ) {
    $mount_point = $puppet_download_mnt_point
  } else {
    $mount_point = 'ora_install/'
  }

  case $facts['kernel'] {
    'Linux': {
      $user_home = "/home/${user}"
    }
    'Windows': {
      notice 'TODO:'
      $user_home = "/export/home/${user}"
    }
    'SunOS': {
      $user_home = "/export/home/${user}"
    }
    default: {
      fail('Unrecognized operating system')
    }
  }

  if ( $init_params =~ Hash or $init_params =~ String ) {
    if ( $init_params =~ Hash ) {
      $init_params_array = sort(join_keys_to_values($init_params, '='))
      $sanitized_ini_params = join($init_params_array,',')
    } else {
      $sanitized_ini_params = $init_params
    }
    $dbca_init_params = "-initParams ${sanitized_ini_params}"
  } else {
    $dbca_init_params = ''
  }

  if ( $db_domain ) {
    $globaldb_name = "${db_name}.${db_domain}"
  } else {
    $globaldb_name = $db_name
  }

  if ( $template ) {
    $template_extention = $template_type ? {
      'non-seed' => 'dbt',
      'seed'     => 'dbc',
    }
    # lint:ignore:no_erb_template
    $template_name = "${download_dir}/${template}_${sanitized_title}.${template_extention}"
    $template_content = template("${mount_point}/${template}.${template_extention}.erb")
    # lint:endignore:no_erb_template
  }

  easy_type::debug_evaluation() # Show local variable on extended debug

  if ( $action == 'create' ) {
    if ( $template ) {
      if ( $container_database ) {
        $command = "${oracle_home}/bin/dbca -silent -createDatabase -templateName ${template_name} -gdbname ${globaldb_name} -createAsContainerDatabase ${container_database} -responseFile NO_VALUE -sysPassword '${sys_password.unwrap}' -systemPassword '${system_password.unwrap}' ${db_snmp_password_setting} ${asm_snmp_password_setting} ${storage_param} -emConfiguration ${em_configuration} -characterSet ${character_set} -nationalCharacterSet ${nationalcharacter_set} ${memory_mgmt_type_param} ${dbca_init_params} ${db_conf_type_param} ${nodelist_param} ${ignore_prereq_param}"
      } else {
        $command = "${oracle_home}/bin/dbca -silent -createDatabase -templateName ${template_name} -gdbname ${globaldb_name} -responseFile NO_VALUE -sysPassword '${sys_password.unwrap}' -systemPassword '${system_password.unwrap}' ${db_snmp_password_setting} ${asm_snmp_password_setting} ${storage_param} -emConfiguration ${em_configuration} -characterSet ${character_set} -nationalCharacterSet ${nationalcharacter_set} ${memory_mgmt_type_param} ${dbca_init_params} ${db_conf_type_param} ${nodelist_param} ${ignore_prereq_param}"
      }
    } else {
      unless ( defined(File["${download_dir}/database_${sanitized_title}.rsp"]) ) {
        file { "${download_dir}/database_${sanitized_title}.rsp":
          ensure  => file,
          content => epp("ora_install/dbca_${version}.rsp.epp",
            {
              operation_type            => $operation_type,
              globaldb_name             => $globaldb_name,
              db_name                   => $db_name,
              cluster_nodes             => $cluster_nodes,
              sys_password              => $sys_password,
              system_password           => $system_password,
              db_snmp_password          => $db_snmp_password,
              data_file_destination     => $data_file_destination,
              recovery_area_destination => $recovery_area_destination,
              storage_type              => $storage_type,
              asm_diskgroup             => $asm_diskgroup,
              asm_snmp_password         => $asm_snmp_password,
              recovery_diskgroup        => $recovery_diskgroup,
              character_set             => $character_set,
              nationalcharacter_set     => $nationalcharacter_set,
              sanitized_ini_params      => ( $init_params =~ Hash or $init_params =~ String )? {
                true  => $sanitized_ini_params,
                false => undef
              },
              sample_schema             => $sample_schema,
              memory_percentage         => $memory_percentage,
              database_type             => $database_type,
              memory_total              => $memory_total,
              db_port                   => $db_port,
              container_database        => $container_database,
              em_configuration          => $em_configuration,
              db_conf_type              => $db_conf_type,
          }),
          mode    => '0770',
          owner   => $user,
          group   => $group,
          before  => Exec["oracle database ${title}"],
        }
      }
      $command = "${oracle_home}/bin/dbca -silent ${action_param} -responseFile ${download_dir}/database_${sanitized_title}.rsp"
    }
    if $template {
      # We use an exec here to create a file. Because the file resource doesn't allow
      # an unless clause. And we need tht here.
      exec { "create template ${template_name}":
        command => "/bin/cat > ${template_name} <<'EOL'\n${template_content}\nEOL",
        unless  => "/bin/test -d ${oracle_base}/admin/${db_name}",
        user    => $user,
        group   => $group,
        before  => Exec["oracle database ${title}"],
      }

      cleanup { $template_name:
      }
    }

    exec { "oracle database ${title}":
      command     => $command,
      creates     => "${oracle_base}/admin/${db_name}",
      timeout     => 0,
      path        => $exec_path,
      user        => $user,
      group       => $group,
      cwd         => $oracle_base,
      environment => ["USER=${user}","ORACLE_HOME=${oracle_home}","ORACLE_BASE=${oracle_base}","LD_LIBRARY_PATH=${oracle_home}/lib"],
      logoutput   => $logoutput,
    }
  } elsif ( $action == 'delete' ) {
    exec { "oracle database ${title}":
      command     => $delete_command,
      onlyif      => "ls ${oracle_base}/admin/${db_name}",
      timeout     => 0,
      path        => $exec_path,
      user        => $user,
      group       => $group,
      cwd         => $oracle_base,
      environment => ["USER=${user}","ORACLE_HOME=${oracle_home}","ORACLE_BASE=${oracle_base}",],
      logoutput   => $logoutput,
    }
  }
}
