#
# ora_install::database_pluggable
#
# @summary Creates or delete a pluggable database.
# Under the hood, this defined type calls the `dbca` utility to create or delete a pluggable database in the specified source database.
# 
# Here is an example on how to create pluggable database:
# 
# ```puppet
# ora_install::database_pluggable{'pdb1':
#   ensure                   => 'present',
#   version                  => '12.1',
#   oracle_home_dir          => '/oracle/product/12.1/db',
#   user                     => 'oracle',
#   group                    => 'dba',
#   source_db                => 'orcl',
#   pdb_name                 => 'pdb1',
#   pdb_admin_username       => 'pdb_adm',
#   pdb_admin_password       => 'Welcome01',
#   pdb_datafile_destination => "/oracle/oradata/orcl/pdb1",
#   create_user_tablespace   => true,
#   logoutput                => true,
# }
# ```
# Here is an example on how to remove a pluggable database:
# 
# ```puppet
# ora_install::database_pluggable{'pdb1':
#   ensure                   => 'absent',
#   version                  => '12.1',
#   oracle_home_dir          => '/oracle/product/12.1/db',
#   user                     => 'oracle',
#   group                    => 'dba',
#   source_db                => 'orcl',
#   pdb_name                 => 'pdb1',
#   pdb_datafile_destination => "/oracle/oradata/orcl/pdb1",
#   logoutput                => true,
# }
# ```
#
# @param [Boolean] create_user_tablespace
#    Create a user tablespace in the PDB.
#    The default value is: `false`
#
# @param [Enum[present, absent]] ensure
#    State to obtain.
#    The ensure attribute can be one of two values:
#    - present
#    - absent
#    When you specify `present`, Puppet will make sure the resource is available with all specified options and properties.
#    When the resource is already available(installed), and all attributes are as the are specified, Puppet will do nothing.
#    When you specify `absent`, Puppet will remove the resource if it is available. If it is not installed, Puppet will do nothing.
#
# @param [String[1]] group
#    The os group to use for these Oracle puppet definitions.
#    The default value is: `dba`
#
# @param [Variant[Boolean, Enum['on_failure']]] logoutput
#    log the outputs of Puppet exec or not.
#    When you specify `true` Puppet will log all output of `exec` types.
#    Valid values are:
#    - `true`
#    - `false`
#    - `on_failure`
#
# @param [Stdlib::Absolutepath] oracle_home_dir
#    A directory to be used as Oracle home directory for this software.
#
# @param [Easy_type::Password] pdb_admin_password
#    Password for the admin user in the PDB.
#
# @param [String[1]] pdb_admin_username
#    Username for the admin user in the pluggable database.
#    The default value is: `pdb_adm`
#
# @param [String[1]] pdb_datafile_destination
#    The location where the PDB datafiles will be stored.
#
# @param [String[1]] pdb_name
#    Name of the pluggable database.
#
# @param [String[1]] source_db
#    The database name of the container(source) database.
#
# @param [String[1]] user
#    The user used for the specified installation.
#    The install class will not create the user for you. You must do that yourself.
#    The default value is: `oracle`
#
# @param [Enum['12.1', '12.2', '18.0', '19.0', '21.0']] version
#    The version that is installed in the used Oracle home.
#    Puppet uses this value to decide on version specific actions.
#
#
# See the file "LICENSE" for the full license governing this code.
#
define ora_install::database_pluggable (
  Enum[present,absent]                          $ensure                   = present,
  Enum['12.1','12.2', '18.0', '19.0', '21.0']   $version                  = '12.1',
  Stdlib::Absolutepath                          $oracle_home_dir          = undef,
  String[1]                                     $user                     = 'oracle',
  String[1]                                     $group                    = 'dba',
  String[1]                                     $source_db                = undef,
  String[1]                                     $pdb_name                 = undef,
  String[1]                                     $pdb_datafile_destination = undef,
  String[1]                                     $pdb_admin_username       = 'pdb_adm',
  Easy_type::Password                           $pdb_admin_password       = undef,
  Boolean                                       $create_user_tablespace   = true,
  Variant[Boolean,Enum['on_failure']]           $logoutput                = lookup({ name => 'logoutput', default_value => 'on_failure' }),
) {
  ora_install { $title: ensure => 'present' }
  # TODO: This will only create a pluggable database on filesystem because of the
  #       pdb_datafile_destination which is in the 'creates' of the exec. We need
  #       to find a way to create the database in ASM too.

  if ( $ensure == present ) {
    unless ( $pdb_admin_username ) { fail('You must specify a pdb_admin_username') }
    unless ( $pdb_admin_password ) { fail('You must specify a pdb_admin_password') }
  }

  $exec_path = '/usr/local/bin:/bin:/usr/bin:/usr/local/sbin:/usr/sbin:/sbin'

  easy_type::debug_evaluation() # Show local variable on extended debug

  if ( $ensure == present ) {
    $command = "${oracle_home_dir}/bin/dbca -silent -createPluggableDatabase -sourceDB ${source_db} -pdbName ${pdb_name} -createPDBFrom DEFAULT -pdbAdminUserName ${pdb_admin_username} -pdbAdminPassword ${pdb_admin_password.unwrap} -pdbDatafileDestination ${pdb_datafile_destination} -createUserTableSpace ${create_user_tablespace}"

    exec { "dbca pdb execute ${title}":
      command   => $command,
      timeout   => 0,
      path      => $exec_path,
      cwd       => $oracle_home_dir,
      user      => $user,
      group     => $group,
      creates   => $pdb_datafile_destination,
      logoutput => $logoutput,
    }
  } else {
    $command = "${oracle_home_dir}/bin/dbca -silent -deletePluggableDatabase -sourceDB ${source_db} -pdbName ${pdb_name}"

    exec { "dbca pdb execute ${title}":
      command   => $command,
      timeout   => 0,
      path      => $exec_path,
      cwd       => $oracle_home_dir,
      user      => $user,
      group     => $group,
      onlyif    => "ls ${pdb_datafile_destination}",
      logoutput => $logoutput,
    }

    -> file { $pdb_datafile_destination:
      ensure => absent,
      force  => true,
    }
  }
}
