#
# ora_install::dbactions
#
# @summary Defined type to start and stop a database.
# [DEPRECATED]
# 
# This is defined type to start or stop a database. Usage of this type is discouraged. Please use [db_control](/docs/ora_install/db_control.html) for this function.
# 
# Here is an example on how to use this:
# ```puppet
# ora_install::dbactions{ 'stop testDb':
#   db_name     => 'test',
#   oracle_home => '/oracle/product/11.2/db',
#   user        => 'oracle',
#   group       => 'dba',
#   action      => 'stop',
# }
# ```
#
# @param [Enum['running', 'start', 'stop', 'abort']] action
#    The action you want to take.
#    Valid values are:
#    - `start`
#    - `running`
#    - `stop`
#    - `abort`
#    Default value is: `start`
#
# @param [String[1]] db_name
#    The database you want to do the action on.
#
# @param [Enum['database', 'asm', 'grid']] db_type
#    The type of service you want to manage.
#    Valid values are:
#    - `database`
#    - `grid`
#    - `asm`
#
# @param [Optional[Stdlib::Absolutepath]] grid_home
#    The oracle home directory to use for the GRID software.
#
# @param [String[1]] group
#    The os group to use for these Oracle puppet definitions.
#    The default value is: `dba`
#
# @param [Stdlib::Absolutepath] oracle_home
#    A directory to be used as Oracle home directory for this software.
#
# @param [String[1]] user
#    The user used for the specified installation.
#    The install class will not create the user for you. You must do that yourself.
#    The default value is: `oracle`
#
#
# See the file "LICENSE" for the full license governing this code.
#
define ora_install::dbactions (
  Enum['database','asm','grid']          $db_type     = 'database',
  Stdlib::Absolutepath                   $oracle_home = undef,
  Optional[Stdlib::Absolutepath]         $grid_home   = undef,
  String[1]                              $user        = 'oracle',
  String[1]                              $group       = 'dba',
  Enum['running','start','stop','abort'] $action      = 'start',
  String[1]                              $db_name     = 'orcl',
) {
  easy_type::debug_evaluation() # Show local variable on extended debug

  ora_install { $title: ensure => 'present' }
  # TODO: Make this more logical. When specifying db_type = asm or grid it will
  #       stop ASM eventhough you specify a 'normal' database in db_name. I was
  #       thinking it would use srvctl to stop the database specified in db_name.
  #       It also does this from oracle_home, so why we have to specify the grid_home?

  if ( $db_type in ['grid','asm'] and !$grid_home ) { fail('You must specify grid_home if db_type is asm or grid') }

  if $db_type == 'database' {
    db_control { "instance control ${title}":
      ensure                  => $action,
      instance_name           => $db_name,
      oracle_product_home_dir => $oracle_home,
      os_user                 => $user,
    }
  } elsif $db_type in ['grid','asm'] {
    db_control { "instance control ${title}":
      ensure                  => $action,
      provider                => 'srvctl',
      instance_name           => $db_name,
      oracle_product_home_dir => $oracle_home,
      grid_product_home_dir   => $grid_home,
      os_user                 => $user,
      db_type                 => 'grid',
    }
  }
}
