#
# ora_install::fact_caching
#
# @summary This class ensure's that the ora_install facts that can be cached and are cached with the correct settings.
# You can change all TTL settings for the facts if an other setting is more fitting for your setup.
#
# @param [Boolean] enabled
#    When set to true, this enables facter caching for the ora_install module facts.
#    Beware that you need to have Facter version 4.1.0 or higher to enabled this. When you set this value to true, automatically TTL settings for most of the ora_install facts are set to a default value. For most installations, the default settings are fine.
#    When you set this value to `true`, the [facter setting `force-dot-resolution`](https://tickets.puppetlabs.com/browse/PUP-11089) is enabled. You have to ensure that all of your facts keep on working with this setting.
#
# @param [Optional[String[1]]] product_version_ttl
#    The Time to Live for the `ora_install_homes.
#    product_version` part of the `ora_install_homes` fact.
#
# @param [Optional[String[1]]] opatch_version_ttl
#    The Time to Live for the `ora_install_homes.
#    opatch_version` part of the `ora_install_homes` fact.
#
# @param [Optional[String[1]]] installed_patches_ttl
#    The Time to Live for the `ora_install_homes.
#    installed_patches` part of the `ora_install_homes` fact.
#
# @param [Optional[String[1]]] running_processes_ttl
#    The Time to Live for the `ora_install_homes.
#    running_processes` part of the `ora_install_homes` fact.
#
# @param [Optional[String[1]]] defined_sids_ttl
#    The Time to Live for the `ora_install_homes.
#    defined_sids` part of the `ora_install_homes` fact.
#
# @param [Optional[String[1]]] ora_install_inventory_loc_ttl
#    The Time to Live for the `ora_install_inventory_loc` fact.
#
#
# See the file "LICENSE" for the full license governing this code.
#
class ora_install::fact_caching (
  Optional[String[1]] $defined_sids_ttl,
  Boolean             $enabled,
  Optional[String[1]] $installed_patches_ttl,
  Optional[String[1]] $opatch_version_ttl,
  Optional[String[1]] $ora_install_inventory_loc_ttl,
  Optional[String[1]] $product_version_ttl,
  Optional[String[1]] $running_processes_ttl,
) {
  if $enabled {
    include ora_install::clear_caches
    #
    # If enabled, we need to enable the force-dot-resolution
    # setting for facter.
    #
    unless defined(Fact_setting['force-dot-resolution']) {
      fact_setting { 'force-dot-resolution':
        value => true,
      }
    }

    fact_config { 'ora_install_homes.product_version':
      ttl => $product_version_ttl,
    }

    fact_config { 'ora_install_homes.opatch_version':
      ttl => $opatch_version_ttl,
    }

    fact_config { 'ora_install_homes.installed_patches':
      ttl => $installed_patches_ttl,
    }

    fact_config { 'ora_install_homes.running_processes':
      ttl => $running_processes_ttl,
    }

    fact_config { 'ora_install_homes.defined_sids':
      ttl => $defined_sids_ttl,
    }

    fact_config { 'ora_install_inventory_loc':
      ttl => $ora_install_inventory_loc_ttl,
    }
  } else {
    #
    # On puppet versions older than V6, the required libs
    # for fact_config are not installed. Because it also doesn't
    # serve any purpose, we skip this part on those versions.
    #
    if versioncmp($clientversion, '6.0.0') != -1 {
      #
      # Disable fact caching for all ora_install facts
      #
      $ora_install_facts = [
        'ora_install_homes.product_version',
        'ora_install_homes.opatch_version',
        'ora_install_homes.installed_patches',
        'ora_install_homes.defined_sids',
        'ora_install_homes.running_processes',
        'ora_install_inventory_loc',
      ]

      fact_config { $ora_install_facts:
        ttl => absent,
      }
    }
  }
}
