#
# ora_install::goldengate
#
# @summary Installs the Oracle goldengate software.
# This module supports the following versions:
# 
# - 12.1.2
# - 12.2.1
# - 18.1.0
# - 19.1.0
# - 21.3.0
# 
# Here is an example on how to use this:
# 
# ```puppet
# ora_install::goldengate{ 'ggate12.1.2':
#   version                  => '12.1.2',
#   file                     => '121200_fbo_ggs_Linux_x64_shiphome.zip',
#   database_type            => 'Oracle',
#   database_version         => 'ORA11g',
#   database_home            => '/oracle/product/12.1/db',
#   oracle_base              => '/oracle',
#   goldengate_home          => "/oracle/product/12.1/ggate",
#   manager_port             => 16000,
#   user                     => 'ggate',
#   group                    => 'dba',
#   group_install            => 'oinstall',
#   download_dir             => '/install',
#   puppet_download_mnt_point => puppet://modules/software,
# }
# ```
#
# @param [Stdlib::Absolutepath] database_home
#    The home directory of the database.
#
# @param [Enum['Oracle']] database_type
#    The type of database.
#    The default value is: `Oracle`
#
# @param [Ora_install::DatabaseVersion] database_version
#    The version of the database.
#    Valid values are:
#    - `ORA11g`
#    - `ORA12c`
#    - `ORA18c`
#    - `ORA19c`
#    Default version is `ORA11g`
#
# @param [Stdlib::Absolutepath] download_dir
#    The directory where the Puppet software puts all downloaded files.
#    Before Puppet can actually use remote files, they must be downloaded first. Puppet uses this directory to put all files in.
#    The default value is: `/install`
#
# @param [String[1]] file
#    The file containing the software.
#
# @param [Stdlib::Absolutepath] goldengate_home
#    The home to use for the goldengate software.
#
# @param [String[1]] group
#    The os group to use for these Oracle puppet definitions.
#    The default value is: `dba`
#
# @param [String[1]] group_install
#    The os group to use for installation.
#    The default value is: `oinstall`
#
# @param [Variant[Boolean, Enum['on_failure']]] logoutput
#    log the outputs of Puppet exec or not.
#    When you specify `true` Puppet will log all output of `exec` types.
#    Valid values are:
#    - `true`
#    - `false`
#    - `on_failure`
#
# @param [Integer] manager_port
#    The IP port number of the manager.
#    This is a required value.
#
# @param [Stdlib::Absolutepath] oracle_base
#    A directory to use as Oracle base directory.
#
# @param [Optional[String[1]]] puppet_download_mnt_point
#    The base path of all remote files for the defined type or class.
#    The default value is: `puppet:///modules/ora_install`
#
# @param [Stdlib::Absolutepath] temp_dir
#    Directory to use for temporary files.
#
# @param [String[1]] umask
#    The umask to use for commands that run in the context of either the grid user or the oracle user.
#    The default is `0022`
#
# @param [String[1]] user
#    The user used for the specified installation.
#    The install class will not create the user for you. You must do that yourself.
#    The default value is: `oracle`
#
# @param [Enum['12.1.2', '12.2.1', '12.3.0', '18.1.0', '19.1.0', '21.3.0']] version
#    The version that is installed in the used Oracle home.
#    Puppet uses this value to decide on version specific actions.
#
# @param [Boolean] allow_insecure
#    When set to `true` Ignore HTTPS certificate errors
#    (default: false)
#
# @param [Optional[Stdlib::Absolutepath]] ora_inventory_dir
#    The directory that contains the oracle inventory.
#    The default value is: `/oracle_base/oraInventory`
#
#
# See the file "LICENSE" for the full license governing this code.
#
define ora_install::goldengate (
  # lint:ignore:strict_indent
  Enum['12.1.2','12.2.1', '12.3.0','18.1.0', '19.1.0', '21.3.0']
                                      $version                   = '12.3.0',
  String[1]                           $file                      = undef,
  Enum['Oracle']                      $database_type             = 'Oracle',
  Ora_install::DatabaseVersion        $database_version          = 'ORA11g',
  Stdlib::Absolutepath                $database_home             = undef,
  Stdlib::Absolutepath                $oracle_base               = undef,
  Stdlib::Absolutepath                $goldengate_home           = undef,
  Integer                             $manager_port              = undef,
  String[1]                           $umask                     = '0022',
  String[1]                           $user                      = 'ggate',
  String[1]                           $group                     = 'dba',
  String[1]                           $group_install             = 'oinstall',
  Stdlib::Absolutepath                $download_dir              = '/install',
  Stdlib::Absolutepath                $temp_dir                  = '/tmp',
  Optional[String[1]]                 $puppet_download_mnt_point = undef,
  Optional[Stdlib::Absolutepath]      $ora_inventory_dir         = undef,
  Variant[Boolean,Enum['on_failure']] $logoutput                 = lookup({ name => 'logoutput', default_value => 'on_failure' }),
  Boolean                             $allow_insecure            = lookup({ name => 'allow_insecure', default_value => false }),
  # lint:endignore:strict_indent
) {
  ora_install { $title: ensure => 'present' }

  $version_elements = split($version,'\.')
  $major_version = $version_elements[0]

  if ( $ora_inventory_dir ) {
    $ora_inventory = "${ora_inventory_dir}/oraInventory"
  } else {
    $ora_inventory = "${oracle_base}/oraInventory"
  }

  $exec_path = '/usr/local/bin:/bin:/usr/bin:/usr/local/sbin:/usr/sbin:/sbin'

  easy_type::debug_evaluation() # Show local variable on extended debug

  db_directory_structure { "oracle goldengate structure ${version}":
    ensure            => present,
    oracle_base_dir   => $oracle_base,
    ora_inventory_dir => $ora_inventory,
    download_dir      => $download_dir,
    os_user           => $user,
    os_group          => $group_install,
  }

  unless ( ora_install::oracle_exists( $goldengate_home ) ) {
    if ( $puppet_download_mnt_point ) {
      $mount_point     = $puppet_download_mnt_point
    } else {
      $mount_point     = 'puppet:///modules/ora_install/'
    }

    if ( $mount_point == $download_dir ) {
      $archive_cleanup = false
    } else {
      $archive_cleanup = true
    }
    if $version == '21.3.0' {
      $ggate_install_dir = 'fbo_ggs_Linux_x64_Oracle_shiphome'
    } else {
      $ggate_install_dir = 'fbo_ggs_Linux_x64_shiphome'
    }

    archive { "${download_dir}/${file}":
      ensure         => present,
      allow_insecure => $allow_insecure,
      cleanup        => $archive_cleanup,
      creates        => "${download_dir}/${ggate_install_dir}",
      extract        => true,
      extract_path   => $download_dir,
      group          => $group,
      require        => Db_directory_structure["oracle goldengate structure ${version}"],
      source         => "${mount_point}/${file}",
      temp_dir       => $temp_dir,
      user           => $user,
    }

    file { "${download_dir}/oggcore.rsp":
      content => epp("ora_install/oggcore_${major_version}.rsp.epp",
        {
          'database_version' => $database_version,
          'goldengate_home'  => $goldengate_home,
          'manager_port'     => $manager_port,
          'database_home'    => $database_home,
          'ora_inventory'    => $ora_inventory,
          'group_install'    => $group_install,
      }),
      owner   => $user,
      group   => $group,
      require => Db_directory_structure["oracle goldengate structure ${version}"],
    }

    ora_install::utils::dborainst { "ggate orainst ${version}":
      ora_inventory_dir => $ora_inventory,
      os_group          => $group_install,
      os_user           => $user,
    }

    exec { 'install oracle goldengate':
      command     => "/bin/sh -c 'unset DISPLAY;${download_dir}/${ggate_install_dir}/Disk1/runInstaller -silent -waitforcompletion -responseFile ${download_dir}/oggcore.rsp'",
      require     => [
        File["${download_dir}/oggcore.rsp"],
        Ora_install::Utils::Dborainst["ggate orainst ${version}"
        ],
      Archive["${download_dir}/${file}"]],
      creates     => $goldengate_home,
      environment => ["TMP=${temp_dir}","TMPDIR=${temp_dir}","TEMP=${temp_dir}"],
      timeout     => 0,
      path        => $exec_path,
      logoutput   => $logoutput,
      user        => $user,
      umask       => $umask,
      group       => $group_install,
      returns     => [3,0],
    }

    # cleanup

    cleanup { "remove oracle goldengate extract folder ${download_dir}/${ggate_install_dir}":
      file_name => "${download_dir}/${ggate_install_dir}",
    }
    if ( $archive_cleanup ) {
      cleanup { "${download_dir}/${file}":
      }
    }
  }
}
