#
# ora_install::installasm
#
# @summary Installs and configures Oracle grid.
# This defined class supports the following versions of grid:
# - 11.2.0.4
# - 12.1.0.1
# - 12.1.0.2
# - 12.2.0.1
# - 18.0.0.0
# - 19.0.0.0 (and all sub versions)
# - 21.0.0.0 (and all sub versions)
# 
# 
# Here is an example on how to use this:
# 
# ```puppet
# ora_install::installasm{ 'db_linux-x64':
#   version                   => hiera('db_version'),
#   file                      => hiera('asm_file'),
#   grid_type                 => 'HA_CONFIG',
#   grid_base                 => hiera('grid_base_dir'),
#   grid_home                 => hiera('grid_home_dir'),
#   ora_inventory_dir         => hiera('oraInventory_dir'),
#   user_base_dir             => '/home',
#   user                      => hiera('grid_os_user'),
#   group                     => 'asmdba',
#   group_install             => 'oinstall',
#   group_oper                => 'asmoper',
#   group_asm                 => 'asmadmin',
#   sys_asm_password          => 'Welcome01',
#   asm_monitor_password      => 'Welcome01',
#   asm_diskgroup             => 'DATA',
#   disk_discovery_string     => "/nfs_client/asm*",
#   disks                     => "/nfs_client/asm_sda_nfs_b1,/nfs_client/asm_sda_nfs_b2",
#   # disk_discovery_string   => "ORCL:*",
#   # disks                   => "ORCL:DISK1,ORCL:DISK2",
#   disk_redundancy           => "EXTERNAL",
#   download_dir              => hiera('oracle_download_dir'),
#   remote_file               => false,
#   puppet_download_mnt_point => hiera('oracle_source'),
# }
# ```
#
# @param [String[1]] asm_diskgroup
#    The name of the ASM diskgroup to use.
#    `DATA`
#
# @param [Easy_type::Password] asm_monitor_password
#    The password for the ASMSNMP user for monitoring in EM.
#
# @param [Optional[String[1]]] cluster_name
#    The name of the cluster.
#
# @param [Optional[String[1]]] cluster_nodes
#    The names of the nodes in the RAC cluster.
#    Valid values are:
#    - `node1:node1-vip,node2:node2-vip`                     (version >= 11 <= 12.1)
#    - `node1:node1-vip:HUB,node2:node2-vip:LEAF`            (version >= 12.1 Flex Cluster)
#    - `node1,node2`                                         (version = 12.2 Application Cluster)
#    - `node1:node1-vip:HUB:site1,node2:node2-vip:HUB:site2` (version = 12.2 Extended Cluster)
#
# @param [Boolean] configure_afd
#    Specify whether or not to configure ASM Filter Driver instead of ASMLib.
#    The default value is: `false`
#
# @param [Enum['1', '2', '4', '8', '16', '32', '64']] disk_au_size
#    The disk_au size to use for the ASM disks.
#    **Attention** this is a String value not an Integer.
#    The default value is: `1`
#
# @param [Optional[String[1]]] disk_discovery_string
#    The search string to use for discovering disks for ASM.
#
# @param [Enum['NORMAL', 'HIGH', 'EXTERNAL']] disk_redundancy
#    The type of redundancy to use.
#    Valid values are:
#    - `NORMAL`
#    - `HIGH`
#    - `EXTERNAL`
#    The default value is: `NORMAL`
#
# @param [Optional[String[1]]] disks_failgroup_names
#    A comma seperated list of device and failure group name.
#    Valid values are:
#    - `/dev/sdb,CRSFG1,/dev/sdc,CRSFG2,/dev/sdd,CRSFG3`                                 (NORMAL redundancy)
#    - `/dev/sdb,,/dev/sdc,,/dev/sdd,,/dev/sde,`                                         (EXTERNAL redundancy)
#    - `/dev/sdb,CRSFG1,/dev/sdc,CRSFG2,/dev/sdd,CRSFG3,/dev/sde,CRSFG4,/dev/sdf,CRSFG5` (HIGH redundancy)
#
# @param [Boolean] bash_profile
#    Create a bash profile for the specified user or not.
#    Valid values are `true` and `false`.
#    When you specify a `true` for the parameter, Puppet will create a standard bash profile for the specified user. The bash profile will be placed in a directory named `user_base_dir/user`.
#    ```puppet
#    ora_install::client { 'Oracle client':
#      ...
#      bash_profile  => true,
#      user          => 'oracle',
#      user_base_dir => '/home',
#      ...
#    }
#    ```
#    This snippet will create a bash profile called `/home/oracle/.bash_profile`.
#
# @param [String] bash_additions
#    The text to add at the end of the bash_profile.
#    This parameter will only be used when you have specified `true` for the parameter `bash_profile`
#    The default value is an empty string.
#
# @param [Optional[String[1]]] disks
#    List of disks to create a ASM DiskGroup.
#
# @param [Stdlib::Absolutepath] download_dir
#    The directory where the Puppet software puts all downloaded files.
#    Before Puppet can actually use remote files, they must be downloaded first. Puppet uses this directory to put all files in.
#    The default value is: `/install`
#
# @param [String[1]] file
#    The source file to use.
#
# @param [Boolean] gimr_configure
#    Specify whether or not to configure GIMR database (MGMTDB), 19c and higher only.
#    The default value is: `false`
#
# @param [Enum['1', '2', '4', '8', '16', '32', '64']] gimr_dg_ausize
#    The disk_au size to use for the for the GIMR database (MGMTDB) diskgroup.
#    **Attention** this is a String value not an Integer.
#    The default value is: `1`
#
# @param [Optional[String[1]]] gimr_dg_disks
#    List of disks to create the GIMR database (MGMTDB).
#
# @param [Optional[String[1]]] gimr_dg_disks_failgroup_names
#    A comma seperated list of device and failure group name for the GIMR database (MGMTDB).
#    Valid values are:
#    - `/dev/sdb,GIMRFG1,/dev/sdc,GIMRFG2,/dev/sdd,GIMRFG3`                                   (NORMAL redundancy)
#    - `/dev/sdb,,/dev/sdc,,/dev/sdd,,/dev/sde,`                                              (EXTERNAL redundancy)
#    - `/dev/sdb,GIMRFG1,/dev/sdc,GIMRFG2,/dev/sdd,GIMRFG3,/dev/sde,GIMRFG4,/dev/sdf,GIMRFG5` (HIGH redundancy)
#
# @param [Optional[String[1]]] gimr_dg_fg_names
#    Failure Groups for the GIMR storage data ASM disk group.
#
# @param [Optional[String[1]]] gimr_dg_name
#    Specify the name of the diskgroup for the GIMR database (MGMTDB), 12.
#    2 and up only.
#
# @param [Optional[Enum['NORMAL', 'HIGH', 'EXTERNAL']]] gimr_dg_redundancy
#    The type of redundancy to use for the GIMR database (MGMTDB) diskgroup.
#    Valid values are:
#    - `NORMAL`
#    - `HIGH`
#    - `EXTERNAL`
#
# @param [Stdlib::Absolutepath] grid_base
#    The directory to use as grid base.
#
# @param [Stdlib::Absolutepath] grid_home
#    The directory to use as grid home.
#
# @param [Enum['CRS_CONFIG', 'HA_CONFIG', 'UPGRADE', 'CRS_SWONLY', 'HA_SWONLY']] grid_type
#    The type of grid.
#    Valid values are:
#    - `HA_CONFIG`
#    - `CRS_CONFIG`
#    - `HA_SWONLY`   (versions > 11)
#    - `UPGRADE`
#    - `CRS_SWONLY`
#    The default value is: `HA_CONFIG`
#
# @param [String[1]] group
#    The os group to use for these Oracle puppet definitions.
#    The default value is: `dba`
#
# @param [String[1]] group_asm
#    The OS group to use.
#    The default value is: `asmdba`
#
# @param [String[1]] group_install
#    The os group to use for installation.
#    The default value is: `oinstall`
#
# @param [String[1]] group_oper
#    The OS group to allow ASM oper operations.
#    The default value is: `asmoper`
#
# @param [Variant[Boolean, Enum['on_failure']]] logoutput
#    log the outputs of Puppet exec or not.
#    When you specify `true` Puppet will log all output of `exec` types.
#    Valid values are:
#    - `true`
#    - `false`
#    - `on_failure`
#
# @param [Optional[String[1]]] network_interface_list
#    The list of interfaces to use for RAC.
#    The value should be a comma separated strings where each string is as shown below
#    ```
#    InterfaceName:SubnetAddress:InterfaceType
#    ```
#    where InterfaceType can be either "1", "2", or "3" (1 indicates public, 2 indicates private, and 3 indicates the interface is not used)
#    An example on how to use this:
#    ```puppet
#    installasm{'asm12':
#      ...
#      network_interface_list => 'eth0:140.87.24.0:1,eth1:10.2.1.0:2,eth2:140.87.52.0:3'
#      ...
#    }
#    ```
#
# @param [Optional[Stdlib::Absolutepath]] ora_inventory_dir
#    The directory that contains the oracle inventory.
#    The default value is: `/oracle_base/oraInventory`
#
# @param [Optional[String[1]]] puppet_download_mnt_point
#    The base path of all remote files for the defined type or class.
#    The default value is: `puppet:///modules/ora_install`
#
# @param [Optional[String[1]]] scan_name
#    The name to use for the SCAN service.
#
# @param [Optional[Integer]] scan_port
#    The IP portnumber to use for the SCAN service.
#
# @param [Boolean] stand_alone
#    Configuring Grid Infrastructure for a Stand-Alone Server.
#    The default value is: `true`
#
# @param [Optional[Enum['FLEX_ASM_STORAGE', 'CLIENT_ASM_STORAGE', 'LOCAL_ASM_STORAGE', 'FILE_SYSTEM_STORAGE', 'ASM_STORAGE']]] storage_option
#    The type of storage to use.
#    Valid values are:
#    - `ASM_STORAGE`          (versions = 11)
#    - `FILE_SYSTEM_STORAGE`  (versions <= 12.1)
#    - `LOCAL_ASM_STORAGE`    (versions >= 12.1)
#    - `CLIENT_ASM_STORAGE`   (versions >= 12.2)
#    - `FLEX_ASM_STORAGE`     (versions >= 12.1)
#
# @param [Easy_type::Password] sys_asm_password
#    The password to use for the SYSASM user.
#
# @param [Stdlib::Absolutepath] temp_dir
#    Directory to use for temporary files.
#
# @param [String[1]] user
#    The user used for the specified installation.
#    The install class will not create the user for you. You must do that yourself.
#    The default value is: `oracle`
#
# @param [String[1]] umask
#    The umask to use for commands that run in the context of either the grid user or the oracle user.
#    The default is `0022`
#
# @param [Stdlib::Absolutepath] user_base_dir
#    The directory to use as base directory for the users.
#
# @param [Ora_install::Version] version
#    The version that is installed in the used Oracle home.
#    Puppet uses this value to decide on version specific actions.
#
# @param [Boolean] zip_extract
#    The specified source file is a zip file that needs te be extracted.
#    When you specify a value of false, the source attribute must contain a reference to a directory instead of a zip file.
#    The default value is: `true`
#
# @param [Enum['ALL', 'EXTRACT', 'SETUP']] install_task
#    Kind of install task you want the class to do.
#
# @param [Boolean] gimr_dg
#    Specify whether or not to configure a seperate diskgroup for the GIMR database (MGMTDB), 12.
#    2 and up only.
#    The default value is: `false`
#
# @param [Boolean] allow_insecure
#    When set to `true` Ignore HTTPS certificate errors
#    (default: false)
#
#
# See the file "LICENSE" for the full license governing this code.
#
define ora_install::installasm (
  # lint:ignore:strict_indent
  Easy_type::Password                   $sys_asm_password,
  Easy_type::Password                   $asm_monitor_password,
  Ora_install::Version                  $version                       = undef,
  String[1]                             $file                          = undef,
  Enum['CRS_CONFIG','HA_CONFIG','UPGRADE','CRS_SWONLY','HA_SWONLY']
                                        $grid_type                     = 'HA_CONFIG',
  Boolean                               $stand_alone                   = true, # in case of 'CRS_SWONLY' and used as stand alone or in RAC
  Stdlib::Absolutepath                  $grid_base                     = undef,
  Stdlib::Absolutepath                  $grid_home                     = undef,
  Optional[Stdlib::Absolutepath]        $ora_inventory_dir             = undef,
  Boolean                               $bash_profile                  = true,
  String                                $bash_additions                = '',
  String[1]                             $user                          = 'grid',
  String[1]                             $umask                         = '0022',
  Stdlib::Absolutepath                  $user_base_dir                 = '/home',
  String[1]                             $group                         = 'asmdba',
  String[1]                             $group_install                 = 'oinstall',
  String[1]                             $group_oper                    = 'asmoper',
  String[1]                             $group_asm                     = 'asmadmin',
  String[1]                             $asm_diskgroup                 = 'DATA',
  Optional[String[1]]                   $disk_discovery_string         = undef,
  Enum['NORMAL','HIGH','EXTERNAL']      $disk_redundancy               = 'NORMAL',
  Enum['1','2','4','8','16','32','64']  $disk_au_size                  = '1',
  Optional[String[1]]                   $disks                         = undef,
  Optional[String[1]]                   $disks_failgroup_names         = undef,
  Boolean                               $configure_afd                 = false,
  Stdlib::Absolutepath                  $download_dir                  = '/install',
  Stdlib::Absolutepath                  $temp_dir                      = '/tmp',
  Boolean                               $zip_extract                   = true,
  Enum['ALL','EXTRACT','SETUP']         $install_task                  = 'ALL',
  Optional[String[1]]                   $puppet_download_mnt_point     = undef,
  Optional[String[1]]                   $cluster_name                  = undef,
  Optional[String[1]]                   $scan_name                     = undef,
  Optional[Integer]                     $scan_port                     = undef,
  Optional[String[1]]                   $cluster_nodes                 = undef,
  Optional[String[1]]                   $network_interface_list        = undef,
  Optional[Enum['FLEX_ASM_STORAGE','CLIENT_ASM_STORAGE','LOCAL_ASM_STORAGE','FILE_SYSTEM_STORAGE','ASM_STORAGE']]
                                        $storage_option                = undef,
  Boolean                               $gimr_configure                = false,
  Boolean                               $gimr_dg                       = false,
  Optional[String[1]]                   $gimr_dg_name                  = undef,
  Optional[Enum['NORMAL','HIGH','EXTERNAL']]
                                        $gimr_dg_redundancy            = undef,
  Enum['1','2','4','8','16','32','64']
                                        $gimr_dg_ausize                = '1',
  Optional[String[1]]                   $gimr_dg_fg_names              = undef,
  Optional[String[1]]                   $gimr_dg_disks_failgroup_names = undef,
  Optional[String[1]]                   $gimr_dg_disks                 = undef,
  Variant[Boolean,Enum['on_failure']]   $logoutput                     = lookup({ name => 'logoutput', default_value => 'on_failure' }),
  Boolean                               $allow_insecure                = lookup({ name => 'allow_insecure', default_value => false }),
  # lint:endignore:strict_indent
) {
  ora_install { $title: ensure => 'present' }

  $file_without_ext = regsubst($file, '(.+?)(\.zip*$|$)', '\1')

  if($cluster_name) { # We've got a RAC cluster. Check the cluster specific parameters
    unless ( $scan_name ) { fail('You must specify scan_name if cluster_name is defined') }
    unless ( $scan_port ) { fail('You must specify scan_port if cluster_name is defined') }
    unless ( $cluster_nodes ) { fail('You must specify cluster_nodes if cluster_name is defined') }
    unless ( $network_interface_list ) { fail('You must specify network_interface_list if cluster_name is defined') }
    unless ( $storage_option ) { fail('You must specify storage_option if cluster_name is defined') }
  }

  unless ( $facts['kernel'] in ['Linux','SunOS']) {
    fail('Unrecognized operating system, please use it on a Linux or SunOS host')
  } else {
    if ( $grid_type == 'HA_SWONLY' ) {
      fail('grid_type = HA_SWONLY is only supported on Windows')
    }
  }

  case $version {
    '11.2.0.4': {
      unless ( $storage_option in ['ASM_STORAGE','FILE_SYSTEM_STORAGE',undef]) { fail("storage_option should be defined as ASM_STORAGE or FILE_SYSTEM_STORAGE for version ${version}") }
      unless ( $grid_type in ['CRS_CONFIG','HA_CONFIG','UPGRADE','CRS_SWONLY']) { fail("grid_type should be defined as CRS_CONFIG, HA_CONFIG, UPGRADE or CRS_SWONLY for version ${version}") }
      $files = [$file]
      $extract_staged = true
      $ohas_service_template = 'ora_install/ohas.service.epp'
      $install_command = "${download_dir}/${file_without_ext}/grid/runInstaller -silent -waitforcompletion -ignoreSysPrereqs -ignorePrereq -responseFile ${download_dir}/grid_install_${version}.rsp"
      $install_creates = $grid_home
      $run_grid_user_config = false
    }
    '12.1.0.1', '12.1.0.2': {
      unless ( $storage_option in ['LOCAL_ASM_STORAGE','FLEX_ASM_STORAGE','FILE_SYSTEM_STORAGE',undef]) { fail("storage_option should be defined as LOCAL_ASM_STORAGE, FLEX_ASM_STORAGE or FILE_SYSTEM_STORAGE for version ${version}") }
      unless ( $grid_type in ['CRS_CONFIG','HA_CONFIG','UPGRADE','CRS_SWONLY','HA_SWONLY']) { fail("grid_type should be defined as CRS_CONFIG, HA_CONFIG, UPGRADE, CRS_SWONLY or HA_SWONLY for version ${version}") }
      $files = ["${file}_1of2.zip", "${file}_2of2.zip"]
      $extract_staged = true
      $ohas_service_template = undef
      $install_command = "${download_dir}/${file_without_ext}/grid/runInstaller -silent -waitforcompletion -ignoreSysPrereqs -ignorePrereq -responseFile ${download_dir}/grid_install_${version}.rsp"
      $install_creates = $grid_home
      $run_grid_user_config = false
    }
    '12.2.0.1': {
      unless ( $storage_option in ['FLEX_ASM_STORAGE','CLIENT_ASM_STORAGE',undef]) { fail("storage_option should be defined as FLEX_ASM_STORAGE or CLIENT_ASM_STORAGE for version ${version}") }
      unless ( $grid_type in ['CRS_CONFIG','HA_CONFIG','UPGRADE','CRS_SWONLY','HA_SWONLY']) { fail("grid_type should be defined as CRS_CONFIG, HA_CONFIG, UPGRADE, CRS_SWONLY or HA_SWONLY for version ${version}") }
      $files = ["${file}.zip"]
      $extract_staged = false
      $ohas_service_template = undef
      $install_command = "${grid_home}/gridSetup.sh -silent -waitForCompletion -skipPrereqs -responseFile ${download_dir}/grid_install_${version}.rsp"
      $install_creates = "${grid_home}/install/config/oracle.rdbms.util.xml.ouibak"
      $run_grid_user_config = true
    }
    '18.0.0.0', '19.0.0.0', '21.0.0.0': {
      unless ( $storage_option in ['FLEX_ASM_STORAGE','CLIENT_ASM_STORAGE',undef]) { fail("storage_option should be defined as FLEX_ASM_STORAGE or CLIENT_ASM_STORAGE for version ${version}") }
      unless ( $grid_type in ['CRS_CONFIG','HA_CONFIG','UPGRADE','CRS_SWONLY','HA_SWONLY']) { fail("grid_type should be defined as CRS_CONFIG, HA_CONFIG, UPGRADE, CRS_SWONLY or HA_SWONLY for version ${version}") }
      $files = ["${file}.zip"]
      $extract_staged = false
      $ohas_service_template = undef
      $install_command = "${grid_home}/gridSetup.sh -silent -waitForCompletion -skipPrereqs -responseFile ${download_dir}/grid_install_${version}.rsp"
      $install_creates = "${grid_home}/install/config/oracle.rdbms.util.xml.ouibak"
      $run_grid_user_config = true
    }
    default: {
      fail "Version ${version} is not supported."
    }
  }

  easy_type::debug_evaluation() # Show local variable on extended debug

  if ( $bash_profile ) {
    unless ( defined(File["${user_base_dir}/${user}/.bash_profile"]) ) {
      file { "${user_base_dir}/${user}/.bash_profile":
        ensure  => file,
        content => regsubst(epp('ora_install/grid_bash_profile.epp',
            {
              'grid_home'      => $grid_home,
              'grid_base'      => $grid_base,
              'grid_type'      => $grid_type,
              'bash_additions' => $bash_additions,
        }), '\r\n', "\n", 'EMG'),
        mode    => '0775',
        owner   => $user,
        group   => $group,
      }
    }
  }

  if ( $ora_inventory_dir ) {
    $ora_inventory = "${ora_inventory_dir}/oraInventory"
  } else {
    $ora_inventory = "${grid_base}/oraInventory"
  }

  db_directory_structure { "Install task ${install_task} grid structure ${version}":
    ensure            => present,
    oracle_base_dir   => $grid_base,
    ora_inventory_dir => $ora_inventory,
    download_dir      => $download_dir,
    os_user           => $user,
    os_group          => $group_install,
  }

  unless ( ora_install::oracle_exists( $grid_home ) ) {
    $exec_path     = '/usr/local/bin:/bin:/usr/bin:/usr/local/sbin:/usr/sbin:/sbin:'

    if ( $puppet_download_mnt_point ) {
      $mount_point     = $puppet_download_mnt_point
    } else {
      $mount_point     = 'puppet:///modules/ora_install/'
    }

    if ( $mount_point == $download_dir ) {
      $archive_cleanup = false
    } else {
      $archive_cleanup = true
    }

    if ( $install_task == 'ALL' ) or ( $install_task == 'EXTRACT' ) {
      if ( $zip_extract ) {
        if ( $extract_staged ) {
          file { "${download_dir}/${file_without_ext}":
            ensure  => directory,
            owner   => $user,
            group   => $group,
            mode    => '0755',
            require => Db_directory_structure["Install task ${install_task} grid structure ${version}"],
          }

          $files.each | $filename | {
            archive { "${download_dir}/${filename}":
              ensure         => present,
              allow_insecure => $allow_insecure,
              cleanup        => $archive_cleanup,
              creates        => "${download_dir}/${file}/installasm",
              extract        => true,
              extract_path   => "${download_dir}/${file_without_ext}",
              group          => $group_install,
              require        => File["${download_dir}/${file_without_ext}"],
              source         => "${mount_point}/${filename}",
              temp_dir       => $temp_dir,
              user           => $user,
            }
          }
        } else {
          file { $grid_home:
            ensure  => directory,
            recurse => false,
            replace => false,
            mode    => '0775',
            owner   => $user,
            group   => $group_install,
            require => Db_directory_structure["Install task ${install_task} grid structure ${version}"],
          }

          $files.each | $filename | {
            archive { "${download_dir}/${filename}":
              ensure         => present,
              allow_insecure => $allow_insecure,
              cleanup        => $archive_cleanup,
              creates        => "${grid_home}/installasm",
              extract        => true,
              extract_path   => $grid_home,
              group          => $group_install,
              require        => File[$grid_home],
              source         => "${mount_point}/${filename}",
              temp_dir       => $temp_dir,
              user           => $user,
            }
          }
        }
      }
    }

    if ( $install_task == 'ALL' ) or ( $install_task == 'SETUP' ) {
      ora_install::utils::dborainst { "grid orainst ${version}":
        ora_inventory_dir => $ora_inventory,
        os_group          => $group_install,
        os_user           => $user,
      }

      unless ( defined(File["${download_dir}/grid_install_${version}.rsp"]) ) {
        file { "${download_dir}/grid_install_${version}.rsp":
          ensure  => file,
          content => epp("ora_install/grid_install_${version}.rsp.epp",
            {
              'ora_inventory'                 => $ora_inventory,
              'grid_type'                     => $grid_type,
              'grid_base'                     => $grid_base,
              'grid_home'                     => $grid_home,
              'group'                         => $group,
              'group_oper'                    => $group_oper,
              'group_asm'                     => $group_asm,
              'scan_name'                     => $scan_name,
              'scan_port'                     => $scan_port,
              'cluster_name'                  => $cluster_name,
              'cluster_nodes'                 => $cluster_nodes,
              'network_interface_list'        => $network_interface_list,
              'gimr_dg'                       => $gimr_dg,
              'storage_option'                => $storage_option,
              'sys_asm_password'              => $sys_asm_password,
              'asm_diskgroup'                 => $asm_diskgroup,
              'disk_redundancy'               => $disk_redundancy,
              'disk_au_size'                  => $disk_au_size,
              'disks_failgroup_names'         => $disks_failgroup_names,
              'disks'                         => $disks,
              'disk_discovery_string'         => $disk_discovery_string,
              'asm_monitor_password'          => $asm_monitor_password,
              'gimr_dg_name'                  => $gimr_dg_name,
              'gimr_dg_redundancy'            => $gimr_dg_redundancy,
              'gimr_dg_ausize'                => $gimr_dg_ausize,
              'gimr_dg_fg_names'              => $gimr_dg_fg_names,
              'gimr_dg_disks_failgroup_names' => $gimr_dg_disks_failgroup_names,
              'gimr_dg_disks'                 => $gimr_dg_disks,
              'configure_afd'                 => $configure_afd,
              'gimr_configure'                => $gimr_configure,
          }),
          mode    => '0770',
          owner   => $user,
          group   => $group,
          require => [
            Ora_install::Utils::Dborainst["grid orainst ${version}"],
            Db_directory_structure["Install task ${install_task} grid structure ${version}"],
          ],
        }
      }

      exec { "install oracle grid ${title}":
        command     => "/bin/sh -c 'unset DISPLAY;${install_command}'",
        creates     => $install_creates,
        environment => ["USER=${user}","LOGNAME=${user}","TMP=${temp_dir}","TMPDIR=${temp_dir}","TEMP=${temp_dir}"],
        timeout     => 0,
        returns     => [6,0],
        path        => $exec_path,
        user        => $user,
        group       => $group_install,
        cwd         => $grid_base,
        umask       => $umask,
        logoutput   => $logoutput,
        require     => [
          Ora_install::Utils::Dborainst["grid orainst ${version}"],
          File["${download_dir}/grid_install_${version}.rsp"]
        ],
      }

      # because of RHEL7 uses systemd we need to create the service differently
      if ( $facts['os']['family'] == 'RedHat' ) and ( $facts['os']['release']['major'] == '7' ) {
        if ( $ohas_service_template ) {
          file { '/etc/systemd/system/ohas.service':
            ensure  => 'file',
            content => epp($ohas_service_template, {}),
            mode    => '0644',
            require => Exec["install oracle grid ${title}"],
          }

          -> exec { 'daemon-reload for ohas':
            command => '/bin/systemctl daemon-reload',
          }

          -> service { 'ohas.service':
            ensure => running,
            enable => true,
            before => Exec["run root.sh grid script ${title}"],
          }
        }
      }

      exec { "run root.sh grid script ${title}":
        timeout   => 0,
        command   => "${grid_home}/root.sh",
        user      => 'root',
        group     => 'root',
        path      => $exec_path,
        cwd       => $grid_base,
        logoutput => $logoutput,
        require   => Exec["install oracle grid ${title}"],
      }

      # cleanup
      if ( $zip_extract ) {
        cleanup { "remove oracle asm extract folder ${title}":
          file_name => "${download_dir}/${file_without_ext}",
        }
        if ( $archive_cleanup ) {
          $files.each | $filename | {
            cleanup { "${download_dir}/${filename}": }
          }
        }
      }

      unless ( $grid_type in ['CRS_SWONLY', 'HA_SWONLY']) {
        if ( $run_grid_user_config ) {
          exec { "run configTools grid script ${title}":
            timeout     => 0,
            command     => "/bin/sh -c 'unset DISPLAY;${grid_home}/gridSetup.sh -executeConfigTools -silent -skipPrereqs -responseFile ${download_dir}/grid_install_${version}.rsp'",
            user        => $user,
            group       => $group_install,
            environment => ["USER=${user}","LOGNAME=${user}","TMP=${temp_dir}","TMPDIR=${temp_dir}","TEMP=${temp_dir}"],
            path        => $exec_path,
            cwd         => $grid_base,
            umask       => $umask,
            logoutput   => $logoutput,
            returns     => [0,3], # when a scan adress is not defined in the DNS, it fails, but we can continue
            require     => Exec["run root.sh grid script ${title}"],
          }
        } else {
          file { "${download_dir}/cfgrsp.properties":
            ensure    => file,
            content   => epp('ora_install/grid_password.properties.epp',
              {
                'sys_asm_password' => $sys_asm_password
            }),
            mode      => '0600',
            owner     => $user,
            group     => $group,
            show_diff => false, # Don;t show passwords
            require   => Exec["run root.sh grid script ${title}"],
          }
          exec { "run configToolAllCommands grid tool ${title}":
            timeout   => 0, # This can sometimes take a long time
            command   => "${grid_home}/cfgtoollogs/configToolAllCommands RESPONSE_FILE=${download_dir}/cfgrsp.properties",
            user      => $user,
            group     => $group_install,
            path      => $exec_path,
            provider  => 'shell',
            umask     => $umask,
            cwd       => "${grid_home}/cfgtoollogs",
            logoutput => $logoutput,
            returns   => [0,3], # when a scan adress is not defined in the DNS, it fails, but we can continue
            require   => [
              File["${download_dir}/cfgrsp.properties"],
              Exec["run root.sh grid script ${title}"],
              Exec["install oracle grid ${title}"],
            ],
          }
        }
      }
    }
  }
}
