#
# ora_install::installdb
#
# @summary You can use this class to install a working oracle database on your system.
# This defined type supports the following versions of Oracle:
# 
# - 11.2.0.1
# - 11.2.0.3
# - 11.2.0.4
# - 12.1.0.1
# - 12.1.0.2
# - 12.2.0.1
# - 18.0.0.0
# - 19.0.0.0 ( and all subversions)
# - 21.0.0.0 ( and all subversions)
# 
# ## Example
# 
# Here is an example on how you can use this class to install an Oracle database
# on your system.
# 
# ```puppet
# ora_install::installdb{ '12.1.0.2_Linux-x86-64':
#   version                   => '12.1.0.2',
#   file                      => 'V46095-01',
#   database_type             => 'SE',
#   oracle_base               => '/oracle',
#   oracle_home               => '/oracle/product/12.1/db',
#   bash_profile              => true,
#   user                      => 'oracle',
#   group                     => 'dba',
#   group_install             => 'oinstall',
#   group_oper                => 'oper',
#   download_dir              => '/data/install',
#   zip_extract               => true,
#   puppet_download_mnt_point => $puppet_download_mnt_point,
# }
# ```
#
# @param [Enum['SE2', 'SE', 'EE', 'SEONE']] database_type
#    Selects the type of database you want to install.
#    At this point in time the following database types are supported and allowed:
#    - `EE`     : Enterprise Edition   (all versions)
#    - `SE`     : Standard Edition     (versions <= 12.1.0.1)
#    - `SEONE`  : Standard Edition One (versions <= 12.1.0.1)
#    - `SE2`    : Standard Edition Two (versions >= 12.1.0.2)
#    The default value is: `SE2`
#
# @param [Stdlib::Absolutepath] download_dir
#    The directory where the Puppet software puts all downloaded files.
#    Before Puppet can actually use remote files, they must be downloaded first. Puppet uses this directory to put all files in.
#    The default value is: `/install`
#
# @param [Boolean] bash_profile
#    Create a bash profile for the specified user or not.
#    Valid values are `true` and `false`.
#    When you specify a `true` for the parameter, Puppet will create a standard bash profile for the specified user. The bash profile will be placed in a directory named `user_base_dir/user`.
#    ```puppet
#    ora_install::client { 'Oracle client':
#      ...
#      bash_profile  => true,
#      user          => 'oracle',
#      user_base_dir => '/home',
#      ...
#    }
#    ```
#    This snippet will create a bash profile called `/home/oracle/.bash_profile`.
#
# @param [String] bash_additions
#    The text to add at the end of the bash_profile.
#    This parameter will only be used when you have specified `true` for the parameter `bash_profile`
#    The default value is an empty string.
#
# @param [Boolean] cleanup_install_files
#    Cleanup extracted files after use.
#    This is a boolean value. When you set this value to `true`. The installer class will
#    remove all extracted zip files after it has done its work.
#    The default value is: `true`
#    Here is an example:
#    ```puppet
#    ora_install::....{...
#      ...
#      cleanup_install_files => false,  # Keep all unzipped files
#      ...
#    }
#    ```
#
# @param [Optional[String[1]]] cluster_nodes
#    A comma seperated list of the nodes running in the RAC cluster.
#    Example: `node1,node2`
#
# @param [Optional[String[1]]] ee_optional_components
#    This variable is used to enable or disable custom install.
#    When its is set to  true, the attribute `ee_options_selection` is used. When this value is false, the attribute `ee_options_selection` is ignored.
#
# @param [Boolean] ee_options_selection
#    List of Enterprise Edition Options you would like to install.
#    Check the oracle documentation what values are valid.
#
# @param [String[1]] file
#    The source file to use.
#
# @param [String[1]] group
#    The os group to use for these Oracle puppet definitions.
#    The default value is: `dba`
#
# @param [String[1]] group_install
#    The os group to use for installation.
#    The default value is: `oinstall`
#
# @param [String[1]] group_oper
#    The OS group to allow operator rights.
#    The default value is: `oper`
#
# @param [Boolean] is_rack_one_install
#    This variable is used to enable or disable RAC One Node install.
#    - true  : Value of RAC One Node service name is used.
#    - false : Value of RAC One Node service name is not used.
#    The default value is: `false`
#
# @param [Optional[Stdlib::Absolutepath]] ora_inventory_dir
#    The directory that contains the oracle inventory.
#    The default value is: `/oracle_base/oraInventory`
#
# @param [Stdlib::Absolutepath] oracle_base
#    A directory to use as Oracle base directory.
#
# @param [Stdlib::Absolutepath] oracle_home
#    A directory to be used as Oracle home directory for this software.
#
# @param [Optional[String[1]]] puppet_download_mnt_point
#    The base path of all remote files for the defined type or class.
#    The default value is: `puppet:///modules/ora_install`
#
# @param [Stdlib::Absolutepath] temp_dir
#    Directory to use for temporary files.
#
# @param [Boolean] bypass_tmp_noexec
#    This variable is used to enable or disable bypassing noexec mount option on $temp_dir.
#    - true  : The $temp_dir filesystem will be remounted with exec option for the duration
#              of the database software installation, provided $temp_dir is a filesystem.
#    - false : The $temp_dir filesystem will be left alone.
#    The default value is: `true`
#
# @param [String[1]] user
#    The user used for the specified installation.
#    The install class will not create the user for you. You must do that yourself.
#    The default value is: `oracle`
#
# @param [String[1]] umask
#    The umask to use for commands that run in the context of either the grid user or the oracle user.
#    The default is `0022`
#
# @param [Stdlib::Absolutepath] user_base_dir
#    The directory to use as base directory for the users.
#
# @param [Ora_install::Version] version
#    Specifies the version of the component you want to manage or install.
#    At this point in type we support the installation of:
#    - `11.2.0.1`
#    - `11.2.0.3`
#    - `11.2.0.4`
#    - `12.1.0.1`
#    - `12.1.0.2`
#    - `12.2.0.1`
#    - `18.0.0.0`
#    - `19.0.0.0` (and all subversions)
#    - `21.0.0.0` (and all subversions)
#    Here is an example on how to specify the version:
#    ```puppet
#    ora_install::....{...
#      ...
#      version => '12.1.0.2',
#      ...
#    }
#    ```
#
# @param [Boolean] zip_extract
#    The specified source file is a zip file that needs te be extracted.
#    When you specify a value of false, the source attribute must contain a reference to a directory instead of a zip file.
#    The default value is: `true`
#
# @param [Optional[String[1]]] user_password
#    The password for the Oracle user on Windows.
#    This parameter is only used on windows systems
#
# @param [Variant[Boolean, Enum['on_failure']]] logoutput
#    log the outputs of Puppet exec or not.
#    When you specify `true` Puppet will log all output of `exec` types.
#    Valid values are:
#    - `true`
#    - `false`
#    - `on_failure`
#
# @param [Boolean] allow_insecure
#    When set to `true` Ignore HTTPS certificate errors
#    (default: false)
#
#
# See the file "LICENSE" for the full license governing this code.
#
define ora_install::installdb (
  # lint:ignore:2sp_soft_tabs
  Ora_install::Version                         $version                   = undef,
  Stdlib::Absolutepath                         $oracle_base               = undef,
  Stdlib::Absolutepath                         $oracle_home               = undef,
  String[1]                                    $file                      = undef,
  Enum['SE2', 'SE', 'EE', 'SEONE']             $database_type             = 'SE2',
  Optional[Stdlib::Absolutepath]               $ora_inventory_dir         = undef,
  Boolean                                      $ee_options_selection      = false,
  Optional[String[1]]                          $ee_optional_components    = undef, # 'oracle.rdbms.partitioning:11.2.0.4.0,oracle.oraolap:11.2.0.4.0,oracle.rdbms.dm:11.2.0.4.0,oracle.rdbms.dv:11.2.0.4.0,oracle.rdbms.lbac:11.2.0.4.0,oracle.rdbms.rat:11.2.0.4.0'
  Boolean                                      $bash_profile              = true,
  String                                       $bash_additions            = '',
  String[1]                                    $umask                     = '0022',
  String[1]                                    $user                      = 'oracle',
  Optional[String[1]]                          $user_password             = undef,
  Stdlib::Absolutepath                         $user_base_dir             = '/home',
  String[1]                                    $group                     = 'dba',
  String[1]                                    $group_install             = 'oinstall',
  String[1]                                    $group_oper                = 'oper',
  Stdlib::Absolutepath                         $download_dir              = '/install',
  Stdlib::Absolutepath                         $temp_dir                  = '/tmp',
  Boolean                                      $bypass_tmp_noexec         = true,
  Boolean                                      $zip_extract               = true,
  Optional[String[1]]                          $puppet_download_mnt_point = undef,
  Optional[String[1]]                          $cluster_nodes             = undef,
  Boolean                                      $cleanup_install_files     = true,
  Boolean                                      $is_rack_one_install       = false,
  # lint:ignore:manifest_whitespace_opening_brace_before
  Variant[Boolean, Enum['on_failure']]         $logoutput                 = lookup({ name => 'logoutput', default_value => 'on_failure' }),
  Boolean                                      $allow_insecure            = lookup({ name => 'allow_insecure', default_value => false }),
  # lint:endignore
) {
  ora_install { $title: ensure => 'present' }

  if $facts['kernel'] == 'Windows' {
    $oracle_context = {}
    $root_context   = {}
  } else {
    $oracle_context = {
      user  => $user,
      umask => $umask,
      group => $group_install,
    }
    $root_context   = {
      user  => 'root',
      group => 'root',
    }
  }

  unless ( $facts['kernel'] in ['Linux','SunOS', 'Windows']) {
    fail('Unrecognized operating system, please use it on a Linux, Windows or SunOS host')
  }

  unless ( $oracle_base in $oracle_home ) {
    fail('oracle_home folder should be under the oracle_base folder')
  }

  $sanitized_oracle_home = easy_type::sanetize($oracle_home)
  $extract_path          = "${download_dir}/${file}_${sanitized_oracle_home}"

  case $facts['os']['family'] {
    'windows': {
      $ora_inventory = "C:\\Program Files\\Oracle\\Inventory"
      $response_file = "${download_dir}\\db_install_${version}.rsp"
    }
    default: {
      if ( $ora_inventory_dir ) {
        $ora_inventory = "${ora_inventory_dir}/oraInventory"
      } else {
        $ora_inventory = "${oracle_base}/oraInventory"
      }
      $response_file = "${download_dir}/db_install_${version}_${sanitized_oracle_home}.rsp"
    }
  }
  case $version {
    '11.2.0.1', '12.1.0.1': {
      unless ( $database_type in ['EE', 'SE', 'SEONE']) {
        fail("Unrecognized database_type(${database_type}) use EE, SE or SEONE for version ${version}")
      }
      if $facts['os']['family'] == 'windows' {
        fail("Oracle version ${version} not (yet) supported by Puppet on Windows.")
      }
      $files = ["${file}_1of2.zip", "${file}_2of2.zip"]
      $extract_staged = true
      $install_command = "/bin/sh -c 'unset DISPLAY;${extract_path}/database/runInstaller -silent -waitforcompletion -ignoreSysPrereqs -ignorePrereq -responseFile ${response_file}'"
      $install_creates = "${oracle_home}/dbs"
      if $version == '11.2.0.1' {
        $ee_args = {
          'ee_options_selection'   => $ee_options_selection,
          'ee_optional_components' => $ee_optional_components,
        }
      } else {
        $ee_args = {}
      }
      $dbinstall_rsp_template_arguments = {
        'oracle_home'   => $oracle_home,
        'oracle_base'   => $oracle_base,
        'ora_inventory' => $ora_inventory,
        'database_type' => $database_type,
        'group_install' => $group_install,
        'group'         => $group,
        'group_oper'    => $group_oper,
        'cluster_nodes' => $cluster_nodes,
      } + $ee_args
    }
    '11.2.0.3', '11.2.0.4': {
      unless $database_type in ['EE', 'SE', 'SEONE'] {
        fail("Unrecognized database_type(${database_type}) use EE, SE or SEONE for version ${version}")
      }
      if $facts['os']['family'] == 'windows' {
        fail("Oracle version ${version} not (yet) supported by Puppet on Windows.")
      }
      $files = ["${file}_1of7.zip", "${file}_2of7.zip"]
      $extract_staged = true
      $install_command = "/bin/sh -c 'unset DISPLAY;${extract_path}/database/runInstaller -silent -waitforcompletion -ignoreSysPrereqs -ignorePrereq -responseFile ${response_file}'"
      $install_creates = "${oracle_home}/dbs"
      $dbinstall_rsp_template_arguments = {
        'oracle_home'            => $oracle_home,
        'oracle_base'            => $oracle_base,
        'ora_inventory'          => $ora_inventory,
        'database_type'          => $database_type,
        'group_install'          => $group_install,
        'group'                  => $group,
        'group_oper'             => $group_oper,
        'cluster_nodes'          => $cluster_nodes,
        'ee_options_selection'   => $ee_options_selection,
        'ee_optional_components' => $ee_optional_components,
      }
    }
    '12.1.0.2': {
      unless $database_type in ['EE', 'SE'] {
        fail("Unrecognized database_type(${database_type}) use EE or SE2 for version ${version}")
      }
      if $facts['os']['family']  == 'windows' {
        fail("Oracle version ${version} not (yet) supported by Puppet on Windows.")
      }
      $files = ["${file}_1of2.zip", "${file}_2of2.zip"]
      $extract_staged = true
      $install_command = "/bin/sh -c 'unset DISPLAY;${extract_path}/database/runInstaller -silent -waitforcompletion -ignoreSysPrereqs -ignorePrereq -responseFile ${response_file}'"
      $install_creates = "${oracle_home}/dbs"
      $dbinstall_rsp_template_arguments = {
        'oracle_home'         => $oracle_home,
        'oracle_base'         => $oracle_base,
        'ora_inventory'       => $ora_inventory,
        'database_type'       => $database_type,
        'group_install'       => $group_install,
        'group'               => $group,
        'group_oper'          => $group_oper,
        'cluster_nodes'       => $cluster_nodes,
        'is_rack_one_install' => $is_rack_one_install,
      }
    }
    '12.2.0.1': {
      unless $database_type in ['EE', 'SE2'] {
        fail("Unrecognized database_type(${database_type}) use EE or SE2 for version ${version}")
      }
      if $facts['os']['family'] == 'windows' {
        fail("Oracle version ${version} not (yet) supported by Puppet on Windows.")
      }

      $files = ["${file}.zip"]
      $extract_staged = true
      $install_command = "/bin/sh -c 'unset DISPLAY;${extract_path}/database/runInstaller -silent -waitforcompletion -ignoreSysPrereqs -ignorePrereq -responseFile ${response_file}'"
      $install_creates = "${oracle_home}/dbs"
      $dbinstall_rsp_template_arguments = {
        'oracle_home'         => $oracle_home,
        'oracle_base'         => $oracle_base,
        'ora_inventory'       => $ora_inventory,
        'database_type'       => $database_type,
        'group_install'       => $group_install,
        'group'               => $group,
        'group_oper'          => $group_oper,
        'cluster_nodes'       => $cluster_nodes,
        'is_rack_one_install' => $is_rack_one_install,
      }
    }
    '18.0.0.0', '19.0.0.0', '21.0.0.0': {
      unless $database_type in ['EE', 'SE2'] {
        fail("Unrecognized database_type(${database_type}) use EE or SE2 for version ${version}")
      }
      $files = ["${file}.zip"]
      $extract_staged = false
      case $facts['os']['family'] {
        'windows': {
          $install_command = "${oracle_home}\\setup.exe"
          $install_command_args = "-silent -waitForCompletion -ignorePrereq -responseFile ${download_dir}\\db_install_${version}.rsp"
          $install_creates = "${oracle_home}/oraInst.loc"
        }
        default: {
          $install_command = "/bin/sh -c 'unset DISPLAY;${oracle_home}/runInstaller -silent -waitforcompletion -ignorePrereq -responseFile ${response_file}'"
          $install_creates = "${oracle_home}/oraInst.loc"
        }
      }
      $dbinstall_rsp_template_arguments = {
        'oracle_home'            => $oracle_home,
        'oracle_base'            => $oracle_base,
        'ora_inventory'          => $ora_inventory,
        'database_type'          => $database_type,
        'group_install'          => $group_install,
        'group'                  => $group,
        'group_oper'             => $group_oper,
        'cluster_nodes'          => $cluster_nodes,
      }
    }
    default: {}
  }

  $exec_path = '/usr/local/bin:/bin:/usr/bin:/usr/local/sbin:/usr/sbin:/sbin:'

  if ( $puppet_download_mnt_point ) {
    $mount_point = $puppet_download_mnt_point
  } else {
    $mount_point = 'puppet:///modules/ora_install/'
  }

  if ( $mount_point == $download_dir ) {
    $archive_cleanup = false
  } else {
    $archive_cleanup = true
  }

  easy_type::debug_evaluation() # Show local variable on extended debug

  if ( $bash_profile ) {
    unless ( defined(File["${user_base_dir}/${user}/.bash_profile"]) ) {
      file { "${user_base_dir}/${user}/.bash_profile":
        ensure  => file,
        content => regsubst(epp('ora_install/bash_profile.epp', { 'oracle_home' => $oracle_home, 'oracle_base' => $oracle_base, 'bash_additions' => $bash_additions }), '\r\n', "\n", 'EMG'),
        mode    => '0775',
        owner   => $user,
        group   => $group,
      }
    }
  }

  unless ( ora_install::oracle_exists( $oracle_home ) ) {
    unless $facts['os']['family'] == 'windows' {
      if ( $bypass_tmp_noexec ) {
        $mountpoints = $facts['mountpoints']
        if ( $mountpoints.dig($temp_dir,'options') == Array ) {
          $tmp_mount_options = $mountpoints.dig($temp_dir, 'options')
          if ( member($tmp_mount_options, 'noexec') ) {
            $tmp_exec_options = $tmp_mount_options - 'noexec' + 'exec'

            exec { "Remount ${temp_dir} without noexec option for install of ${oracle_home}":
              command => "mount -o remount,${tmp_exec_options.join(',')} ${temp_dir}",
              path    => $exec_path,
              before  => Exec["install oracle database ${oracle_home}"],
            }

            exec { "Remount ${temp_dir} with original options for install of ${oracle_home}":
              command => "mount -o remount,${tmp_mount_options.join(',')} ${temp_dir}",
              path    => $exec_path,
              require => Exec["install oracle database ${oracle_home}"],
            }
          }
        }
      }
    }

    db_directory_structure { "oracle structure ${version} for ${oracle_home}":
      ensure            => present,
      oracle_base_dir   => $oracle_base,
      ora_inventory_dir => $ora_inventory,
      download_dir      => $download_dir,
      os_user           => $user,
      os_group          => $group_install,
    }

    if ( $zip_extract ) {
      if ( $extract_staged ) {
        case $facts['os']['family'] {
          'windows': {
            file { "${download_dir}\\${file}":
              ensure => directory,
              owner  => $user,
            }
          }
          default: {
            file { $extract_path:
              ensure => directory,
              owner  => $user,
              group  => $group,
              mode   => '0755',
            }
          }
        }

        $files.each | $filename | {
          archive { "${download_dir}/${sanitized_oracle_home}_${filename}":
            ensure         => present,
            allow_insecure => $allow_insecure,
            before         => Exec["install oracle database ${oracle_home}"],
            cleanup        => $archive_cleanup,
            creates        => "${extract_path}/installdb",
            group          => $group_install,
            extract        => true,
            extract_path   => $extract_path,
            require        => Db_directory_structure["oracle structure ${version} for ${oracle_home}"],
            source         => "${mount_point}/${filename}",
            temp_dir       => $temp_dir,
            user           => $user,
          }
          if $archive_cleanup {
            cleanup { "${download_dir}/${sanitized_oracle_home}_${filename}": }
          }
        }
      } else {
        case $facts['os']['family'] {
          'windows': {
            file { $oracle_home:
              ensure  => directory,
              recurse => false,
              replace => false,
              owner   => $user,
              require => Db_directory_structure["oracle structure ${version} for ${oracle_home}"],
            }

            $files.each | $filename | {
              archive { "${download_dir}\\${sanitized_oracle_home}_${filename}":
                ensure         => present,
                allow_insecure => $allow_insecure,
                before         => Exec["install oracle database ${oracle_home}"],
                cleanup        => $archive_cleanup,
                creates        => "${oracle_home}\\installdb",
                extract        => true,
                extract_path   => $oracle_home,
                require        => File[$oracle_home],
                source         => "${mount_point}/${filename}",
                temp_dir       => $temp_dir,
                user           => $user,
              }
              if $archive_cleanup {
                cleanup { "${download_dir}\\${sanitized_oracle_home}_${filename}": }
              }
            }
          }
          default: {
            file { $oracle_home:
              ensure  => directory,
              recurse => false,
              replace => false,
              mode    => '0775',
              owner   => $user,
              group   => $group_install,
              require => Db_directory_structure["oracle structure ${version} for ${oracle_home}"],
            }

            $files.each | $filename | {
              archive { "${download_dir}/${sanitized_oracle_home}_${filename}":
                ensure         => present,
                allow_insecure => $allow_insecure,
                before         => Exec["install oracle database ${oracle_home}"],
                cleanup        => $archive_cleanup,
                creates        => "${oracle_home}/installdb",
                extract        => true,
                extract_path   => $oracle_home,
                group          => $group_install,
                require        => File[$oracle_home],
                source         => "${mount_point}/${filename}",
                temp_dir       => $temp_dir,
                user           => $user,
              }
              if $archive_cleanup {
                cleanup { "${download_dir}/${sanitized_oracle_home}_${filename}": }
              }
            }
          }
        }
      }
    }

    ora_install::utils::dborainst { "database orainst ${version} for ${oracle_home}":
      ora_inventory_dir => $ora_inventory,
      os_group          => $group_install,
      os_user           => $user,
    }

    if ( $facts['os']['family'] == 'windows' ) {
      unless ( defined(File["${download_dir}\\db_install_${version}.rsp"]) ) {
        file { "${download_dir}\\db_install_${version}.rsp":
          ensure  => file,
          content => epp("ora_install/db_install_${version}.rsp.epp", $dbinstall_rsp_template_arguments),
          owner   => $user,
          require => [
            Ora_install::Utils::Dborainst["database orainst ${version} for ${oracle_home}"],
            Db_directory_structure["oracle structure ${version} for ${oracle_home}"]
          ],
        }
      }
      unless ( defined(File["${download_dir}\\installdb_${version}.ps1"]) ) {
        file { "${download_dir}\\installdb_${version}.ps1":
          ensure  => file,
          content => epp('ora_install/db_install.ps1.epp',
            {
              'user_password'        => $user_password,
              'user'                 => $user,
              'install_command'      => $install_command,
              'install_command_args' => $install_command_args,
          }),
          require => [
            Ora_install::Utils::Dborainst["database orainst ${version} for ${oracle_home}"],
            Db_directory_structure["oracle structure ${version} for ${oracle_home}"]
          ],
        }
      }

      exec { "install oracle database ${oracle_home}":
        command   => "${download_dir}\\installdb_${version}.ps1",
        creates   => $install_creates,
        provider  => 'powershell',
        timeout   => 3600,
        returns   => [6,0],
        logoutput => true,
        require   => [
          Ora_install::Utils::Dborainst["database orainst ${version} for ${oracle_home}"],
          File["${download_dir}\\db_install_${version}.rsp"],
          File["${download_dir}\\installdb_${version}.ps1"]
        ],
      }
    } else {
      unless ( defined(File[$response_file]) ) {
        file { $response_file:
          ensure  => file,
          content => epp("ora_install/db_install_${version}.rsp.epp",
          $dbinstall_rsp_template_arguments),
          mode    => '0775',
          owner   => $user,
          group   => $group,
          require => [
            Ora_install::Utils::Dborainst["database orainst ${version} for ${oracle_home}"],
            Db_directory_structure["oracle structure ${version} for ${oracle_home}"]
          ],
        }
      }

      #
      # On new not (yet) recognised OS-es simulate OEL 7.6
      #
      $base_environment = ["USER=${user}","LOGNAME=${user}","TMP=${temp_dir}","TMPDIR=${temp_dir}","TEMP=${temp_dir}"]
      if $facts['os']['family'] == 'RedHat' and $facts['os']['release']['major'] == '8' {
        $environment = $base_environment << 'CV_ASSUME_DISTID=OEL7.6'
      } else {
        $environment = $base_environment
      }

      exec { "install oracle database ${oracle_home}":
        command     => $install_command,
        creates     => $install_creates,
        environment => $environment,
        timeout     => 0,
        returns     => [6,0],
        path        => $exec_path,
        cwd         => $oracle_base,
        logoutput   => $logoutput,
        require     => [
          Ora_install::Utils::Dborainst["database orainst ${version} for ${oracle_home}"],
          File[$response_file]
        ],
        *           => $oracle_context,
      }

      # Disable default read-only ORACLE_HOME for version 21c for now
      # TODO: Check if we can support read-only oracle homes with config files like init.ora in $(orabaseconfig)/dbs
      if ( $version == '21.0.0.0' ) {
        exec { "disable read-only oracle home for ${oracle_home}":
          command     => "${oracle_home}/bin/roohctl -disable",
          environment => ["USER=${user}","LOGNAME=${user}","TMP=${temp_dir}","TMPDIR=${temp_dir}","TEMP=${temp_dir}","ORACLE_HOME=${oracle_home}"],
          path        => $exec_path,
          user        => $user,
          umask       => $umask,
          group       => $group_install,
          require     => Exec["install oracle database ${oracle_home}"],
        }
      }

      unless ( $facts['os']['family'] == 'windows' ) {
        exec { "run root.sh script ${title}":
          command   => "${oracle_home}/root.sh",
          path      => $exec_path,
          cwd       => $oracle_base,
          logoutput => $logoutput,
          require   => Exec["install oracle database ${oracle_home}"],
          *         => $root_context,
        }
      }
    }

    if ( $extract_staged ) {
      file { $oracle_home:
        ensure  => directory,
        recurse => false,
        replace => false,
        mode    => '0775',
        owner   => $user,
        group   => $group_install,
        require => Exec["install oracle database ${oracle_home}"],
      }
    }

    # cleanup
    if ( $cleanup_install_files ) {
      if $facts['os']['family'] != 'windows' {
        cleanup { "cleanup ${oracle_home} response file":
          file_name => $response_file,
        }
      }

      if ( $zip_extract ) {
        if ( $facts['os']['family'] == 'windows' ) {
          cleanup { "remove oracle db extract folder ${title}":
            file_name => ["${download_dir}\\${file}","${download_dir}\\${file}"],
          }
        } else {
          cleanup { "remove oracle db extract folder ${title}":
            file_name => $extract_path,
          }
        }
      }
    }
  }
}
