#
# ora_install::installem
#
# @summary Installs the Oracle Enterprse Manager.
# Here is an example on how to use this:
# 
# ```puppet
# ora_install::installem{ 'em13200':
#   version                     => '13.2.0.0',
#   file                        => 'em13200p1_linux64',
#   oracle_base_dir             => '/u01/app/oracle',
#   oracle_home_dir             => '/u01/app/oracle/product/13.2.0.0/em_home1',
#   agent_base_dir              => '/u01/app/oracle/agent',
#   software_library_dir        => '/u01/app/oracle/swlib',
#   weblogic_user               => 'weblogic',
#   weblogic_password           => 'Welcome01',
#   database_hostname           => 'em1320.example.com',
#   database_listener_port      => 1521,
#   database_service_sid_name   => 'EM122.domain.local',
#   database_sys_password       => 'Welcome01',
#   sysman_password             => 'Welcome01',
#   agent_registration_password => 'Welcome01',
#   deployment_size             => 'SMALL',
#   user                        => 'oracle',
#   group                       => 'oinstall',
#   download_dir                => '/install',
#   zip_extract                 => true,
#   puppet_download_mnt_point   => 'puppet:///modules/software',
#   logoutput                   => true,
# }
# 
# ```
#
# @param [Integer] admin_server_https_port
#    The https IP port from the WLS Admin server.
#    The default value is: `7101`
#
# @param [Stdlib::Absolutepath] agent_base_dir
#    The directory to use as base for the agent software.
#
# @param [Integer] agent_port
#    The IP port to use to for the agent.
#    The default value is: `3872`
#
# @param [Easy_type::Password] agent_registration_password
#    The password to use to register the agent.
#
# @param [Integer] bi_publisher_http_port
#    The http port to use for the BI Publisher.
#    The default value is: `9701`
#
# @param [Integer] bi_publisher_https_port
#    The https port to use for the BI Publisher.
#    The default value is: `9801`
#
# @param [String[1]] database_hostname
#    The DNS name of the database host.
#
# @param [Integer] database_listener_port
#    The IP port for the database listener.
#    The default value is: `1521`
#
# @param [String[1]] database_service_sid_name
#    The database service SID name for the database.
#
# @param [Easy_type::Password] database_sys_password
#    The password of the SYS user of the database.
#
# @param [Enum['SMALL', 'MEDIUM', 'LARGE']] deployment_size
#    The size of the deployment.
#    Valid values are:
#    - `SMALL`
#    - `MEDIUM`
#    - `LARGE`
#    The default value is: `SMALL`
#
# @param [Stdlib::Absolutepath] download_dir
#    The directory where the Puppet software puts all downloaded files.
#    Before Puppet can actually use remote files, they must be downloaded first. Puppet uses this directory to put all files in.
#    The default value is: `/install`
#
# @param [Integer] em_central_console_http_port
#    The port number of the HTTP port for the central console.
#    The default value is: `7788`
#
# @param [Integer] em_central_console_https_port
#    The port number of the HTTPS port for the central console.
#    The default value is: `7799`
#
# @param [Integer] em_upload_http_port
#    The port number of the HTTP port for the upload service.
#    The default value is: `1159`
#
# @param [Integer] em_upload_https_port
#    The port number of the HTTPS port for the upload service.
#    The default value is: `4889`
#
# @param [String[1]] file
#    The source file to use.
#
# @param [String[1]] group
#    The os group to use for these Oracle puppet definitions.
#    The default value is: `dba`
#
# @param [Variant[Boolean, Enum['on_failure']]] logoutput
#    log the outputs of Puppet exec or not.
#    When you specify `true` Puppet will log all output of `exec` types.
#    Valid values are:
#    - `true`
#    - `false`
#    - `on_failure`
#
# @param [Integer] managed_server_http_port
#    The port number of the HTTP port for the Managed server.
#    The default value is: `7201`
#
# @param [Integer] managed_server_https_port
#    The port number of the HTTPS port for the Managed server.
#    The default value is: `7301`
#
# @param [Integer] nodemanager_https_port
#    The port number of the HTTPS port for the Node manager.
#    The default value is: `7401`
#
# @param [Optional[Stdlib::Absolutepath]] ora_inventory_dir
#    The directory that contains the oracle inventory.
#    The default value is: `/oracle_base/oraInventory`
#
# @param [Stdlib::Absolutepath] oracle_base_dir
#    A directory to use as Oracle base directory.
#
# @param [Stdlib::Absolutepath] oracle_home_dir
#    A directory to be used as Oracle home directory for this software.
#
# @param [String[1]] puppet_download_mnt_point
#    The base path of all remote files for the defined type or class.
#    The default value is: `puppet:///modules/ora_install`
#
# @param [Stdlib::Absolutepath] software_library_dir
#    The directory to use for the software library.
#
# @param [Boolean] swonly
#    Only install the software without configuration (true) or not (false).
#    The default value is: `false`
#
# @param [Easy_type::Password] sysman_password
#    The password to use for sysman.
#
# @param [Stdlib::Absolutepath] temp_dir
#    Directory to use for temporary files.
#
# @param [String[1]] user
#    The user used for the specified installation.
#    The install class will not create the user for you. You must do that yourself.
#    The default value is: `oracle`
#
# @param [Enum['12.1.0.4', '12.1.0.5', '13.1.0.0', '13.2.0.0', '13.3.0.0', '13.4.0.0', '13.5.0.0']] version
#    The version that is installed in the used Oracle home.
#    Puppet uses this value to decide on version specific actions.
#
# @param [Easy_type::Password] weblogic_password
#    The password to use for WebLogic.
#
# @param [String[1]] weblogic_user
#    The username to use for WebLogic.
#    The default value is: `weblogic`
#
# @param [Boolean] zip_extract
#    The specified source file is a zip file that needs te be extracted.
#    When you specify a value of false, the source attribute must contain a reference to a directory instead of a zip file.
#    The default value is: `true`
#
# @param [Boolean] allow_insecure
#    When set to `true` Ignore HTTPS certificate errors
#    (default: false)
#
# @param [Optional[String[1]]] tablespace_location
#    The location where the tablespace datafiles should be located.
#    Only needed to specify when using ASM as underlying storage.
#
#
# See the file "LICENSE" for the full license governing this code.
#
define ora_install::installem (
  Easy_type::Password                               $weblogic_password,
  Easy_type::Password                               $database_sys_password,
  Easy_type::Password                               $sysman_password,
  Easy_type::Password                               $agent_registration_password,
  # lint:ignore:strict_indent
  Enum[
    '12.1.0.4',
    '12.1.0.5',
    '13.1.0.0',
    '13.2.0.0',
    '13.3.0.0',
    '13.4.0.0',
    '13.5.0.0']                                     $version                       = '12.1.0.5',
  # lint:endignore:strict_indent
  String[1]                                         $file                          = undef,
  Optional[Stdlib::Absolutepath]                    $ora_inventory_dir             = undef,
  Stdlib::Absolutepath                              $oracle_base_dir               = undef,
  Stdlib::Absolutepath                              $oracle_home_dir               = undef,
  Stdlib::Absolutepath                              $agent_base_dir                = undef,
  Stdlib::Absolutepath                              $software_library_dir          = undef,
  String[1]                                         $weblogic_user                 = 'weblogic',
  String[1]                                         $database_hostname             = undef,
  Integer                                           $database_listener_port        = 1521,
  String[1]                                         $database_service_sid_name     = undef,
  Enum['SMALL','MEDIUM','LARGE']                    $deployment_size               = 'SMALL',
  String[1]                                         $user                          = 'oracle',
  String[1]                                         $group                         = 'oinstall',
  Stdlib::Absolutepath                              $download_dir                  = '/install',
  Stdlib::Absolutepath                              $temp_dir                      = '/tmp',
  Boolean                                           $zip_extract                   = true,
  String[1]                                         $puppet_download_mnt_point     = undef,
  Integer                                           $admin_server_https_port       = 7101,
  Integer                                           $managed_server_http_port      = 7201,
  Integer                                           $managed_server_https_port     = 7301,
  Integer                                           $em_upload_http_port           = 4889,
  Integer                                           $em_upload_https_port          = 1159,
  Integer                                           $em_central_console_http_port  = 7788,
  Integer                                           $em_central_console_https_port = 7799,
  Integer                                           $bi_publisher_http_port        = 9701,
  Integer                                           $bi_publisher_https_port       = 9801,
  Integer                                           $nodemanager_https_port        = 7401,
  Integer                                           $agent_port                    = 3872,
  Optional[String[1]]                               $tablespace_location           = undef,
  Variant[Boolean,Enum['on_failure']]               $logoutput                     = lookup({ name => 'logoutput', default_value => 'on_failure' }),
  Boolean                                           $allow_insecure                = lookup({ name => 'allow_insecure', default_value => false }),
  Boolean                                           $swonly                        = false,
) {
  ora_install { $title: ensure => 'present' }

  unless ( $facts['kernel'] in ['Linux','SunOS']) {
    fail('Unrecognized operating system, please use it on a Linux or SunOS host')
  }

  case $version {
    '12.1.0.4','12.1.0.5': {
      $em_files = ["${file}_disk1.zip", "${file}_disk2.zip", "${file}_disk3.zip"]
      $extract_files = true
      $install_command = "/bin/su - ${user} -c 'unset DISPLAY;${download_dir}/${file}/runInstaller -silent -waitforcompletion -ignoreSysPrereqs -ignorePrereq -responseFile ${download_dir}/em_install_${version}.rsp'"
      $rootsh = "${oracle_home_dir}/oms/allroot.sh"
    }
    '13.1.0.0': {
      $em_files = ["${file}.bin", "${file}-2.zip", "${file}-3.zip", "${file}-4.zip", "${file}-5.zip"]
      $extract_files = false
      $install_command = "/bin/su - ${user} -c 'unset DISPLAY;${download_dir}/${file}.bin -silent -waitforcompletion -ignoreSysPrereqs -ignorePrereq -responseFile ${download_dir}/em_install_${version}.rsp -J-Djava.io.tmpdir=${temp_dir}'"
      $rootsh = "${oracle_home_dir}/allroot.sh"
    }
    '13.2.0.0','13.3.0.0': {
      if ( $file =~ /em13200p1/ ) {
        $em_files = ["${file}.bin", "${file}-2.zip", "${file}-3.zip", "${file}-4.zip", "${file}-5.zip", "${file}-6.zip", "${file}-7.zip"]
      } else {
        $em_files = ["${file}.bin", "${file}-2.zip", "${file}-3.zip", "${file}-4.zip", "${file}-5.zip", "${file}-6.zip"]
      }
      $extract_files = false
      $install_command = "/bin/su - ${user} -c 'unset DISPLAY;${download_dir}/${file}.bin -silent -ignoreSysPrereqs -ignorePrereq -responseFile ${download_dir}/em_install_${version}.rsp -J-Djava.io.tmpdir=${temp_dir}'"
      $rootsh = "${oracle_home_dir}/allroot.sh"
    }
    '13.4.0.0': {
      $em_files = ["${file}.bin", "${file}-2.zip", "${file}-3.zip", "${file}-4.zip", "${file}-5.zip", "${file}-6.zip", "${file}-7.zip"]
      $extract_files = false
      $install_command = "/bin/su - ${user} -c 'unset DISPLAY;${download_dir}/${file}.bin -silent -ignoreSysPrereqs -responseFile ${download_dir}/em_install_${version}.rsp -J-Djava.io.tmpdir=${temp_dir}'"
      $rootsh = "${oracle_home_dir}/allroot.sh"
    }
    '13.5.0.0': {
      $em_files = ["${file}.bin", "${file}-2.zip", "${file}-3.zip", "${file}-4.zip", "${file}-5.zip"]
      $extract_files = false
      $install_command = "/bin/su - ${user} -c 'unset DISPLAY;${download_dir}/${file}.bin -silent -ignoreSysPrereqs -responseFile ${download_dir}/em_install_${version}.rsp -J-Djava.io.tmpdir=${temp_dir}'"
      $rootsh = "${oracle_home_dir}/allroot.sh"
    }
    default: {}
  }

  if ( $ora_inventory_dir ) {
    $ora_inventory = "${ora_inventory_dir}/oraInventory"
  } else {
    $ora_inventory = "${oracle_base_dir}/oraInventory"
  }

  easy_type::debug_evaluation() # Show local variable on extended debug

  db_directory_structure { "oracle em structure ${version}":
    ensure            => present,
    oracle_base_dir   => $oracle_base_dir,
    ora_inventory_dir => $ora_inventory,
    download_dir      => $download_dir,
    os_user           => $user,
    os_group          => $group,
  }

  unless ( ora_install::oracle_exists( $oracle_home_dir ) ) {
    $exec_path     = '/usr/local/bin:/bin:/usr/bin:/usr/local/sbin:/usr/sbin:/sbin:'

    if ( $puppet_download_mnt_point ) {
      $mount_point     = $puppet_download_mnt_point
    } else {
      $mount_point     = 'puppet:///modules/ora_install/'
    }

    if ( $mount_point == $download_dir ) {
      $archive_cleanup = false
    } else {
      $archive_cleanup = true
    }

    if ( $zip_extract ) {
      # In $download_dir, will Puppet extract the ZIP files or is this a pre-extracted directory structure.
      if ( $extract_files ) {
        file { "${download_dir}/${file}":
          ensure  => directory,
          owner   => $user,
          group   => $group,
          mode    => '0755',
          require => Db_directory_structure["oracle em structure ${version}"],
        }
      }

      $em_files.each | $filename | {
        if ( $extract_files ) {
          archive { "${download_dir}/${filename}":
            ensure         => present,
            allow_insecure => $allow_insecure,
            before         => Exec["install oracle em ${title}"],
            cleanup        => $archive_cleanup,
            creates        => "${download_dir}/${file}/installem",
            extract        => $extract_files,
            extract_path   => "${download_dir}/${file}",
            group          => $group,
            source         => "${mount_point}/${filename}",
            temp_dir       => $temp_dir,
            user           => $user,
            require        => [
              Db_directory_structure["oracle em structure ${version}"],
              File[$temp_dir],
            ],
          }
        } else {
          archive { "${download_dir}/${filename}":
            ensure         => present,
            allow_insecure => $allow_insecure,
            before         => Exec["install oracle em ${title}"],
            extract        => $extract_files,
            group          => $group,
            source         => "${mount_point}/${filename}",
            temp_dir       => $temp_dir,
            user           => $user,
            require        => [
              Db_directory_structure["oracle em structure ${version}"],
              File[$temp_dir],
            ],
          }
        }
      }
    }

    ora_install::utils::dborainst { "em orainst ${version}":
      ora_inventory_dir => $ora_inventory,
      os_group          => $group,
      os_user           => $user,
    }

    unless ( defined(File["${download_dir}/em_install_${version}.rsp"]) ) {
      file { "${download_dir}/em_install_${version}.rsp":
        ensure  => file,
        content => epp("ora_install/em_install_${version}.rsp.epp",
          {
            'group_install'               => undef,
            'ora_inventory'               => $ora_inventory,
            'oracle_home_dir'             => $oracle_home_dir,
            'agent_base_dir'              => $agent_base_dir,
            'weblogic_user'               => $weblogic_user,
            'weblogic_password'           => $weblogic_password,
            'oracle_base_dir'             => $oracle_base_dir,
            'software_library_dir'        => $software_library_dir,
            'database_hostname'           => $database_hostname,
            'database_listener_port'      => $database_listener_port,
            'database_service_sid_name'   => $database_service_sid_name,
            'database_sys_password'       => $database_sys_password,
            'sysman_password'             => $sysman_password,
            'deployment_size'             => $deployment_size,
            'tablespace_location'         => $tablespace_location,
            'agent_registration_password' => $agent_registration_password,
            'download_dir'                => $download_dir,
            'swonly'                      => $swonly,
        }),
        mode    => '0775',
        owner   => $user,
        group   => $group,
        require => [
          Ora_install::Utils::Dborainst["em orainst ${version}"],
          Db_directory_structure["oracle em structure ${version}"],
        ],
      }
    }
    unless ( defined(File["${download_dir}/em_install_static_${version}.ini"]) ) {
      file { "${download_dir}/em_install_static_${version}.ini":
        ensure  => file,
        content => epp("ora_install/em_install_static_${version}.ini.epp",
          {
            'admin_server_https_port'       => $admin_server_https_port,
            'managed_server_http_port'      => $managed_server_http_port,
            'managed_server_https_port'     => $managed_server_https_port,
            'em_upload_http_port'           => $em_upload_http_port,
            'em_upload_https_port'          => $em_upload_https_port,
            'em_central_console_http_port'  => $em_central_console_http_port,
            'em_central_console_https_port' => $em_central_console_https_port,
            'bi_publisher_http_port'        => $bi_publisher_http_port,
            'bi_publisher_https_port'       => $bi_publisher_https_port,
            'nodemanager_https_port'        => $nodemanager_https_port,
            'agent_port'                    => $agent_port,
          }
        ),
        mode    => '0775',
        owner   => $user,
        group   => $group,
        require => [
          Ora_install::Utils::Dborainst["em orainst ${version}"],
          Db_directory_structure["oracle em structure ${version}"],
        ],
      }
    }

    unless ( defined(File[$temp_dir]) ) {
      # Don't touch privs on default OS temp_dir's
      unless ( $temp_dir == '/tmp' or $temp_dir == '/var/tmp' ) {
        file { $temp_dir:
          ensure => 'directory',
          owner  => $user,
          group  => $group,
          mode   => '0755',
          before => Exec["install oracle em ${title}"],
        }
      }
    }

    if ( "${file}.bin" in $em_files ) {
      exec { "make ${download_dir}/${file}.bin executable":
        command   => "chmod +x ${download_dir}/${file}.bin; chown ${user}:${group} ${download_dir}/${file}*",
        path      => $exec_path,
        cwd       => $oracle_base_dir,
        logoutput => $logoutput,
        before    => Exec["install oracle em ${title}"],
      }
    }

    exec { "install oracle em ${title}":
      command     => $install_command,
      creates     => $oracle_home_dir,
      environment => ["USER=${user}","LOGNAME=${user}","TMP=${temp_dir}","TMPDIR=${temp_dir}","TEMP=${temp_dir}"],
      timeout     => 0,
      returns     => [6,0],
      path        => $exec_path,
      cwd         => $oracle_base_dir,
      logoutput   => $logoutput,
      require     => [
        Ora_install::Utils::Dborainst["em orainst ${version}"],
        File["${download_dir}/em_install_${version}.rsp"],
        File["${download_dir}/em_install_static_${version}.ini"],
      ],
    }

    exec { "run root.sh script ${title}":
      command   => $rootsh,
      user      => 'root',
      group     => 'root',
      path      => $exec_path,
      cwd       => $oracle_base_dir,
      logoutput => $logoutput,
      require   => Exec["install oracle em ${title}"],
    }

    file { $oracle_home_dir:
      ensure  => directory,
      recurse => false,
      replace => false,
      mode    => '0775',
      owner   => $user,
      group   => $group,
      require => Exec["install oracle em ${title}","run root.sh script ${title}"],
    }

    # cleanup
    if ( $zip_extract ) {
      cleanup { 'remove oracle em install files':
        file_name => "${download_dir}/${file}",
      }
      if ( $archive_cleanup ) {
        $em_files.each | $filename | {
          cleanup { "${download_dir}/${filename}": }
        }
      }
    }
  }
}
