#
# ora_install::net
#
# @summary Installs and configures Oracle SQL*Net
# This defined type allows you to install and configure Oracle SQL*net.
# 
# ```puppet
# ora_install::net{ 'config net8':
#   oracle_home   => '/oracle/product/11.2/db',
#   version       => '11.2',
#   user          => 'oracle',
#   group         => 'dba',
#   download_dir  => '/install',
#   db_port       => 1521,
# }
# ```
#
# @param [Integer] db_port
#    The IP port number to use for connecting to the database.
#    The default value is: `1521`
#
# @param [Stdlib::Absolutepath] download_dir
#    The directory where the Puppet software puts all downloaded files.
#    Before Puppet can actually use remote files, they must be downloaded first. Puppet uses this directory to put all files in.
#    The default value is: `/install`
#
# @param [String[1]] group
#    The os group to use for these Oracle puppet definitions.
#    The default value is: `dba`
#
# @param [lint:ignore:manifest_whitespace_opening_brace_beforeVariant[Boolean, Enum['on_failure']]] logoutput
#    log the outputs of Puppet exec or not.
#    When you specify `true` Puppet will log all output of `exec` types.
#    Valid values are:
#    - `true`
#    - `false`
#    - `on_failure`
#
# @param [Stdlib::Absolutepath] oracle_home
#    A directory to be used as Oracle home directory for this software.
#
# @param [Stdlib::Absolutepath] temp_dir
#    Directory to use for temporary files.
#
# @param [String[1]] user
#    The user used for the specified installation.
#    The install class will not create the user for you. You must do that yourself.
#    The default value is: `oracle`
#
# @param [Ora_install::ShortVersion] version
#    The version that is installed in the used Oracle home.
#    Puppet uses this value to decide on version specific actions.
#
# @param [String[1]] listener_name
#    The name of the listener process.
#    The default value is: `listener`
#
#
# See the file "LICENSE" for the full license governing this code.
#
define ora_install::net (
  Stdlib::Absolutepath                $oracle_home   = undef,
  Ora_install::ShortVersion           $version       = '11.2',
  String[1]                           $user          = 'oracle',
  String[1]                           $group         = 'dba',
  Integer                             $db_port       = 1521,
  Stdlib::Absolutepath                $download_dir  = '/install',
  String[1]                           $listener_name = 'listener',
  Stdlib::Absolutepath                $temp_dir      = '/tmp',
  # lint:ignore:manifest_whitespace_opening_brace_before
  Variant[Boolean,Enum['on_failure']] $logoutput     = lookup({ name => 'logoutput', default_value => 'on_failure' }),
  # lint:endignore:manifest_whitespace_opening_brace_before
) {
  ora_install { $title: ensure => 'present' }

  $sanitized_oracle_home = easy_type::sanetize($oracle_home).downcase

  if $facts['kernel'] == 'Windows' {
    $oracle_context = {}
    $response_file = "${download_dir}\\netca_${version}_${sanitized_oracle_home}_${db_port}.rsp"
    $command = "${oracle_home}\\bin\\netca.bat -silent -responsefile ${response_file}"
    $creates_file = "${oracle_home}\\network\\admin\\listener.ora"
  } else {
    $oracle_context = {
      user  => $user,
      group => $group,
    }
    $response_file = "${download_dir}/netca_${version}_${sanitized_oracle_home}_${db_port}.rsp"
    $command = "${oracle_home}/bin/netca -silent -responsefile ${response_file}"
    $creates_file = "${oracle_home}/network/admin/listener.ora"
  }

  $exec_path = '/usr/local/bin:/bin:/usr/bin:/usr/local/sbin:/usr/sbin:/sbin'
  $response_content = epp("ora_install/netca_${version}.rsp.epp",
    {
      db_port       => $db_port,
      listener_name => $listener_name,
  })

  easy_type::debug_evaluation() # Show local variable on extended debug

  if $facts['kernel'] == 'Windows' {
    file { "${response_file}.tmp":
      ensure  => file,
      content => $response_content,
    }
    exec { "Create response file ${response_file}":
      command => "cmd.exe /c copy ${response_file}.tmp ${response_file}",
      path    => $facts['os']['windows']['system32'],
      unless  => "cmd.exe /c if exist \"${creates_file}\" (exit 0) else (exit 1)",
      require => File["${response_file}.tmp"],
    }
  } else {
    exec { "Create response file ${response_file}":
      command => "/bin/cat > ${response_file} <<EOL\n${response_content}\nEOL; chown ${user}:${group} ${response_file}",
      unless  => "/bin/test -e ${creates_file}",
    }
  }

  exec { "install oracle net ${title}":
    command     => $command,
    cwd         => $temp_dir,
    creates     => $creates_file,
    path        => $exec_path,
    environment => ["USER=${user}",],
    logoutput   => $logoutput,
    require     => Exec["Create response file ${response_file}"],
    *           => $oracle_context,
  }

  cleanup { $response_file:
  }
}
