#
# ora_install::prepareautostart_for
#
# @summary Prepare autostart of the database for linux.
# This defined creates the script needed to start the database after a system reboot. It supports both `systemd` and the older startup scripts.
# 
# Here is an example on how to use it:
# 
# ```puppet
#   ora_install::prepareautostart_for {'DB1':
#     oracle_home  => '/oracle/db1',
#     user         => 'oracle',
#     service_name => 'start_db1',
#   }
# ```
#
# @param [String[1]] oracle_home
#    A directory to be used as Oracle home directory for this software.
#
# @param [String[1]] user
#    The user used for the specified installation.
#    The install class will not create the user for you. You must do that yourself.
#    The default value is: `oracle`
#
# @param [String[1]] group
#    The os group to use for these Oracle puppet definitions.
#    The default value is: `dba`
#
# @param [Optional[Hash]] limits
#    The limits for the systemd service.
#    The default value is: undef
#    Below is an axample. Currently only the 'hard' values are used.
#    ```yaml
#    ora_install::autostartdatabase::limits:
#      '*/nofile':
#        soft: 2048
#        hard: 8192
#      'oracle/nofile':
#        soft: 65536
#        hard: 65536
#      'oracle/nproc':
#        soft: 2048
#        hard: 16384
#      'oracle/stack':
#        soft: 10240
#        hard: 32768
#    ```
#
# @param [Variant[Boolean, Enum['on_failure']]] logoutput
#    log the outputs of Puppet exec or not.
#    When you specify `true` Puppet will log all output of `exec` types.
#    Valid values are:
#    - `true`
#    - `false`
#    - `on_failure`
#
# @param [Optional[String[1]]] systemd_template
#    Use custom EPP template for systemd service
#    The default value is: undef
#
# @param [Optional[Hash]] systemd_template_vars
#    The variables to use when specifying a custom EPP template for systemd service
#    The default value is: undef
#
#
# See the file "LICENSE" for the full license governing this code.
#
define ora_install::prepareautostart_for (
  String[1]                           $oracle_home           = undef,
  String[1]                           $user                  = 'oracle',
  String[1]                           $group                 = 'dba',
  Optional[Hash]                      $limits                = undef,
  Optional[String[1]]                 $systemd_template      = undef,
  Optional[Hash]                      $systemd_template_vars = undef,
  Variant[Boolean,Enum['on_failure']] $logoutput             = lookup({ name => 'logoutput', default_value => 'on_failure' }),
) {
  ora_install { $title: ensure => 'present' }

  $service_name = $title
  $exec_path = '/usr/local/bin:/bin:/usr/bin:/usr/local/sbin:/usr/sbin:/sbin:'

  case $facts['kernel'] {
    'Linux': {
      case $facts['os']['name'] {
        'SLES': {
          $systemd_start_version = 12
        }
        'CentOS', 'RedHat', 'OracleLinux', 'AlmaLinux', 'Rocky': {
          $systemd_start_version = 7
        }
        default: {
          $systemd_start_version = 1000
        }
      }
      if Integer($facts['os']['release']['major'].split('\.')[0]) >= $systemd_start_version {
        $dbora_location = '/etc/systemd/system'
        $systemd_enabled = true
      } else {
        $dbora_location = '/etc/init.d'
        $systemd_enabled = false
      }
    }
    'Windows': {
      # TODO: Chose corret value
      $dbora_location = 'c:\\etc'
      $systemd_enabled = false
    }
    'SunOS': {
      $dbora_location = '/etc'
      $systemd_enabled = false
    }
    default: {
      fail('Unrecognized kernel, please use it on a Linux or SunOS host')
    }
  }

  easy_type::debug_evaluation() # Show local variable on extended debug

  if $systemd_enabled {
    if $systemd_template {
      unless ( $systemd_template_vars ) { fail('systemd_template_vars need to be specified when specifying systemd_template') }
      $systemd_content = regsubst(epp($systemd_template, $systemd_template_vars), '\r\n', "\n", 'EMG')
    } else {
      $systemd_content = regsubst(epp("ora_install/dbora_systemd_${facts['kernel']}.epp", {
            oracle_home => $oracle_home,
            user        => $user,
            group       => $group,
            limits      => $limits,
      }), '\r\n', "\n", 'EMG')
    }
  } else {
    file { "${dbora_location}/${service_name}" :
      ensure  => file,
      mode    => '0755',
      owner   => 'root',
      content => regsubst(epp("ora_install/dbora_${facts['kernel']}.epp", {
            oracle_home  => $oracle_home,
            user         => $user,
            service_name => $service_name,
      }), '\r\n', "\n", 'EMG'),
    }
  }

  case $facts['os']['name'] {
    'Windows': {
      notice 'TODO: implement autostart for windows'
    }
    'CentOS', 'RedHat', 'OracleLinux', 'SLES', 'AlmaLinux', 'Rocky': {
      if $systemd_enabled {
        include 'systemd'
        exec { "disable SysV service ${service_name}":
          command   => "chkconfig --del ${service_name}",
          user      => 'root',
          onlyif    => "chkconfig --list ${service_name}",
          path      => $exec_path,
          logoutput => $logoutput,
        }
        -> file { '/etc/init.d/dbora':
          ensure => absent,
        }
        -> systemd::unit_file { "${service_name}.service":
          content => $systemd_content,
        }
        ~> service { "${service_name}.service":
          ensure => running,
          enable => true,
        }
      } else {
        exec { "enable SysV service ${service_name}":
          command   => "chkconfig --add ${service_name}",
          require   => File["${dbora_location}/${service_name}"],
          user      => 'root',
          unless    => "chkconfig --list | /bin/grep \'${service_name}\'",
          path      => $exec_path,
          logoutput => $logoutput,
        }
      }
    }
    'Ubuntu', 'Debian':{
      exec { "enable service ${service_name}":
        command   => "update-rc.d ${service_name} defaults",
        require   => File["${dbora_location}/${service_name}"],
        user      => 'root',
        unless    => "ls /etc/rc3.d/*${service_name} | /bin/grep \'${service_name}\'",
        path      => $exec_path,
        logoutput => $logoutput,
      }
    }
    'Solaris': {
      $file_name = "/tmp/oradb_smf_${service_name}.xml"
      file { $file_name :
        ensure  => file,
        mode    => '0755',
        owner   => 'root',
        content => epp('ora_install/oradb_smf.xml.epp', {
            'dbora_location' => $dbora_location,
            'service_name'   => $service_name,
        }),
      }
      exec { "enable service ${service_name}":
        command   => "svccfg -v import ${file_name}",
        require   => File[$file_name,"${dbora_location}/${service_name}"],
        user      => 'root',
        unless    => 'svccfg list | grep oracledatabase',
        path      => $exec_path,
        logoutput => $logoutput,
      }
    }
    default: {
      fail('Unrecognized operating system, please use it on a Linux or SunOS host')
    }
  }
}
