#
# ora_install::rcu
#
# @summary Install the repository for several Oracle Fusion products.
# ## Creating a repository
# 
# Using this defined class, you can create repositories for several Oracle products. Here are some examples.
# 
# Here is an example on how to use it:
# 
# ### SOA suite repository
# 
# ```puppet
# ora_install::rcu{'install_SOA_v11.1.1.9.0':
#   rcu_file                  => 'V75907-01.zip',
#   product                   => 'soasuite',
#   version                   => '11.1.1.9.0',
#   oracle_home               => '/u01/app/oracle/product/11.2.0.4/db_home1',
#   user                      => 'oracle',
#   group                     => 'dba',
#   download_dir              => '/install',
#   action                    => 'create',
#   db_server                 => 'asm112.example.com:1521',
#   db_service                => 'DB112.domain.local',
#   sys_password              => 'Welcome01',
#   schema_prefix             => 'DEV',
#   repos_password            => 'Welcome02',
#   puppet_download_mnt_point => 'puppet:///modules/software',
# }
# ```
# 
# ### webcenter repository
# 
# ```puppet
# ora_install::rcu{'install_WebCenter_11.1.1.7':
#   rcu_file                  => 'ofm_rcu_linux_11.1.1.7.0_32_disk1_1of1.zip',
#   product                   => 'webcenter',
#   version                   => '11.1.1.7',
#   oracle_home               => '/u01/app/oracle/product/11.2.0.4/db_home1',
#   user                      => 'oracle',
#   group                     => 'dba',
#   download_dir              => '/install',
#   action                    => 'create',
#   db_server                 => 'asm112.example.com:1521',
#   db_service                => 'DB112.domain.local',
#   sys_password              => 'Welcome01',
#   schema_prefix             => 'DEV',
#   temp_tablespace           => 'TEMP',
#   repos_password            => 'Welcome02',
#   puppet_download_mnt_point => 'puppet:///modules/software',
# }
# ```
# 
# ### OIM, OAM repository
# 
# OIM needs an Oracle Enterprise Edition database
# 
# ```puppet
# ora_install::rcu{'install_OIM_11.1.2.1':
#   rcu_file                  => 'V37476-01.zip',
#   product                   => 'oim',
#   version                   => '11.1.2.1',
#   oracle_home               => '/u01/app/oracle/product/11.2.0.4/db_home1',
#   user                      => 'oracle',
#   group                     => 'dba',
#   download_dir              => '/data/install',
#   action                    => 'create',
#   db_server                 => 'asm112.example.com:1521',
#   db_service                => 'DB112.domain.local',
#   sys_password              => 'Welcome01',
#   schema_prefix             => 'DEV',
#   repos_password            => 'Welcome02',
#   puppet_download_mnt_point => 'puppet:///modules/software',
#   logoutput                 => true,
#  }
# ```
# 
# ## deleting a repository
# 
# You can also use this defined type to delete a repository. To do so, you need te specify `delete` as action.
# 
# Here is an example:
# 
# ```puppet
# ora_install::rcu{'delete_SOA_11.1.1.6':
#   rcu_file          => 'ofm_rcu_linux_11.1.1.6.0_disk1_1of1.zip',
#   product           => 'soasuite',
#   version           => '11.1.1.6',
#   oracle_home       => '/oracle/product/11.2/db',
#   user              => 'oracle',
#   group             => 'dba',
#   download_dir      => '/install',
#   action            => 'delete',
#   db_server         => 'asm112.example.com:1521',
#   db_service        => 'DB112.domain.local',
#   sys_password      => 'Welcome01',
#   schema_prefix     => 'DEV',
#   repos_password    => 'Welcome02',
# }
# ```
#
# @param [Enum['create', 'delete']] action
#    The action to perform.
#    Valid actions are:
#    - `create`
#    - `delete`
#    The default value is: `create`
#
# @param [String[1]] db_server
#    The name of the database server to use.
#
# @param [String[1]] db_service
#    The name of the database service to use.
#
# @param [Stdlib::Absolutepath] download_dir
#    The directory where the Puppet software puts all downloaded files.
#    Before Puppet can actually use remote files, they must be downloaded first. Puppet uses this directory to put all files in.
#    The default value is: `/install`
#
# @param [String[1]] group
#    The os group to use for these Oracle puppet definitions.
#    The default value is: `dba`
#
# @param [Variant[Boolean, Enum['on_failure']]] logoutput
#    log the outputs of Puppet exec or not.
#    When you specify `true` Puppet will log all output of `exec` types.
#    Valid values are:
#    - `true`
#    - `false`
#    - `on_failure`
#
# @param [Optional[Stdlib::Absolutepath]] oracle_home
#    A directory to be used as Oracle home directory for this software.
#
# @param [Enum['soasuite', 'webcenter', 'oam', 'oim', 'all']] product
#    The name of the product for which you want to install the RCU.
#    The following strings are supported as product:
#    - `soasuite`
#    - `webcenter`
#    - `oam`
#    - `oim`
#    - `all`
#    The default value is: `soasuite`
#
# @param [Optional[String[1]]] puppet_download_mnt_point
#    The base path of all remote files for the defined type or class.
#    The default value is: `puppet:///modules/ora_install`
#
# @param [String[1]] rcu_file
#    The file containing the RCU definition.
#
# @param [Easy_type::Password] repos_password
#    The password for the repo user.
#
# @param [String[1]] schema_prefix
#    The prefix name to use for the schema.
#
# @param [Easy_type::Password] sys_password
#    The password of the `SYS` user on the database.
#
# @param [String[1]] sys_user
#    The name of the `SYS` user on the database.
#    The default value is: `sys`
#
# @param [Optional[String[1]]] temp_tablespace
#    Create a temporary tablespace with this name.
#
# @param [String[1]] user
#    The user used for the specified installation.
#    The install class will not create the user for you. You must do that yourself.
#    The default value is: `oracle`
#
# @param [String[1]] version
#    The version that is installed in the used Oracle home.
#    Puppet uses this value to decide on version specific actions.
#
# @param [Boolean] allow_insecure
#    When set to `true` Ignore HTTPS certificate errors
#    (default: false)
#
#
# See the file "LICENSE" for the full license governing this code.
#
define ora_install::rcu (
  Easy_type::Password                            $sys_password,
  Easy_type::Password                            $repos_password,
  String[1]                                      $rcu_file                  = undef,
  Enum['soasuite','webcenter','oam','oim','all'] $product                   = 'soasuite',
  String[1]                                      $version                   = '11.1.1.7',
  Optional[Stdlib::Absolutepath]                 $oracle_home               = undef,
  String[1]                                      $user                      = 'oracle',
  String[1]                                      $group                     = 'dba',
  Stdlib::Absolutepath                           $download_dir              = '/install',
  Enum['create','delete']                        $action                    = 'create',
  String[1]                                      $db_server                 = undef,
  String[1]                                      $db_service                = undef,
  String[1]                                      $sys_user                  = 'sys',
  String[1]                                      $schema_prefix             = undef,
  Optional[String[1]]                            $temp_tablespace           = undef,
  Optional[String[1]]                            $puppet_download_mnt_point = undef,
  Variant[Boolean,Enum['on_failure']]            $logoutput                 = lookup({ name => 'logoutput', default_value => 'on_failure' }),
  Boolean                                        $allow_insecure            = lookup({ name => 'allow_insecure', default_value => false }),
) {
  ora_install { $title: ensure => 'present' }

  case $facts['kernel'] {
    'Linux': {
      $exec_path = '/usr/local/bin:/bin:/usr/bin:/usr/local/sbin:/usr/sbin:/sbin'
    }
    default: {
      fail('Unrecognized or not supported operating system')
    }
  }

  if ( $puppet_download_mnt_point ) {
    $mount_point = $puppet_download_mnt_point
  } else {
    $mount_point = 'puppet:///modules/ora_install/'
  }

  if ( $mount_point == $download_dir ) {
    $archive_cleanup = false
  } else {
    $archive_cleanup = true
  }

  easy_type::debug_evaluation() # Show local variable on extended debug

  # create the rcu folder
  unless ( defined(File["${download_dir}/rcu_${version}"]) ) {
    # check rcu install folder
    file { "${download_dir}/rcu_${version}":
      ensure  => directory,
      recurse => false,
      replace => false,
      mode    => '0775',
      owner   => $user,
      group   => $group,
    }
  }

  # download and extract rcu software
  archive { "${download_dir}/${rcu_file}":
    ensure         => present,
    allow_insecure => $allow_insecure,
    cleanup        => $archive_cleanup,
    creates        => "${download_dir}/rcu_${version}/rcuHome",
    extract        => true,
    extract_path   => "${download_dir}/rcu_${version}",
    group          => $group,
    source         => "${mount_point}/${rcu_file}",
    user           => $user,
  }

  unless ( defined(File["${download_dir}/rcu_${version}/rcuHome/rcu/log"]) ) {
    # check rcu log folder
    file { "${download_dir}/rcu_${version}/rcuHome/rcu/log":
      ensure  => directory,
      recurse => false,
      replace => false,
      mode    => '0775',
      owner   => $user,
      group   => $group,
      require => Archive["${download_dir}/${rcu_file}"],
    }
  }

  if $product == 'soasuite' {
    $components           = '-component SOAINFRA -component ORASDPM -component MDS -component OPSS -component BAM'
    $components_passwords  = [$repos_password.unwrap, $repos_password.unwrap, $repos_password.unwrap,$repos_password.unwrap,$repos_password.unwrap]
  } elsif $product == 'webcenter' {
    $components           = '-component MDS -component OPSS -component CONTENTSERVER11 -component CONTENTSERVER11SEARCH -component URM -component PORTLET -component WEBCENTER -component ACTIVITIES -component DISCUSSIONS'
    # extra password for DISCUSSIONS and ACTIVITIES
    $components_passwords  = [$repos_password.unwrap, $repos_password.unwrap, $repos_password.unwrap, $repos_password.unwrap, $repos_password.unwrap, $repos_password.unwrap, $repos_password.unwrap, $repos_password.unwrap, $repos_password.unwrap, $repos_password.unwrap, $repos_password.unwrap]
  } elsif $product == 'oam' {
    $components           = '-component MDS -component OPSS -component IAU -component OAM'
    $components_passwords  = [$repos_password.unwrap, $repos_password.unwrap, $repos_password.unwrap, $repos_password.unwrap, $repos_password.unwrap]
  } elsif $product == 'oim' {
    $components           = '-component SOAINFRA -component ORASDPM -component MDS -component OPSS -component BAM -component IAU -component BIPLATFORM -component OIF -component OIM -component OAM -component OAAM -component OMSM'
    $components_passwords  = [$repos_password.unwrap, $repos_password.unwrap, $repos_password.unwrap,$repos_password.unwrap,$repos_password.unwrap,$repos_password.unwrap, $repos_password.unwrap, $repos_password.unwrap,$repos_password.unwrap, $repos_password.unwrap, $repos_password.unwrap, $repos_password.unwrap]
  } elsif $product == 'all' {
    $components           = '-component SOAINFRA -component ORASDPM -component MDS -component OPSS -component BAM -component CONTENTSERVER11 -component CONTENTSERVER11SEARCH -component URM -component PORTLET -component WEBCENTER -component ACTIVITIES -component DISCUSSIONS'
    # extra password for DISCUSSIONS and ACTIVITIES
    $components_passwords  = [$repos_password.unwrap, $repos_password.unwrap, $repos_password.unwrap,$repos_password.unwrap,$repos_password.unwrap, $repos_password.unwrap, $repos_password.unwrap, $repos_password.unwrap, $repos_password.unwrap, $repos_password.unwrap, $repos_password.unwrap, $repos_password.unwrap, $repos_password.unwrap, $repos_password.unwrap]
  }

  file { "${download_dir}/rcu_${version}/rcu_passwords_${title}.txt":
    ensure    => file,
    content   => epp('ora_install/rcu_passwords.txt.epp', {
        'sys_password'         => $sys_password,
        'components_passwords' => $components_passwords,
    }),
    mode      => '0775',
    owner     => $user,
    group     => $group,
    show_diff => false,
    require   => Archive["${download_dir}/${rcu_file}"],
  }

  if ( $oracle_home ) {
    $pre_command    = "export SQLPLUS_HOME=${oracle_home};${download_dir}/rcu_${version}/rcuHome/bin/rcu -silent"
  } else {
    $pre_command    = "${download_dir}/rcu_${version}/rcuHome/bin/rcu -silent"
  }
  $post_command     = "-databaseType ORACLE -connectString ${db_server}:${db_service} -dbUser ${sys_user} -dbRole SYSDBA -schemaPrefix ${schema_prefix} ${components} "
  $password_command = " -f < ${download_dir}/rcu_${version}/rcu_passwords_${title}.txt"

  #optional set the Temp tablespace
  if ( $temp_tablespace ) {
    $create_command  = "${pre_command} -createRepository ${post_command} -tempTablespace ${temp_tablespace} ${password_command}"
  } else {
    $create_command  = "${pre_command} -createRepository ${post_command} ${password_command}"
  }
  $delete_command  = "${pre_command} -dropRepository ${post_command} ${password_command}"

  if $action == 'create' {
    $statement = $create_command
  } elsif $action == 'delete' {
    $statement = $delete_command
  }

  db_rcu { $schema_prefix:
    ensure       => $action,
    statement    => $statement,
    os_user      => $user,
    oracle_home  => $oracle_home,
    sys_user     => $sys_user,
    sys_password => $sys_password.unwrap,
    db_server    => $db_server,
    db_service   => $db_service,
    require      => [
      Archive["${download_dir}/${rcu_file}"],
      File["${download_dir}/rcu_${version}/rcu_passwords_${title}.txt"],
    ],
  }
}
