#
# ora_install::tnsnames
#
# @summary Manages an entry in a `tnsnames.
# ora` file.
# 
# Here is an examples on how to use this:
# 
# ```puppet
# ora_install::tnsnames{'entry':
#   oracle_home          => '/u01/app/oracle/product/12.2.0.1/db_home1',
#   user                 => 'oracle',
#   group                => 'dba',
#   server               => { myserver1 => { host => DB122a.example.nl, port => '1521', protocol => 'TCP' },
#                             myserver2 => { host => DB122b.example.nl, port => '1521', protocol => 'TCP' }},
#   connect_service_name => 'DB122.domain.local',
#   connect_server       => 'DEDICATED',
#   entry_type           => 'tnsname',
# }
# ```
#
# @param [String[1]] connect_server
#    The name of the server to connect to.
#
# @param [Optional[String[1]]] connect_service_name
#    The name of the service to connect to.
#
# @param [Enum['tnsname', 'listener']] entry_type
#    The type of entry to manage.
#    You can manage the next type of entries:
#    - `tnsname`
#    - `listener`
#    The default value is: `tnsname`
#
# @param [String[1]] failover
#    Failover ON or OFF.
#    Valid values are:
#    - `ON`
#    - `OFF`
#    The default value is: `ON`
#
# @param [String[1]] group
#    The os group to use for these Oracle puppet definitions.
#    The default value is: `dba`
#
# @param [port=>'1521', protocol=>'TCP'}}, String[1]] loadbalance
#    Load balance ON or OFF.
#    Valid values are:
#    - `ON`
#    - `OFF`
#    The default value is: `ON`
#
# @param [Stdlib::Absolutepath] oracle_home
#    A directory to be used as Oracle home directory for this software.
#
# @param [Hash] server
#    A Hash defining the server entry.
#    The default value is: 
#    ```
#    {myserver => { host => undef, port => '1521', protocol => 'TCP' }}
#    ```
#
# @param [String[1]] user
#    The user used for the specified installation.
#    The install class will not create the user for you. You must do that yourself.
#    The default value is: `oracle`
#
# @param [Optional[String[1]]] sid
#    The database SID to use.
#
#
# See the file "LICENSE" for the full license governing this code.
#
define ora_install::tnsnames (
  Stdlib::Absolutepath        $oracle_home          = undef,
  String[1]                   $user                 = 'oracle',
  String[1]                   $group                = 'dba',
  Hash                        $server               = { myserver => { host => undef, port => '1521', protocol => 'TCP' } },
  String[1]                   $loadbalance          = 'ON',
  String[1]                   $failover             = 'ON',
  Optional[String[1]]         $connect_service_name = undef,
  Optional[String[1]]         $sid                  = undef,
  String[1]                   $connect_server       = 'DEDICATED',
  Enum['tnsname','listener']  $entry_type           = 'tnsname',
) {
  easy_type::debug_evaluation() # Show local variable on extended debug

  ora_install { $title: ensure => 'present' }

  if $connect_service_name != undef and $sid != undef { fail('must specify either sid or connect_service_name, not both') }
  if $connect_service_name == undef and $sid == undef { fail('must specify either sid or connect_service_name') }

  unless ( defined(Concat["${oracle_home}/network/admin/tnsnames.ora"]) ) {
    concat { "${oracle_home}/network/admin/tnsnames.ora":
      ensure         => present,
      owner          => $user,
      group          => $group,
      mode           => '0774',
      ensure_newline => true,
    }
  }

  case $entry_type {
    'tnsname'  : {
      $template_path = 'ora_install/tnsnames.epp'
      $args = {
        'title'                => $title,
        'servers'              => $server,
        'connect_server'       => $connect_server,
        'connect_service_name' => $connect_service_name,
        'sid'                  => $sid,
        'loadbalance'          => $loadbalance,
        'failover'             => $failover,
      }
    }
    'listener' : {
      $template_path = 'ora_install/listener.epp'
      $args = {
        'title'                => $title,
        'servers'              => $server,
      }
    }
    default    : { fail("${entry_type} is not a supported entry_type") }
  }

  concat::fragment { $title:
    target  => "${oracle_home}/network/admin/tnsnames.ora",
    content => epp($template_path, $args),
  }
}
