#
# @summary This plan will install a new GRID_HOME.
#
# @example
#   bolt plan run ora_install::install_grid_home -t localhost version=19.0.0.0 file=LINUX.X64_193000_grid_home grid_type=HA_SWONLY grid_base=/u01/app/grid/admin grid_home=/u01/app/grid/product/19.0.0.0/grid_home2 ora_inventory_dir=/u01/app puppet_download_mnt_point=/nfs_share/software sys_asm_password=secret asm_monitor_password=secret
#
# @param [TargetSpec] dbserver
#   The target for this plan.
#
# @param [String[1]] sys_asm_password
#   The password to use for the SYSASM user.
#
# @param [String[1]] asm_monitor_password
#   The password for the ASMSNMP user for monitoring in EM.
#
# @param [Ora_install::Version] version
#   The version that is installed in the used Oracle home.
#
# @param [String[1]] file
#   The source file to use for the GRID_HOME installation.
#
# @param [Enum['CRS_CONFIG','HA_CONFIG','UPGRADE','CRS_SWONLY','HA_SWONLY']] grid_type
#   The type of grid.
#
# @param [Boolean] stand_alone
#   Configuring Grid Infrastructure for a Stand-Alone Server.
#
# @param [Stdlib::Absolutepath] grid_base
#   The directory to use as grid base.
#
# @param [Stdlib::Absolutepath] grid_home
#   The directory to use as grid home.
#
# @param [Optional[Stdlib::Absolutepath]] ora_inventory_dir
#   The directory that contains the oracle inventory.
#
# @param [String[1]] user
#   The user used for the specified installation.
#
# @param [Stdlib::Absolutepath] user_base_dir
#   The directory to use as base directory for the users.
#
# @param [String[1]] group
#   The os group to use for these Oracle puppet definitions.
#
# @param [String[1]] group_install
#   The os group to use for installation.
#
# @param [String[1]] group_oper
#   The OS group to allow ASM oper operations.
#
# @param [String[1]] group_asm
#   The OS group to use.
#
# @param [Stdlib::Absolutepath] download_dir
#   The directory where the Puppet software puts all downloaded files.
#
# @param [Stdlib::Absolutepath] temp_dir
#   Directory to use for temporary files.
#
# @param [Boolean] zip_extract
#   The specified source file is a zip file that needs te be extracted.
#   When you specify a value of false, the source attribute must contain a reference to a directory instead of a zip file.
#
# @param [Enum['ALL','EXTRACT','SETUP']] install_task
#   The task the installation should execute.
#
# @param [Optional[String[1]]] puppet_download_mnt_point
#   The base path of all remote files for the defined type or class.
#
# @param [Optional[Array[Stdlib::Absolutepath]]] dirs
#   Directories that should be created in this plan. For example the parent directory (tree) of the GRID_HOME.
#
# @param [Optional[Enum['json','pretty']]] format
#   Specify the format of the output, human readable (pretty) or not (json).
#
#
# See the file "LICENSE" for the full license governing this code.
#
plan ora_install::install_grid_home(
  TargetSpec                           $dbserver,
  String[1]                            $sys_asm_password,
  String[1]                            $asm_monitor_password,
  Ora_install::Version                 $version                       = undef,
  String[1]                            $file                          = undef,
  Enum['CRS_CONFIG','HA_CONFIG','UPGRADE','CRS_SWONLY','HA_SWONLY']
  $grid_type                     = 'HA_CONFIG',
  Boolean                              $stand_alone                   = true, # in case of 'CRS_SWONLY' and used as stand alone or in RAC
  Stdlib::Absolutepath                 $grid_base                     = undef,
  Stdlib::Absolutepath                 $grid_home                     = undef,
  Optional[Stdlib::Absolutepath]       $ora_inventory_dir             = undef,
  String[1]                            $user                          = 'grid',
  Stdlib::Absolutepath                 $user_base_dir                 = '/home',
  String[1]                            $group                         = 'asmdba',
  String[1]                            $group_install                 = 'oinstall',
  String[1]                            $group_oper                    = 'asmoper',
  String[1]                            $group_asm                     = 'asmadmin',
  Stdlib::Absolutepath                 $download_dir                  = '/install',
  Stdlib::Absolutepath                 $temp_dir                      = '/tmp',
  Boolean                              $zip_extract                   = true,
  Enum['ALL','EXTRACT','SETUP']        $install_task                  = 'ALL',
  Optional[String[1]]                  $puppet_download_mnt_point     = undef,
  Optional[Array[Stdlib::Absolutepath]]
  $dirs                          = [],
  Optional[Enum['json','pretty']]      $format                        = 'pretty',
) {
  if get_targets($dbserver).length > 1 {
    fail_plan("${dbserver} did not resolve to a single target")
  }

  $dbserver.apply_prep

  $result = apply($dbserver, { required_modules => ['easy_type', 'ora_install'], '_catch_errors' => true })

  $dirs.each |$dir| {
    unless defined(File[$dir]) {
      case $facts['os']['family'] {
        'windows': {
          file { $dir:
            ensure => directory,
            owner  => $user,
          }
        }
        default: {
          file { $dir:
            ensure  => directory,
            owner   => $user,
            group   => $group_install,
            seltype => 'default_t',
            mode    => '0770',
          }
        }
      }
    }
  }

  ora_install::installasm { 'install_grid_home':
    sys_asm_password          => Sensitive($sys_asm_password),
    asm_monitor_password      => Sensitive($asm_monitor_password),
    version                   => $version,
    file                      => $file,
    grid_type                 => $grid_type,
    stand_alone               => $stand_alone,
    grid_base                 => $grid_base,
    grid_home                 => $grid_home,
    ora_inventory_dir         => $ora_inventory_dir,
    bash_profile              => false,
    user                      => $user,
    user_base_dir             => $user_base_dir,
    group                     => $group,
    group_install             => $group_install,
    group_oper                => $group_oper,
    group_asm                 => $group_asm,
    download_dir              => $download_dir,
    temp_dir                  => $temp_dir,
    zip_extract               => $zip_extract,
    install_task              => $install_task,
    puppet_download_mnt_point => $puppet_download_mnt_point,
    require                   => case $dirs.empty {
      true:    { undef }
      default: { File[$dirs] }
    },
  }
}
$target_result = $result.first()
if $target_result.error {
  fail_plan($target_result.error())
} else {
  $target_result.report['logs'].each |$log| {
    out::message("${log['source']}: ${log['message']}")
  }
}
}
