#
# @summary This plan will install a new GRID_HOME and apply the specified patches to it.
#
# @example
#   bolt plan run ora_install::install_grid_home_and_patch -t localhost version=19.0.0.0 file=LINUX.X64_193000_grid_home grid_type=HA_SWONLY grid_base=/u01/app/grid/admin grid_home=/u01/app/grid/product/19.0.0.0/grid_home2 ora_inventory_dir=/u01/app puppet_download_mnt_point=/nfs_share/software sys_asm_password=secret asm_monitor_password=secret patch_file=p6880880_190000_Linux-x86-64.zip opversion=12.2.0.1.17 patch_list='{"/u01/app/grid/product/19.0.0.0/grid_home2:30116789": {"source": "/nfs_share/software/p30116789_190000_Linux-x86-64.zip", "sub_patches": ["30125133","30122149","30122167","29401763"]}}'
#
# @param [TargetSpec] dbserver
#   The target for this plan.
#
# @param [String[1]] sys_asm_password
#   The password to use for the SYSASM user.
#
# @param [String[1]] asm_monitor_password
#   The password for the ASMSNMP user for monitoring in EM.
#
# @param [Ora_install::Version] version
#   The version that is installed in the used Oracle home.
#
# @param [String[1]] file
#   The source file to use for the GRID_HOME installation.
#
# @param [Enum['CRS_CONFIG','HA_CONFIG','UPGRADE','CRS_SWONLY','HA_SWONLY']] grid_type
#   The type of grid.
#
# @param [Boolean] stand_alone
#   Configuring Grid Infrastructure for a Stand-Alone Server.
#
# @param [Stdlib::Absolutepath] grid_base
#   The directory to use as grid base.
#
# @param [Stdlib::Absolutepath] grid_home
#   The directory to use as grid home.
#
# @param [Optional[Stdlib::Absolutepath]] ora_inventory_dir
#   The directory that contains the oracle inventory.
#
# @param [String[1]] user
#   The user used for the specified installation.
#
# @param [Stdlib::Absolutepath] user_base_dir
#   The directory to use as base directory for the users.
#
# @param [String[1]] group
#   The os group to use for these Oracle puppet definitions.
#
# @param [String[1]] group_install
#   The os group to use for installation.
#
# @param [String[1]] group_oper
#   The OS group to allow ASM oper operations.
#
# @param [String[1]] group_asm
#   The OS group to use.
#
# @param [Stdlib::Absolutepath] download_dir
#   The directory where the Puppet software puts all downloaded files.
#
# @param [Stdlib::Absolutepath] temp_dir
#   Directory to use for temporary files.
#
# @param [Boolean] zip_extract
#   The specified source file is a zip file that needs te be extracted.
#   When you specify a value of false, the source attribute must contain a reference to a directory instead of a zip file.
#
# @param [Enum['ALL','EXTRACT','SETUP']] install_task
#   The task the installation should execute.
#
# @param [Optional[String[1]]] puppet_download_mnt_point
#   The base path of all remote files for the defined type or class.
#
# @param [Optional[Array[Stdlib::Absolutepath]]] dirs
#   Directories that should be created in this plan. For example the parent directory (tree) of the GRID_HOME.
#
# @param [String[1]] patch_file
#   The zip file containing the Opatch utility.
#
# @param [Optional[Integer]] csi_number
#   The Customer Service Identification number.
#
# @param [Optional[String[1]]] support_id
#   Your support ID.
#
# @param [String[1]] opversion
#   The version of Opatch you want to install.
#
# @param [Optional[String[1]]] source
#   The source of the patch. Can be either an extracted folder or the raw zip file.
#
# @param [Hash] patch_list
#   List of patches to apply.
# @option patch_list [String[1]] source
#   Full path to the zipfile that contains the patch
# @option patch_list [Array] sub_patches
#   List of patch numbers that are part of the patchset
#
# @param [Optional[Enum['json','pretty']]] format
#   Specify the format of the output, human readable (pretty) or not (json).
#
#
# See the file "LICENSE" for the full license governing this code.
#
plan ora_install::install_grid_home_and_patch(
  TargetSpec                           $dbserver,
  String[1]                            $sys_asm_password,
  String[1]                            $asm_monitor_password,
  Ora_install::Version                 $version                       = undef,
  String[1]                            $file                          = undef,
  Enum['CRS_CONFIG','HA_CONFIG','UPGRADE','CRS_SWONLY','HA_SWONLY']
  $grid_type                     = 'HA_CONFIG',
  Boolean                              $stand_alone                   = true, # in case of 'CRS_SWONLY' and used as stand alone or in RAC
  Stdlib::Absolutepath                 $grid_base                     = undef,
  Stdlib::Absolutepath                 $grid_home                     = undef,
  Optional[Stdlib::Absolutepath]       $ora_inventory_dir             = undef,
  String[1]                            $user                          = 'grid',
  Stdlib::Absolutepath                 $user_base_dir                 = '/home',
  String[1]                            $group                         = 'asmdba',
  String[1]                            $group_install                 = 'oinstall',
  String[1]                            $group_oper                    = 'asmoper',
  String[1]                            $group_asm                     = 'asmadmin',
  Stdlib::Absolutepath                 $download_dir                  = '/install',
  Stdlib::Absolutepath                 $temp_dir                      = '/tmp',
  Boolean                              $zip_extract                   = true,
  Enum['ALL','EXTRACT','SETUP']        $install_task                  = 'ALL',
  Optional[String[1]]                  $puppet_download_mnt_point     = undef,
  Optional[Array[Stdlib::Absolutepath]]
  $dirs                          = [],
  # opatch_upgrade
  String[1]                            $patch_file                = undef,
  Optional[Integer]                    $csi_number                = undef,
  Optional[String[1]]                  $support_id                = undef,
  String[1]                            $opversion                 = undef,
  # patch_oracle_home
  Optional[String[1]]                  $source                    = undef,
  Hash                                 $patch_list                = {},
  Optional[Enum['json','pretty']]      $format                    = 'pretty',
) {
  if get_targets($dbserver).length > 1 {
    fail_plan("${dbserver} did not resolve to a single target")
  }
  $install_params = {
    'sys_asm_password'          => $sys_asm_password,
    'asm_monitor_password'      => $asm_monitor_password,
    'version'                   => $version,
    'file'                      => $file,
    'grid_type'                 => $grid_type,
    'grid_base'                 => $grid_base,
    'grid_home'                 => $grid_home,
    'ora_inventory_dir'         => $ora_inventory_dir,
    'user'                      => $user,
    'user_base_dir'             => $user_base_dir,
    'group'                     => $group,
    'group_install'             => $group_install,
    'group_oper'                => $group_oper,
    'group_asm'                 => $group_asm,
    'download_dir'              => $download_dir,
    'temp_dir'                  => $temp_dir,
    'zip_extract'               => $zip_extract,
    'install_task'              => $install_task,
    'dirs'                      => $dirs,
    'puppet_download_mnt_point' => $puppet_download_mnt_point,
    'format'                    => $format,
  }
  $install_grid_home = run_plan('ora_install::install_grid_home', $dbserver, $install_params)
  case $install_grid_home {
    'Error'  : { fail_plan($install_grid_home) }
    default : {}
  }
  $opatch_upgrade_params = {
    'oracle_home'               => $grid_home,
    'patch_file'                => $patch_file,
    'csi_number'                => $csi_number,
    'support_id'                => $support_id,
    'opversion'                 => $opversion,
    'user'                      => $user,
    'group'                     => $group,
    'download_dir'              => $download_dir,
    'puppet_download_mnt_point' => $puppet_download_mnt_point,
  }
  $opatch_upgrade = run_plan('ora_install::opatch_upgrade',      $dbserver, $opatch_upgrade_params)
  case $opatch_upgrade {
    'Error'   : { fail_plan($opatch_upgrade) }
    default : {}
  }
  $patch_home_params = {
    'install_group' => $group_install,
    'os_user'       => $user,
    'source'        => $source,
    'patch_list'    => $patch_list,
    'tmp_dir'       => $temp_dir,
  }
  $patch_home = run_plan('ora_install::patch_home',   $dbserver, $patch_home_params)
  case $patch_home {
    'Error'   : { fail_plan($patch_home) }
    default : {}
  }
}
