#
# See the file "LICENSE" for the full license governing this code.
#
#++--++
#
# @summary This plan will install a new ORACLE_HOME.
#
# @example
#   bolt plan run ora_install::install_oracle_home -t localhost version=18.0.0.0 file=LINUX.X64_180000_db_home oracle_base=/u01/app/oracle/admin oracle_home=/u01/app/oracle/product/18.0.0.0/db_home2 ora_inventory_dir=/u01/app puppet_download_mnt_point=/nfs_share/software
#
# @param [TargetSpec] dbserver
#   The target for this plan.
#
# @param [Ora_install::Version] version
#   The version that is installed in the used Oracle home.
#
# @param [Stdlib::Absolutepath] oracle_base
#   A directory to use as Oracle base directory.
#
# @param [Stdlib::Absolutepath] oracle_home
#   A directory to be used as Oracle home directory for this software.
#
# @param [String[1]] file
#   The source file to use for the ORACLE_HOME installation.
#
# @param [String[1]] user
#   The user used for the specified installation.
#
# @param [String[1]] group
#   The os group to use for these Oracle puppet definitions.
#
# @param [String[1]] group_install
#   The os group to use for installation.
#
# @param [String[1]] group_oper
#   The OS group to allow operator rights.
#
# @param [Enum['SE2', 'SE', 'EE', 'SEONE']] database_type
#   Selects the type of database you want to install.
#
# @param [Stdlib::Absolutepath] download_dir
#   The directory where the Puppet software puts all downloaded files.
#
# @param [Stdlib::Absolutepath] temp_dir
#   Directory to use for temporary files.
#
# @param [Optional[Array[Stdlib::Absolutepath]]] dirs
#   Directories that should be created in this plan. For example the parent directory (tree) of the GRID_HOME.
#
# @param [Optional[String[1]]] cluster_nodes
#   A comma seperated list of the nodes running in the RAC cluster.
#
# @param [Optional[Stdlib::Absolutepath]] ora_inventory_dir
#   The directory that contains the oracle inventory.
#
# @param [Optional[String[1]]] puppet_download_mnt_point
#   The base path of all remote files for the defined type or class.
#
# @param [Optional[Enum['json','pretty']]] format
#   Specify the format of the output, human readable (pretty) or not (json).
#
#--++--
plan ora_install::install_oracle_home(
  TargetSpec                       $dbserver,
  Ora_install::Version             $version                   = undef,
  Stdlib::Absolutepath             $oracle_base               = undef,
  Stdlib::Absolutepath             $oracle_home               = undef,
  String[1]                        $file                      = undef,
  String[1]                        $user                      = 'oracle',
  String[1]                        $group                     = 'dba',
  String[1]                        $group_install             = 'oinstall',
  String[1]                        $group_oper                = 'oper',
  Enum['SE2', 'SE', 'EE', 'SEONE'] $database_type             = 'EE',
  Stdlib::Absolutepath             $download_dir              = '/install',
  Stdlib::Absolutepath             $temp_dir                  = '/tmp',
  Optional[Array[Stdlib::Absolutepath]]
  $dirs                      = [],
  Optional[String[1]]              $cluster_nodes             = undef,
  Optional[Stdlib::Absolutepath]   $ora_inventory_dir         = '/u01/app',
  Optional[String[1]]              $puppet_download_mnt_point = 'puppet:///modules/software',
  Optional[Enum['json','pretty']]  $format                    = 'pretty',
) {
  if get_targets($dbserver).length > 1 {
    fail_plan("${dbserver} did not resolve to a single target")
  }

  $dbserver.apply_prep
  $result = apply($dbserver, { required_modules => ['easy_type', 'ora_install'], '_catch_errors' => true }) {
    $dirs.each |$dir| {
      unless defined(File[$dir]) {
        case $facts['os']['family'] {
          'windows': {
            file { $dir:
              ensure => directory,
              owner  => $user,
            }
          }
          default: {
            file { $dir:
              ensure  => directory,
              owner   => $user,
              group   => $group_install,
              seltype => 'default_t',
              mode    => '0770',
            }
          }
        }
      }
    }

    ora_install::installdb { 'install_oracle_home':
      version                   => $version,
      file                      => $file,
      database_type             => $database_type,
      oracle_base               => $oracle_base,
      oracle_home               => $oracle_home,
      puppet_download_mnt_point => $puppet_download_mnt_point,
      bash_profile              => false,
      group                     => $group,
      group_install             => $group_install,
      group_oper                => $group_oper,
      user                      => $user,
      download_dir              => $download_dir,
      temp_dir                  => $temp_dir,
      cluster_nodes             => $cluster_nodes,
      ora_inventory_dir         => $ora_inventory_dir,
      require                   => case $dirs.empty {
        true:    { undef }
        default: { File[$dirs] }
      },
    }
  }
  $target_result = $result.first()
  if $target_result.error {
    fail_plan($target_result.error())
  } else {
    $target_result.report['logs'].each |$log| {
      out::message("${log['source']}: ${log['message']}")
    }
  }
}
