#
# See the file "LICENSE" for the full license governing this code.
#
#++--++
#
# @summary This plan will install a new ORACLE_HOME and apply the specified patches to it.
#
# @example
#   bolt plan run ora_install::install_oracle_home_and_patch -t localhost version=18.0.0.0 file=LINUX.X64_180000_db_home oracle_base=/u01/app/oracle/admin oracle_home=/u01/app/oracle/product/18.0.0.0/db_home2 ora_inventory_dir=/u01/app puppet_download_mnt_point=/nfs_share/software patch_file=p6880880_190000_Linux-x86-64.zip opversion=12.2.0.1.17 patch_list='{"/u01/app/oracle/product/18.0.0.0/db_home2:30116795": {"source": "/nfs_share/software/p30116795_180000_Linux-x86-64.zip", "sub_patches": ["30112122","30113775"]}}'
#
# @param [TargetSpec] dbserver
#   The target for this plan.
#
# @param [Ora_install::Version] version
#   The version that is installed in the used Oracle home.
#
# @param [Stdlib::Absolutepath] oracle_base
#   A directory to use as Oracle base directory.
#
# @param [Stdlib::Absolutepath] oracle_home
#   A directory to be used as Oracle home directory for this software.
#
# @param [String[1]] file
#   The source file to use for the ORACLE_HOME installation.
#
# @param [String[1]] user
#   The user used for the specified installation.
#
# @param [String[1]] group
#   The os group to use for these Oracle puppet definitions.
#
# @param [String[1]] group_install
#   The os group to use for installation.
#
# @param [String[1]] group_oper
#   The OS group to allow operator rights.
#
# @param [Enum['SE2', 'SE', 'EE', 'SEONE']] database_type
#   Selects the type of database you want to install.
#
# @param [Stdlib::Absolutepath] download_dir
#   The directory where the Puppet software puts all downloaded files.
#
# @param [Stdlib::Absolutepath] temp_dir
#   Directory to use for temporary files.
#
# @param [Optional[Array[Stdlib::Absolutepath]]] dirs
#   Directories that should be created in this plan. For example the parent directory (tree) of the GRID_HOME.
#
# @param [Optional[String[1]]] cluster_nodes
#   A comma seperated list of the nodes running in the RAC cluster.
#
# @param [Optional[Stdlib::Absolutepath]] ora_inventory_dir
#   The directory that contains the oracle inventory.
#
# @param [Optional[String[1]]] puppet_download_mnt_point
#   The base path of all remote files for the defined type or class.
#
# @param [String[1]] patch_file
#   The zip file containing the Opatch utility.
#
# @param [Optional[Integer]] csi_number
#   The Customer Service Identification number.
#
# @param [Optional[String[1]]] support_id
#   Your support ID.
#
# @param [String[1]] opversion
#   The version of Opatch you want to install.
#
# @param [Optional[String[1]]] source
#   The source of the patch. Can be either an extracted folder or the raw zip file.
#
# @param [Hash] patch_list
#   List of patches to apply.
# @option patch_list [String[1]] source
#   Full path to the zipfile that contains the patch
# @option patch_list [Array] sub_patches
#   List of patch numbers that are part of the patchset
#
# @param [Optional[Enum['json','pretty']]] format
#   Specify the format of the output, human readable (pretty) or not (json).
#
plan ora_install::install_oracle_home_and_patch(
  TargetSpec                       $dbserver,
  Ora_install::Version             $version                   = undef,
  Stdlib::Absolutepath             $oracle_base               = undef,
  Stdlib::Absolutepath             $oracle_home               = undef,
  String[1]                        $file                      = undef,
  String[1]                        $user                      = 'oracle',
  String[1]                        $group                     = 'dba',
  String[1]                        $group_install             = 'oinstall',
  String[1]                        $group_oper                = 'oper',
  Enum['SE2', 'SE', 'EE', 'SEONE'] $database_type             = 'EE',
  Stdlib::Absolutepath             $download_dir              = '/install',
  Stdlib::Absolutepath             $temp_dir                  = '/tmp',
  Optional[Array[Stdlib::Absolutepath]]
  $dirs                      = [],
  Optional[String[1]]              $cluster_nodes             = undef,
  Optional[Stdlib::Absolutepath]   $ora_inventory_dir         = '/u01/app',
  Optional[String[1]]              $puppet_download_mnt_point = 'puppet:///modules/software',
  # opatch_upgrade
  String[1]                        $patch_file                = undef,
  Optional[Integer]                $csi_number                = undef,
  Optional[String[1]]              $support_id                = undef,
  String[1]                        $opversion                 = undef,
  # patch_oracle_home
  Optional[String[1]]              $source                    = undef,
  Hash                             $patch_list                = {},
  Optional[Enum['json','pretty']]  $format                    = 'pretty',
) {
  if get_targets($dbserver).length > 1 {
    fail_plan("${dbserver} did not resolve to a single target")
  }
  $install_params = {
    'version'                   => $version,
    'oracle_base'               => $oracle_base,
    'oracle_home'               => $oracle_home,
    'file'                      => $file,
    'user'                      => $user,
    'group'                     => $group,
    'group_install'             => $group_install,
    'group_oper'                => $group_oper,
    'database_type'             => $database_type,
    'download_dir'              => $download_dir,
    'temp_dir'                  => $temp_dir,
    'dirs'                      => $dirs,
    'cluster_nodes'             => $cluster_nodes,
    'ora_inventory_dir'         => $ora_inventory_dir,
    'puppet_download_mnt_point' => $puppet_download_mnt_point,
    'format'                    => $format,
  }
  $install_oracle_home = run_plan('ora_install::install_oracle_home', $dbserver, $install_params)
  case $install_oracle_home {
    'Error'  : { fail_plan($install_oracle_home) }
    default : {}
  }
  $opatch_upgrade_params = {
    'oracle_home'               => $oracle_home,
    'patch_file'                => $patch_file,
    'csi_number'                => $csi_number,
    'support_id'                => $support_id,
    'opversion'                 => $opversion,
    'user'                      => $user,
    'group'                     => $group,
    'download_dir'              => $download_dir,
    'puppet_download_mnt_point' => $puppet_download_mnt_point,
  }
  $opatch_upgrade = run_plan('ora_install::opatch_upgrade',      $dbserver, $opatch_upgrade_params)
  case $opatch_upgrade {
    'Error'   : { fail_plan($opatch_upgrade) }
    default : {}
  }
  $patch_home_params = {
    'install_group' => $group_install,
    'os_user'       => $user,
    'source'        => $source,
    'patch_list'    => $patch_list,
    'tmp_dir'       => $temp_dir,
  }
  $patch_home = run_plan('ora_install::patch_home',   $dbserver, $patch_home_params)
  case $patch_home {
    'Error'   : { fail_plan($patch_home) }
    default : {}
  }
}
