#
# See the file "LICENSE" for the full license governing this code.
#
#++--++
#
# @summary This plan will install a new version of OPatch in specified ORACLE_HOME.
#
# @example
#  bolt plan run ora_install::opatch_upgrade -t localhost oracle_home=/u01/app/oracle/product/18.0.0.0/db_home2 patch_file=p6880880_190000_Linux-x86-64.zip opversion=12.2.0.1.17 puppet_download_mnt_point=/nfs_share/software
#
# @param [TargetSpec] dbserver
#   The target for this plan.
#
# @param [Stdlib::Absolutepath] oracle_home
#   A directory to be used as Oracle home directory for this software.
#
# @param [String[1]] patch_file
#   The zip file containing the Opatch utility.
#
# @param [Optional[Integer]] csi_number
#   The Customer Service Identification number.
#
# @param [Optional[String[1]]] support_id
#   Your support ID.
#
# @param [String[1]] opversion
#   The version of Opatch you want to install.
#
# @param [String[1]] user
#   The user used for the specified installation.
#
# @param [String[1]] group
#   The os group to use for these Oracle puppet definitions.
#
# @param [Stdlib::Absolutepath] download_dir
#   The directory where the Puppet software puts all downloaded files.
#
# @param [Optional[String[1]]] puppet_download_mnt_point
#   The base path of all remote files for the defined type or class.
#
# @param [Optional[Enum['json','pretty']]] format
#   Specify the format of the output, human readable (pretty) or not (json).
#
plan ora_install::opatch_upgrade(
  TargetSpec                       $dbserver,
  Stdlib::Absolutepath             $oracle_home               = undef,
  String[1]                        $patch_file                = undef,
  Optional[Integer]                $csi_number                = undef,
  Optional[String[1]]              $support_id                = undef,
  String[1]                        $opversion                 = undef,
  String[1]                        $user                      = 'oracle',
  String[1]                        $group                     = 'dba',
  Stdlib::Absolutepath             $download_dir              = '/install',
  Optional[String[1]]              $puppet_download_mnt_point = undef,
  Optional[Enum['json','pretty']]  $format                    = 'pretty',
) {
  if get_targets($dbserver).length > 1 {
    fail_plan("${dbserver} did not resolve to a single target")
  }

  $dbserver.apply_prep

  $result = apply($dbserver, { required_modules => ['easy_type', 'ora_install'], '_catch_errors' => true }) {
    ora_install::opatchupgrade { "opatch_upgrade_${oracle_home}_${opversion}":
      oracle_home               => $oracle_home,
      patch_file                => $patch_file,
      csi_number                => $csi_number,
      support_id                => $support_id,
      opversion                 => $opversion,
      user                      => $user,
      group                     => $group,
      download_dir              => $download_dir,
      puppet_download_mnt_point => $puppet_download_mnt_point,
    }
  }
  $target_result = $result.first()
  if $target_result.error {
    fail_plan($target_result.error())
  } else {
    $target_result.report['logs'].each |$log| {
      out::message("${log['source']}: ${log['message']}")
    }
  }
}
