#
# See the file "LICENSE" for the full license governing this code.
#
#++--++
#
# @summary This plan will install patches into specified home.
#
# @example
#   bolt plan run ora_install::patch_home -t localhost patch_list='{"/u01/app/oracle/product/18.0.0.0/db_home2:30116795": {"source": "/nfs_share/software/p30116795_180000_Linux-x86-64.zip", "sub_patches": ["30112122","30113775"]}}'
#
# @param [TargetSpec] dbserver
#   The target for this plan.
#
# @param [String[1]] install_group
#   The os group to use for installation.
#
# @param [String[1]] os_user
#   The user used for the specified installation.
#
# @param [Optional[String[1]]] source
#   The source of the patch. Can be either an extracted folder or the raw zip file.
#
# @param [Hash] patch_list
#   List of patches to apply.
# @option patch_list [String[1]] source
#   Full path to the zipfile that contains the patch
# @option patch_list [Array] sub_patches
#   List of patch numbers that are part of the patchset
#
# @param [Stdlib::Absolutepath] tmp_dir
#   Directory to use for temporary files.
#
# @param [Optional[Enum['json','pretty']]] format
#   Specify the format of the output, human readable (pretty) or not (json).
#
#--++--
plan ora_install::patch_home(
  TargetSpec                      $dbserver,
  String[1]                       $install_group = 'oinstall',
  String[1]                       $os_user       = 'oracle',
  Optional[String[1]]             $source        = undef,
  Hash                            $patch_list    = {},
  Stdlib::Absolutepath            $tmp_dir       = '/install',
  Optional[Enum['json','pretty']] $format        = 'pretty',
) {
  if get_targets($dbserver).length > 1 {
    fail_plan("${dbserver} did not resolve to a single target")
  }

  $dbserver.apply_prep

  $result = apply($dbserver, { required_modules => ['easy_type', 'ora_install'], '_catch_errors' => true }) {
    $converted_patch_list = $patch_list.map | $i, $j | { $j['sub_patches'].map | $x | { "${i.split(':')[0]}:${x}" } }.flatten

    if ora_install::ora_patches_installed($converted_patch_list) {
      notify { 'All DB patches already installed. Skipping patches.': }
    } else {
      # Check that there is no db running in the oracle_home(s) in the patch_list
      $oracle_homes = $patch_list.keys.map | $k | { split($k,':')[0] }.unique
      $oracle_homes.each | $oracle_home | {
        if $facts['ora_install_homes']['running_processes'][$oracle_home]['sids'].length > 0 {
          $databases = join($facts['ora_install_homes']['running_processes'][$oracle_home]['sids'].keys,',')
          fail("Database(s) ${databases} still running from oracle home ${oracle_home}, stop it before patching.")
        }
        if $facts['ora_install_homes']['running_processes'][$oracle_home]['listeners'].length > 0 {
          $listeners = join($facts['ora_install_homes']['running_processes'][$oracle_home]['listeners'].keys,',')
          fail("Listeners(s) ${listeners} still running from oracle home ${oracle_home}, stop it before patching.")
        }
      }
      $defaults = {
        ensure  => 'present',
        tmp_dir => "${tmp_dir}/patches",               # always use subdir, the whole directory will be removed when done
      }
      ensure_resources('ora_opatch', $patch_list, $defaults)
    }
  }
  $target_result = $result.first()
  if $target_result.error {
    fail_plan($target_result.error())
  } else {
    $target_result.report['logs'].each |$log| {
      out::message("${log['source']}: ${log['message']}")
    }
  }
}
