#
# See the file "LICENSE" for the full license governing this code.
#
#++--++
#
# @summary This plan will switch an Grid Infrastructure installation to an already existing other GRID_HOME.
#
# @example
#   bolt plan run ora_install::switch_home_asm -t localhost new_home=/u01/app/grid/product/19.0.0.0/grid_home2
#
# @param [TargetSpec] dbserver
#   The target for this plan.
#
# @param [Stdlib::Absolutepath] new_home
#   The ORACLE_HOME where the database should be switched to.
#
# @param [Optional[Enum['json','pretty']]] format
#   Specify the format of the output, human readable (pretty) or not (json).
#
# @param [Optional[Optional[Enum['debug', 'info', 'notice', 'warning', 'err', 'alert', 'emerg', 'crit']]]] verbose
#   The level of verbosity to use.
#
#--++--
plan ora_install::switch_home_asm(
  #lint:ignore:strict_indent
  TargetSpec                      $dbserver,
  Stdlib::Absolutepath            $new_home,
  Optional[Enum['json','pretty']] $format  = 'pretty',
  Optional[Optional[Enum['debug', 'info', 'notice', 'warning', 'err', 'alert', 'emerg', 'crit']]]
                                  $verbose = 'info',
  #lint:endignore:strict_indent
) {
  if get_targets($dbserver).length > 1 {
    fail_plan("${dbserver} did not resolve to a single target")
  }
  $defaults = { format => $format, verbose => $verbose, '_catch_errors' => true }
  $tasks = [
    { 'ora_install::agent_action'       => { action => 'disable', comment => "Switching grid home to ${new_home}" } },
    { 'ora_install::grid_action'        => { action => 'stop' } },
    { 'ora_install::switch_grid_home'   => { new_home => $new_home } },
    { 'ora_install::grid_action'        => { action => 'start' } },
    { 'ora_install::agent_action'       => { action => 'enable' } },
  ]

  $tasks.each |$ind, $task| {
    $task.each |$key, $params| {
      $all_params = $params + $defaults
      $output = run_task($key, $dbserver, $all_params)
      $output.each |$result| {
        if $result.ok {
          if is_hash($result['message']) {
            $message = $result['message']['message']
          } else {
            $message = $result['message']
          }
          out::message("Task ${key} on ${dbserver} returned a value:\n${message}")
        } else {
          out::message("Task ${key} on ${dbserver} failed:\n${result['message']}")
          fail_plan("Task ${key} on ${dbserver} failed:\n${result['_error']['msg']}", $result['_error']['kind'], { 'failedtargets' => $dbserver })
        }
      }
    }
  }
}
