#!/opt/puppetlabs/puppet/bin/ruby
#
#
# See the file "LICENSE" for the full license governing this code.
#
#
# TODO: Switch grid home
require 'json'
require 'open3'
require 'puppet'
require 'yaml'
require 'fileutils'
require 'digest'
begin
  Puppet.initialize_settings
rescue Puppet::DevError
  # When run in Rspec, this is called multiple times
end

class SwitchOracleHome

  def self.run
    instance = self.new
    instance.run
  end

  def run()
    @starttime = Time.now.iso8601
    begin
      fetch_input()
      execute()
      output_message = @message.join("\n")
      output(0, output_message)
    rescue => e
      error_output(1, e.message, "enterprisemodules-ora_install/task-error")
    end
  end

  def fetch_input
    params        = JSON.parse(STDIN.read)
    #
    # Use loadpath specfied in the params
    #
    $LOAD_PATH << "#{params['_installdir']}/easy_type/lib"
    $LOAD_PATH << "#{params['_installdir']}/ora_install/lib"
    require 'easy_type'
    require 'puppet_x/enterprisemodules/ora_install/tasks'
    extend ::PuppetX::EnterpriseModules::OraInstall::Tasks

    #
    # Fetch verbose level
    #
    @verbose = params.fetch('verbose') { 'info' }
    Puppet::Util::Log.level = @verbose

    @sid            = params.fetch('sid')
    @asm_instance   = @sid =~ /^\+ASM/
    @cluster        = @sid =~ /^\+ASM[1-9]/
    @oracle_homes   = Facter.value('ora_install_homes')
    @current_home   = home_for_sid(@sid)
    @new_home       = params.fetch('new_home')
    @format         = params.fetch('format') { 'pretty' }
    error_output(1, "new Oracle home #{@new_home} does not exist.", 'enterprisemodules-ora_install/parameter-error') unless @oracle_homes['product_version'].keys.include?(@new_home)
    error_output(1, "Current Oracle home and new Oracle home cannot be same.", 'enterprisemodules-ora_install/parameter-error') if @new_home == @current_home
    @message = []
  end

  def execute
    copy_network_files
    if @asm_instance
      if @cluster
        copy_cluster_files
      else
        copy_asm_files
      end
    else
      copy_files
    end
    update_config if running_on_asm?
    update_oratab
    switch_grid_home if @asm_instance
    update_ora_settings if ora_settings?
  end

  def hostname
    Facter.value('hostname')
  end

  def copy_files
    log_msg 'Copying files...'
    files = [
      "ab_#{@sid}.dat",
      "hc_#{@sid}.dat",
      "lk#{@sid}",
      "orapw#{@sid}",
      "spfile#{@sid}.ora",
      "init#{@sid}.ora",
      "orapw#{@sid}",
      "dr1#{@sid}.dat",
      "dr2#{@sid}.dat",
      "oranfstab"
    ]
    copy(files, "#{@current_home}/dbs", "#{@new_home}/dbs")
  end

  def copy_network_files
    log_msg 'Copying network files...'
    network_files = [
      'sqlnet.ora',
      'tnsnames.ora',
      'listener.ora'
    ]
    copy(network_files, "#{@current_home}/network/admin", "#{@new_home}/network/admin")
  end

  def copy_asm_files
    log_msg 'Copying asm files...'
    olr_files = [
      "#{hostname}.olr"
    ]
    copy(olr_files, "#{@current_home}/cdata/localhost", "#{@new_home}/cdata/localhost")

    file_name = '/etc/oracle/ocr.loc'
    contents = File.read(file_name)
    new_contents = contents.gsub(@current_home, @new_home)
    File.open(file_name, 'w') { |file| file.write(new_contents) }
    log_msg "Contents of file #{file_name} updated with new oracle_home"
  end

  def copy_cluster_files
    log_msg 'Copying cluster files...'
    olr_files = [
      "#{hostname}.olr"
    ]
    copy(olr_files, "#{@current_home}/cdata", "#{@new_home}/cdata")
    wallet_files = [
      'cwallet.sso'
    ]
    copy(wallet_files, "#{@current_home}/gpnp/#{hostname}/wallets/prdr", "#{@new_home}/gpnp/#{hostname}/wallets/prdr")
  end

  def update_oratab
    log_msg 'Updating home in oratab file.'
    file = '/etc/oratab'
    FileUtils.cp(file, "#{file}.before_switch")
    contents = File.read(file)
    contents = contents.gsub(/#{Regexp.quote(@sid)}\:#{Regexp.quote(@current_home)}\:/,"#{@sid}:#{@new_home}:")
    File.write(file, contents)
  end

  def update_config
    log_msg "Changing config for #{@sid}"
    exec("export ORACLE_HOME=#{@new_home};export ORACLE_SID=#{@sid};#{@new_home}/bin/srvctl modify database -d #{@sid} -oraclehome #{@new_home}", :uid => os_user)
  end

  def update_ora_settings
    log_msg 'Updating home in ora_settings file.'
    data = YAML.load(File.read('/etc/ora_setting.yaml'))
    data[@sid]['oracle_home'] = @new_home
    File.write('/etc/ora_setting.yaml', data.to_yaml)
  end

  private

    def exec( command, options =  {} )
      options = {:failonfail => true, :combine => true}.merge(options)
      output = Puppet::Util::Execution.execute(command, options).to_s
      Puppet.debug output
      output
    end

    def ora_settings?
      File.exist?('/etc/ora_setting.yaml')
    end

    def home_for_sid(sid)
      selected_home_info = @oracle_homes['defined_sids'].select {|d| @oracle_homes['defined_sids'].fetch(d){[]}.include?(sid)}
      fail "No home found for sid #{sid}" if selected_home_info.empty?
      selected_home = selected_home_info.keys.first
      # fail "Sid #{sid} must be running for switch." unless selected_home_info[selected_home].fetch('running_sids') {[]}.include?(sid)
      selected_home
    end

    def asm_home
      @oracle_homes['defined_sids'].select {|d| @oracle_homes['defined_sids'].fetch(d).grep(/^\+ASM/).any?}.keys.first
    end

    # We are a databse sid but running on top of ASM.
    def running_on_asm?
      !asm_home.nil? && @sid !~ /^\+ASM/
    end

    def asm_sid
      @oracle_homes['defined_sids'][asm_home].first
    end

    def os_user
      uid = File.stat("#{@current_home}/bin/oracle").uid
      Etc.getpwuid(uid).name
    end

    def os_group
      gid = File.stat(@current_home).gid
      Etc.getgrgid(gid).name
    end

    def unsuccessful?
      @output.scan(@succes_output).empty?
    end

    def sql(command, home)
      if @asm_instance
        command     = "sqlplus /nolog <<-EOF
connect / as sysasm
#{command}
EOF"
      else
        command     = "sqlplus /nolog <<-EOF
connect / as sysdba
#{command}
EOF"
      end

      exec("export ORACLE_HOME=#{home};export PATH=#{home}/bin:$PATH;export ORACLE_SID=#{@sid};export LD_LIBRARY_PATH=#{home}/lib;#{command}", :uid => os_user)
    end
end

SwitchOracleHome.run if __FILE__ == $0
